@@ -1,3 +1,18 @@
+#include "../machine.h"
+
+#if !(defined HAVE_FINITE) && (defined hppa)
+
+#include <math.h>
+
+int finite(double x)
+{
+  if (isfinite(x)==0) return 0;
+  return 1;
+}
+
+#else
+#if !(defined HAVE_FINITE) || (defined __MINGW32__) || (defined __ABSC__) 
+
 typedef unsigned int __uint32_t;
 typedef union 
 {
@@ -23,3 +38,5 @@ double x;
   hx = gh_u.parts.msw;
   return  (int)((__uint32_t)((hx&0x7fffffff)-0x7ff00000)>>31);
 }
+#endif
+#endif
