@@ -20,7 +20,7 @@ c Serge Steer INRIA 1996
 c!
 c     Copyright INRIA
       integer ierr
-      double precision vr,vi,pr,pi,sr,si,rr,ri
+      double precision vr,vi,pr,pi,sr,si,rr,ri,infinity
 c
       ierr=0
 c     
