@@ -1,36 +1,42 @@
 /* Copyright Jean-Philippe Chancelier 
    ENPC */
-
+#include <string.h>
 #include "../../routines/stack-c.h"
 
 /** external functions to be called through this interface **/
+#ifdef __STDC__
+void  sciprint(char *fmt,...);
+#else
+/*VARARGS0*/
+void sciprint(char *, ...);
+#endif  
+ 
+extern int C2F(cdfbet) __PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdfnbn) __PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdffnc) __PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdfbin) __PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdfchi) __PARAMS((int *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdft) __PARAMS((int *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdfpoi) __PARAMS((int *,double *,double *,double *,double *, int *,double *));
+extern int C2F(cdff) __PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
+extern int C2F(cdfgam) __PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
+extern int C2F(cdff) __PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
+extern int C2F(cdfchn) __PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
+extern int C2F(cdfnor) __PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
 
-extern int C2F(cdfbet) _PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdfnbn) _PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdffnc) _PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdfbin) _PARAMS((int *,double *,double *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdfchi) _PARAMS((int *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdft) _PARAMS((int *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdfpoi) _PARAMS((int *,double *,double *,double *,double *, int *,double *));
-extern int C2F(cdff) _PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
-extern int C2F(cdfgam) _PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
-extern int C2F(cdff) _PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
-extern int C2F(cdfchn) _PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
-extern int C2F(cdfnor) _PARAMS((int *,double *,double *,double*,double *,double *, int *,double *));
+static int CdfBase __PARAMS((char *fname,int inarg,int oarg,int *callpos,char *option,char *errnames,int which,int (*fonc)(),void (*foncErr)()));
 
-static int CdfBase _PARAMS((char *fname,int inarg,int oarg,int *callpos,char *option,char *errnames,int which,int (*fonc)(),void (*foncErr)()));
-
-static void cdfbetErr _PARAMS((int status,double bound));
-static void cdfbinErr _PARAMS((int status,double bound));
-static void cdfchiErr _PARAMS((int status,double bound));
-static void cdffErr _PARAMS((int status,double bound));
-static void cdffncErr _PARAMS((int status,double bound));
-static void cdfgamErr _PARAMS((int status,double bound));
-static void cdfnbnErr _PARAMS((int status,double bound));
-static void cdfnorErr _PARAMS((int status,double bound));
-static void cdfpoiErr _PARAMS((int status,double bound));
-static void cdftErr _PARAMS((int status,double bound));
-static void cdfchnErr _PARAMS((int status,double bound));
+static void cdfbetErr __PARAMS((int status,double bound));
+static void cdfbinErr __PARAMS((int status,double bound));
+static void cdfchiErr __PARAMS((int status,double bound));
+static void cdffErr __PARAMS((int status,double bound));
+static void cdffncErr __PARAMS((int status,double bound));
+static void cdfgamErr __PARAMS((int status,double bound));
+static void cdfnbnErr __PARAMS((int status,double bound));
+static void cdfnorErr __PARAMS((int status,double bound));
+static void cdfpoiErr __PARAMS((int status,double bound));
+static void cdftErr __PARAMS((int status,double bound));
+static void cdfchnErr __PARAMS((int status,double bound));
 
 /**************************************************
  *  hand written interface 
@@ -41,8 +47,7 @@ static void cdfchnErr _PARAMS((int status,double bound));
  *                         BETa Distribution
  ***********************************************************************/
 
-int cdfbetI(fname)
-  char* fname;
+int cdfbetI(char* fname,unsigned long l)
 { 
   int minrhs = 5,maxrhs = 6,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -75,8 +80,7 @@ int cdfbetI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
       return 0;
     }
   return 0;
@@ -89,13 +93,13 @@ static void cdfbetErr(status,bound)
   static char param[]="-PQXYAB";
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
-    case 4 : sciprint(" X + Y .ne. 1 \r\n");break;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
+    case 4 : Scierror(999," X + Y .ne. 1 \r\n");break;
     default : 
-      sciprint("input parameter %c is out of range \r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -110,8 +114,7 @@ static void cdfbetErr(status,bound)
  ***********************************************************************/
 
 
-int cdfbinI(fname)
-  char* fname;
+int cdfbinI(char* fname,unsigned long l)
 { 
   int minrhs = 5,maxrhs = 6,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -144,8 +147,7 @@ int cdfbinI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -157,13 +159,13 @@ static void cdfbinErr(status,bound)
   static char *param[7]={"Which", "P","Q","Pr","Ompr","S","Xn"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
-    case 4 : sciprint(" Pr + Ompr .ne. 1 \r\n");break;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
+    case 4 : Scierror(999," Pr + Ompr .ne. 1 \r\n");break;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -178,8 +180,7 @@ static void cdfbinErr(status,bound)
  ***********************************************************************/
 
 
-int cdfchiI(fname)
-  char* fname;
+int cdfchiI(char* fname,unsigned long l)
 { 
   int minrhs = 3,maxrhs = 4,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -206,8 +207,7 @@ int cdfchiI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -219,12 +219,13 @@ static void cdfchiErr(status,bound)
   static char *param[7]={"X", "P","Q","X","Df"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
+    case 10 : Scierror(999,"cdfchi: Error in  cumgam\r\n");break;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -237,8 +238,7 @@ static void cdfchiErr(status,bound)
  ***********************************************************************/
 
 
-int cdffI(fname)
-  char* fname;
+int cdffI(char* fname,unsigned long l)
 { 
   int minrhs = 4,maxrhs = 5,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -271,8 +271,7 @@ int cdffI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -285,12 +284,12 @@ static void cdffErr(status,bound)
   static char *param[7]={"X", "P","Q","F","Dfn","Dfd"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -304,8 +303,7 @@ static void cdffErr(status,bound)
  ***********************************************************************/
 
 
-int cdffncI(fname)
-  char* fname;
+int cdffncI(char* fname,unsigned long l)
 { 
   int minrhs = 5,maxrhs = 6,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -344,8 +342,7 @@ int cdffncI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -358,12 +355,12 @@ static void cdffncErr(status,bound)
   static char *param[7]={"X", "P","Q","F","Dfn","Dfd","Pnonc"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -377,8 +374,8 @@ static void cdffncErr(status,bound)
  ***********************************************************************/
 
 
-int cdfgamI(fname)
-  char* fname;
+int cdfgamI(char* fname,unsigned long l)
+
 { 
   int minrhs = 4,maxrhs = 5,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -411,8 +408,7 @@ int cdfgamI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -425,13 +421,13 @@ static void cdfgamErr(status,bound)
   static char *param[7]={"X", "P","Q","X","Shape","Scale"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
-    case 10 : sciprint(" cannot compute the answer \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
+    case 10 : Scierror(999," cannot compute the answer \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -444,8 +440,8 @@ static void cdfgamErr(status,bound)
  *               Negative BiNomial distribution
  ***********************************************************************/
 
-int cdfnbnI(fname)
-  char* fname;
+int cdfnbnI(char* fname,unsigned long l)
+
 { 
   int minrhs = 5,maxrhs = 6,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -479,8 +475,7 @@ int cdfnbnI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -492,13 +487,13 @@ static void cdfnbnErr(status,bound)
   static char param[]="-PQXYAB";
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
-    case 4 : sciprint(" Pr + Ompr .ne. 1 \r\n");break;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
+    case 4 : Scierror(999," Pr + Ompr .ne. 1 \r\n");break;
     default : 
-      sciprint("input parameter %c is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -512,8 +507,8 @@ static void cdfnbnErr(status,bound)
  ***********************************************************************/
 
 
-int cdfnorI(fname)
-  char* fname;
+int cdfnorI(char* fname,unsigned long l)
+  
 { 
   int minrhs = 4,maxrhs = 5,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -546,8 +541,7 @@ int cdfnorI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -560,13 +554,13 @@ static void cdfnorErr(status,bound)
   static char *param[7]={"X", "P","Q","X","Mean","Std"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
-    case 4 : sciprint(" Std must not be zero \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
+    case 4 : Scierror(999," Std must not be zero \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -578,8 +572,8 @@ static void cdfnorErr(status,bound)
  ***********************************************************************/
 
 
-int cdfpoiI(fname)
-  char* fname;
+int cdfpoiI(char* fname,unsigned long l)
+  
 { 
   int minrhs = 3,maxrhs = 4,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -606,8 +600,7 @@ int cdfpoiI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -619,12 +612,12 @@ static void cdfpoiErr(status,bound)
   static char *param[7]={"X", "P","Q","S","Xlam"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -637,8 +630,8 @@ static void cdfpoiErr(status,bound)
  ***********************************************************************/
 
 
-int cdftI(fname)
-  char* fname;
+int cdftI(char* fname,unsigned long l)
+  
 { 
   int minrhs = 3,maxrhs = 4,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -665,8 +658,7 @@ int cdftI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -678,12 +670,12 @@ static void cdftErr(status,bound)
   static char *param[7]={"X", "P","Q","T","Df"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -695,8 +687,7 @@ static void cdftErr(status,bound)
  ***********************************************************************/
 
 
-int cdfchnI(fname)
-  char* fname;
+int cdfchnI(char* fname,unsigned long l)
 { 
   int minrhs = 4,maxrhs = 5,minlhs=1,maxlhs=2,m1,n1,l1;
   Nbvars = 0;
@@ -729,8 +720,7 @@ int cdfchnI(fname)
     }
   else 
     {
-      sciprint("%s Wrong first argument %s\r\n",fname,cstk(l1));
-      Error(999);
+      Scierror(999,"%s: Wrong first argument %s\r\n",fname,cstk(l1));
     }      
   return 0;
 }
@@ -743,12 +733,12 @@ static void cdfchnErr(status,bound)
   static char *param[7]={"X", "P","Q","F","Dfn","Dfd"};
   switch ( status ) 
     {
-    case 1 : sciprint("answer appears to be lower than lowest search bound %f\r\n",bound);break;
-    case 2 : sciprint("answer appears to be higher than greatest search bound %f\r\n",bound);break;
-    case 3 : sciprint(" P + Q .ne. 1 \r\n");break ;
+    case 1 : Scierror(999,"answer appears to be lower than lowest search bound %f\r\n",bound);break;
+    case 2 : Scierror(999,"answer appears to be higher than greatest search bound %f\r\n",bound);break;
+    case 3 : Scierror(999," P + Q .ne. 1 \r\n");break ;
     default : 
-      sciprint("input parameter %s is out of range\r\n",param[-status-1]);
-      sciprint("      bound exceeded: %f \r\n",bound);
+      Scierror(999,"input parameter %c is out of range \r\n\tbound exceeded: %f\r\n",
+		    param[-status-1],bound);
     }
 }
 
@@ -765,13 +755,13 @@ static TabF Tab[]={
   {cdfnbnI, "cdfnbn"},
   {cdfnorI, "cdfnor"},
   {cdfpoiI, "cdfpoi"},
-  {cdftI, "cdft"},
+  {cdftI, "cdft"}
 };
 
 int C2F(dcd)()
 {
   Rhs = Max(0, Rhs);
-  (*(Tab[Fin-1].f))(Tab[Fin-1].name);
+  (*(Tab[Fin-1].f))(Tab[Fin-1].name,strlen(Tab[Fin-1].name));
   return 0;
 }
 
@@ -785,8 +775,8 @@ static int  CdfBase(fname,inarg,oarg,callpos,option,errnames,which,fonc,foncErr)
   double bound;
   if ( Rhs != inarg+1 ) 
     {
-      sciprint("%s Rhs must be %d for '%s' option'\r\n",fname,inarg+1,option);
-      Error(999); return 1;
+      Scierror(999,"%s: Rhs must be %d for '%s' option'\r\n",fname,inarg+1,option);
+      return 1;
     }
   for ( i = 0 ; i < inarg ; i++ )
     {
@@ -795,8 +785,8 @@ static int  CdfBase(fname,inarg,oarg,callpos,option,errnames,which,fonc,foncErr)
   for ( i = 1 ; i < inarg ; i++) 
     if ( m[i] != m[i-1] || n[i] != n[i-1]) 
       {
-	sciprint("%s %s must have same size\r\n",fname,errnames);
-	Error(999); return 1;
+	Scierror(999,"%s %s must have same size\r\n",fname,errnames);
+	return 1;
       } 
   for ( i = 0 ; i < oarg ; i++) 
     CreateVar(i+2+inarg,"d",&m[0],&n[0],&l[i+inarg]);
@@ -811,7 +801,7 @@ static int  CdfBase(fname,inarg,oarg,callpos,option,errnames,which,fonc,foncErr)
 		  &status,&bound);
 	  if (status != 0) 
 	    {
-	      (*foncErr)(status,bound); Error(999); return 1;
+	      (*foncErr)(status,bound); return 1;
 	    }
 	}
       break;
@@ -824,8 +814,8 @@ static int  CdfBase(fname,inarg,oarg,callpos,option,errnames,which,fonc,foncErr)
 		  &status,&bound);
 	  if (status != 0) 
 	    {
-	      /** sciprint("i=%d\r\n",i); **/
-	      (*foncErr)(status,bound); Error(999); return 1;
+	      /** Scierror(999,"i=%d\r\n",i); **/
+	      (*foncErr)(status,bound); return 1;
 	    }
 	}
       break;
@@ -837,7 +827,7 @@ static int  CdfBase(fname,inarg,oarg,callpos,option,errnames,which,fonc,foncErr)
 		  &status,&bound);
 	  if (status != 0) 
 	    {
-	      (*foncErr)(status,bound); Error(999); return 1;
+	      (*foncErr)(status,bound); return 1;
 	    }
 	}
       break;
