@@ -9,8 +9,9 @@
 /* MONO returns total intensity of r,g,b components */
 #define MONO(rd,gn,bl) (((rd)*11 + (gn)*16 + (bl)*5) >> 5)  /*.33R+ .5G+ .17B*/
 
-static void gdImageBrushApply _PARAMS((gdImagePtr im, int x, int y));
-static void gdImageTileApply _PARAMS((gdImagePtr im, int x, int y));
+static void gdImageBrushApply __PARAMS((gdImagePtr im, int x, int y));
+static void gdImageTileApply __PARAMS((gdImagePtr im, int x, int y));
+
 
 /*SG modified to add alu modes and clipping fiels */
 gdImagePtr gdImageCreate(sx, sy)
@@ -462,7 +463,7 @@ void gdImageThickLine(im, x1, y1, x2, y2, color, thick)
     c1 = c;
     c = gdBrushed;
   }
-  gdImageLine(imv,x1, y1, x2, y2, c);
+    gdImageLine(imv,x1, y1, x2, y2, c);
 
   if (thick > 1 && color >= 0) { /* deallocate square brush */
     gdImageDestroy(brush);
@@ -632,9 +633,11 @@ void gdImagePolyLine(im, X, Y, n, color, thick, close)
   }
   for (i=0;i<n-1;i++) 
     gdImageLine(imv,X[i],Y[i],X[i+1],Y[i+1],c);
+
     
   if (close) 
-    gdImageLine(imv,X[n-1],X[n-1],X[0],Y[0],c);
+    gdImageLine(imv,X[n-1],Y[n-1],X[0],Y[0],c);
+
 
   if (thick > 1 && color >= 0) { /* deallocate square brush */
     gdImageDestroy(brush);
@@ -652,124 +655,6 @@ void gdImagePolyLine(im, X, Y, n, color, thick, close)
     gdImageDestroy(imv);
   }
 }
-
-static void dashedSet();
-
-void gdImageDashedLine(im, x1, y1, x2, y2, color)
-     gdImagePtr im;
-     int x1;
-     int y1;
-     int x2;
-     int y2;
-     int color;
-{
-	int dx, dy, incr1, incr2, d, x, y, xend, yend, xdirflag, ydirflag;
-	int dashStep = 0;
-	int on = 1;
-	dx = abs(x2-x1);
-	dy = abs(y2-y1);
-	if (dy <= dx) {
-		d = 2*dy - dx;
-		incr1 = 2*dy;
-		incr2 = 2 * (dy - dx);
-		if (x1 > x2) {
-			x = x2;
-			y = y2;
-			ydirflag = (-1);
-			xend = x1;
-		} else {
-			x = x1;
-			y = y1;
-			ydirflag = 1;
-			xend = x2;
-		}
-		dashedSet(im, x, y, color, &on, &dashStep);
-		if (((y2 - y1) * ydirflag) > 0) {
-			while (x < xend) {
-				x++;
-				if (d <0) {
-					d+=incr1;
-				} else {
-					y++;
-					d+=incr2;
-				}
-				dashedSet(im, x, y, color, &on, &dashStep);
-			}
-		} else {
-			while (x < xend) {
-				x++;
-				if (d <0) {
-					d+=incr1;
-				} else {
-					y--;
-					d+=incr2;
-				}
-				dashedSet(im, x, y, color, &on, &dashStep);
-			}
-		}		
-	} else {
-		d = 2*dx - dy;
-		incr1 = 2*dx;
-		incr2 = 2 * (dx - dy);
-		if (y1 > y2) {
-			y = y2;
-			x = x2;
-			yend = y1;
-			xdirflag = (-1);
-		} else {
-			y = y1;
-			x = x1;
-			yend = y2;
-			xdirflag = 1;
-		}
-		dashedSet(im, x, y, color, &on, &dashStep);
-		if (((x2 - x1) * xdirflag) > 0) {
-			while (y < yend) {
-				y++;
-				if (d <0) {
-					d+=incr1;
-				} else {
-					x++;
-					d+=incr2;
-				}
-				dashedSet(im, x, y, color, &on, &dashStep);
-			}
-		} else {
-			while (y < yend) {
-				y++;
-				if (d <0) {
-					d+=incr1;
-				} else {
-					x--;
-					d+=incr2;
-				}
-				dashedSet(im, x, y, color, &on, &dashStep);
-			}
-		}
-	}
-}
-
-static void dashedSet(im, x, y, color, onP, dashStepP)
-     gdImagePtr im;
-     int x;
-     int y;
-     int color;
-     int *onP;
-     int *dashStepP;
-{
-	int dashStep = *dashStepP;
-	int on = *onP;
-	dashStep++;
-	if (dashStep == gdDashSize) {
-		dashStep = 0;
-		on = !on;
-	}
-	if (on) {
-		gdImageSetPixel(im, x, y, color);
-	}
-	*dashStepP = dashStep;
-	*onP = on;
-}
 	
 /*SG modified to add clipping region */
 int gdImageBoundsSafe(im, x, y)
@@ -808,6 +693,9 @@ int gdImageChar(im, f, x, y, c, color)
 	cx = 0;
 	cy = 0;
 	mx = x;
+	if (c == 32)  /* a q & d way to solve bug 1113 (bruno 7 dec 2004) */
+	  return((f->w-2)/3);
+
 	if ((c < f->offset) || (c >= (f->offset + f->nchars))) {
 		return(0);
 	}
@@ -823,8 +711,12 @@ int gdImageChar(im, f, x, y, c, color)
 		cx = 0;
 		cy++;
 	}
-	return(mx-x+1);
+	if (f->fixed) 
+	  return(f->w-2);
+	else
+	  return(mx-x+1);
 }
+
 /*SG gdCharWidth has been added to compute the effective width of a character */
 int gdCharWidth(f, c)
      gdFontPtr f;
@@ -853,7 +745,50 @@ int gdCharWidth(f, c)
 		cx = 0;
 		cy++;
 	}
-	return(mx+1);
+	if (f->fixed) 
+	  return(f->w-2);
+	else
+	  return(mx+1);
+}
+
+/*SG gdImageSymb added for centered symbols */
+int gdImageSymb(im, f, x, y, c, color)
+     gdImagePtr im;
+     gdFontPtr f;
+     int x;
+     int y;
+     int c;
+     int color;
+{
+	int cx, cy,xx,yy;
+	int px, py;
+	int fline;
+	int mx;
+	int w;
+	if ((c < f->offset) || (c >= (f->offset + f->nchars))) {
+		return(0);
+	}
+
+
+	cx = 0;
+	cy = 0;
+	mx = x;
+
+	w = gdCharWidth(f,c);
+	xx=x-w/2;yy=y-f->h/2;
+	fline = (c - f->offset) * f->h * f->w;
+	for (py = yy; (py < (yy + f->h)); py++) {
+		for (px = xx; (px < (xx + f->w)); px++) {
+			if (f->data[fline + cy * f->w + cx]) {
+				gdImageSetPixel(im, px, py, color);	
+				mx = Max(mx,px+1);
+			}
+			cx++;
+		}
+		cx = 0;
+		cy++;
+	}
+	return(1); /* Correction Warning missing return statement at end of non-void function */
 }
 
 
@@ -1866,7 +1801,7 @@ static void rl_flush_withtable(count)
     repleft = 1 + compute_triangle_count(leftover,max_ocodes);
   }
  if (VERBOSE) printf("rl_flush_withtable repmax=%d leftover=%d repleft=%d\n",repmax,leftover,repleft);
- if (1+compute_triangle_count(count,max_ocodes) < repmax+repleft)
+ if ( (int) (1+compute_triangle_count(count,max_ocodes)) < repmax+repleft)
   { output(code_clear);
     did_clear();
     rl_flush_fromclear(count);
@@ -2570,6 +2505,20 @@ void gdImageRectangle(im, x1, y1, x2, y2, color)
 	gdImageLine(im, x1, y1, x1, y2, color);
 	gdImageLine(im, x2, y1, x2, y2, color);
 }
+void gdImageThickRectangle(im, x1, y1, x2, y2, color, thick)
+     gdImagePtr im;
+     int x1;
+     int y1;
+     int x2;
+     int y2;
+     int color;
+     int thick;
+{
+	gdImageThickLine(im, x1, y1, x2, y1, color, thick);		
+	gdImageThickLine(im, x1, y2, x2, y2, color, thick);		
+	gdImageThickLine(im, x1, y1, x1, y2, color, thick);
+	gdImageThickLine(im, x2, y1, x2, y2, color, thick);
+}
 
 void gdImageFilledRectangle(im, x1, y1, x2, y2, color)
      gdImagePtr im;
@@ -2999,16 +2948,15 @@ void gdImagePolygon(im, p, n, c)
 }	
 	
 int gdCompareInt();
-	
-void gdImageFilledPolygon(im, p, n, c)
-     gdImagePtr im;
-     gdPointPtr p;
-     int n;
-     int c;
+void gdImageFilledPolygon(im,p,n,c)
+     gdImagePtr im; gdPointPtr p; int n; int c;
 {
 	int i;
 	int y;
-	int y1, y2;
+	int miny, maxy;
+	int x1, y1;
+	int x2, y2;
+	int ind1, ind2;
 	int ints;
 	if (!n) {
 		return;
@@ -3024,29 +2972,24 @@ void gdImageFilledPolygon(im, p, n, c)
 		im->polyInts = (int *) realloc(im->polyInts,
 			sizeof(int) * im->polyAllocated);
 	}
-	y1 = p[0].y;
-	y2 = p[0].y;
+	miny = p[0].y;
+	maxy = p[0].y;
 	for (i=1; (i < n); i++) {
-		if (p[i].y < y1) {
-			y1 = p[i].y;
+		if (p[i].y < miny) {
+			miny = p[i].y;
 		}
-		if (p[i].y > y2) {
-			y2 = p[i].y;
+		if (p[i].y > maxy) {
+			maxy = p[i].y;
 		}
 	}
 	/* Fix in 1.3: count a vertex only once */
-	for (y=y1; (y < y2); y++) {
-		int interLast = 0;
-		int dirLast = 0;
-		int interFirst = 1;
+	for (y=miny; (y <= maxy); y++) {
+/*1.4		int interLast = 0; */
+/*		int dirLast = 0; */
+/*		int interFirst = 1; */
 		ints = 0;
-		for (i=0; (i <= n); i++) {
-			int x1, x2;
-			int y1, y2;
-			int dir;
-			int ind1, ind2;
-			int lastInd1 = 0;
-			if ((i == n) || (!i)) {
+		for (i=0; (i < n); i++) {
+			if (!i) {
 				ind1 = n-1;
 				ind2 = 0;
 			} else {
@@ -3056,70 +2999,30 @@ void gdImageFilledPolygon(im, p, n, c)
 			y1 = p[ind1].y;
 			y2 = p[ind2].y;
 			if (y1 < y2) {
-				y1 = p[ind1].y;
-				y2 = p[ind2].y;
 				x1 = p[ind1].x;
 				x2 = p[ind2].x;
-				dir = -1;
 			} else if (y1 > y2) {
 				y2 = p[ind1].y;
 				y1 = p[ind2].y;
 				x2 = p[ind1].x;
 				x1 = p[ind2].x;
-				dir = 1;
 			} else {
-				/* Horizontal; just draw it */
-				gdImageLine(im, 
-					p[ind1].x, y1, 
-					p[ind2].x, y1,
-					c);
 				continue;
 			}
-			if ((y >= y1) && (y <= y2)) {
-				int inter = 
-					(y-y1) * (x2-x1) / (y2-y1) + x1;
-				/* Only count intersections once
-					except at maxima and minima. Also, 
-					if two consecutive intersections are
-					endpoints of the same horizontal line
-					that is not at a maxima or minima,	
-					discard the leftmost of the two. */
-				if (!interFirst) {
-					if ((p[ind1].y == p[lastInd1].y) &&
-						(p[ind1].x != p[lastInd1].x)) {
-						if (dir == dirLast) {
-							if (inter > interLast) {
-								/* Replace the old one */
-								im->polyInts[ints] = inter;
-							} else {
-								/* Discard this one */
-							}	
-							continue;
-						}
-					}
-					if (inter == interLast) {
-						if (dir == dirLast) {
-							continue;
-						}
-					}
-				} 
-				if (i > 0) {
-					im->polyInts[ints++] = inter;
-				}
-				lastInd1 = i;
-				dirLast = dir;
-				interLast = inter;
-				interFirst = 0;
+			if ((y >= y1) && (y < y2)) {
+				im->polyInts[ints++] = (y-y1) * (x2-x1) / (y2-y1) + x1;
+			} else if ((y == maxy) && (y > y1) && (y <= y2)) {
+				im->polyInts[ints++] = (y-y1) * (x2-x1) / (y2-y1) + x1;
 			}
 		}
 		qsort(im->polyInts, ints, sizeof(int), gdCompareInt);
-		for (i=0; (i < (ints-1)); i+=2) {
+
+		for (i=0; (i < (ints)); i+=2) {
 			gdImageLine(im, im->polyInts[i], y,
 				im->polyInts[i+1], y, c);
 		}
 	}
 }
-	
 int gdCompareInt(a, b)
      /*     const void *a;
 	    const void *b; */
@@ -3223,3 +3126,28 @@ void gdImageChangeColor(im, old, new)
     }
   }
 }
+void gdImagePPM(im, out)
+     gdImagePtr im;
+     FILE *out;
+{
+  int x,y,k;
+  int interlace, transparent, background, BitsPerPixel;
+  interlace = im->interlace;
+  transparent = im->transparent;
+  background = im->background;
+  BitsPerPixel = colorstobpp(im->colorsTotal);
+
+               
+  /*    Write the 'header' information  */
+  fprintf(out,"P6\n%d %d\n255\n",im->sx,im->sy);
+  for (y=0; (y < im->sy); y++) {	
+    for (x=0; (x < im->sx); x++) {	
+      k=im->pixels[y][x];
+      putc(im->red[k],out);
+      putc(im->green[k],out);
+      putc(im->blue[k],out);
+    }
+  }
+}
+
+
