@@ -1,8 +1,11 @@
 /*------------------------------------------------------------------------
-    Graphic library for 2D and 3D plotting 
-    Copyright (C) 1998 Chancelier Jean-Philippe
-    jpc@cergrene.enpc.fr 
+ *    Graphic library
+ *    Copyright (C) 1998-2001 Enpc/Jean-Philippe Chancelier
+ *    jpc@cermics.enpc.fr 
  --------------------------------------------------------------------------*/
+/*------------------------------------------------------------------------
+ * exported function : int C2F(xloadplots) (fname1,lvx)
+ * --------------------------------------------------------------------------*/
 
 #include <string.h> 
 
@@ -19,7 +22,7 @@
 #	ifndef __ABSC__
 #   	include <sys/types.h>	/* for <netinet/in.h> on some systems */
 #       endif
-#   	if !(defined __MSC__) && !(defined __ABSC__)
+#   	if !(defined __MSC__) && !(defined __ABSC__) && !(defined __MINGW32__)
 #          include <netinet/in.h>	/* for htonl() */
 #   	endif
 #	endif
@@ -39,14 +42,36 @@
 
 #define assert(ex) {if (!(ex)){ sciprint("Graphic Load/Save Error \r\n");return(0);}}
 
+extern void  xdrstdio_create();
+
 static char *SciF_version;
 static int LoadVectC(),LoadLI(),LoadVectLI(), LoadVectF(),LoadC(),LoadD();
 static int LoadTPlot();
-/*---------------------------------------------------------------------
-  Partie Load/load en utilisant xdr 
----------------------------------------------------------------------------*/
+static int LoadVectS   __PARAMS((char ***nx));
 
-int LoadXcall1()
+static int LoadPlot  __PARAMS((void)); 
+static int LoadSciAxis  __PARAMS((void)); 
+static int LoadGrid __PARAMS((void)); 
+static int LoadParam3D __PARAMS((void)); 
+static int LoadParam3D1 __PARAMS((void)); 
+static int LoadPlot3D __PARAMS((void)); 
+static int LoadFac3D __PARAMS((void)); 
+static int LoadFec __PARAMS((void)); 
+static int LoadContour __PARAMS((void)); 
+static int LoadContour2D __PARAMS((void)); 
+static int LoadGray __PARAMS((void)); 
+static int LoadChamp __PARAMS((void)); 
+static int LoadXcall1 __PARAMS((void)); 
+static int LoadEch __PARAMS((void)); 
+
+extern void C2F(syncexec)(char * str, int *ns, int *ierr, int *seq);
+extern int version_flag();
+
+/*---------------------------------------------------------------------
+ * reload a xcall1 statement 
+ *---------------------------------------------------------------------------*/
+
+static int LoadXcall1()
 {
   struct xcall1_rec *lplot ;
   lplot= ((struct xcall1_rec *) MALLOC(sizeof(struct xcall1_rec)));
@@ -85,10 +110,10 @@ int LoadXcall1()
 }
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas des changement d'echelle}
-  ---------------------------------------------------------------------------*/
+ * scales 
+ *---------------------------------------------------------------------------*/
 
-int LoadEch()
+static int LoadEch()
 {
   struct scale_rec *lplot;
   lplot= ((struct scale_rec *) MALLOC(sizeof(struct scale_rec)));
@@ -110,15 +135,34 @@ int LoadEch()
   return(1);
 }
 
+static int LoadNEch()
+{
+  struct nscale_rec *lplot;
+  lplot= ((struct nscale_rec *) MALLOC(sizeof(struct nscale_rec)));
+  if (lplot != NULL)
+    {
+      if (LoadC(lplot->logflag) == 0) return(0);
+      if (LoadVectC(&(lplot->name)) == 0) return(0);
+      if (LoadVectC(&(lplot->flag)) == 0) return(0);
+      if (LoadVectF(&(lplot->Wrect)) == 0) return(0);
+      if (LoadVectF(&(lplot->Frect)) == 0) return(0);
+      if (LoadVectF(&(lplot->Arect)) == 0) return(0);
+      if (LoadVectF(&(lplot->Frect_kp))   == 0) return(0);
+      if (Store("nscale",(char *) lplot) == 0) return(0);
+    }
+  else 
+    {
+      Scistring("\nLoad Plot (Ech): No more place \n");
+      return(0);
+    }
+  return(1);
+}
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas des plot2d}
-  ---------------------------------------------------------------------------*/
+ * 2D plots  
+ *---------------------------------------------------------------------------*/
 
-/* Load the plot2d  in the plot list 
-   same arguments as plot2d */
-
-int LoadPlot()
+static int LoadPlot()
 {
   integer n=0, nstyle;
   struct plot2d_rec *lplot;
@@ -162,10 +206,50 @@ int LoadPlot()
 
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas de xgrid}
-  ---------------------------------------------------------------------------*/
+ * xgrid 
+ *---------------------------------------------------------------------------*/
 
-int LoadGrid()
+static int LoadSciAxis()
+{
+  char type[3] ;
+  struct sciaxis_rec *lplot;
+  if ((lplot= ((struct sciaxis_rec *) MALLOC(sizeof(struct sciaxis_rec)))) == NULL) 
+    {
+      Scistring("\nLoad Plot (grid): No more place \n");
+      return(0);
+    }
+  if ( LoadC(type)== 0) return(0);
+  if ( LoadLI(&lplot->nx)==0) return(0);
+  if ( LoadLI(&lplot->ny)==0) return(0);
+  if ( LoadLI(&lplot->subtics)==0) return(0);
+  if ( LoadLI(&lplot->fontsize)==0) return(0);
+  if ( LoadLI(&lplot->textcolor)==0) return(0);
+  if ( LoadLI(&lplot->ticscolor)==0) return(0);
+  if ( LoadLI(&lplot->seg_flag)==0) return(0);
+  if ( LoadLI(&lplot->f_l)==0) return(0);
+  if (LoadVectC(&(lplot->name)) == 0) return(0);
+  if ( lplot->f_l == 1) 
+    {
+      if ( LoadVectC( &(lplot->format)) == 0) return(0);
+    }
+  else
+    lplot->format =0;
+  if ( LoadVectF(&lplot->x) == 0) return(0);
+  if ( LoadVectF(&lplot->y) == 0) return(0);
+  if ( LoadVectS(&lplot->str) == 0) return(0);
+  
+  lplot->pos =  type[0] ;
+  lplot->xy_type =  type[1];
+  lplot->logflag  = type[2];
+  if (Store("axis",(char *) lplot) == 0) return(0);
+  return(1);
+}
+
+/*---------------------------------------------------------------------
+ * xgrid 
+ *---------------------------------------------------------------------------*/
+
+static int LoadGrid()
 { 
   struct xgrid_rec *lplot ;
   lplot= ((struct xgrid_rec *) MALLOC(sizeof(struct xgrid_rec)));
@@ -186,10 +270,10 @@ int LoadGrid()
 
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas du param3d}
-  ---------------------------------------------------------------------------*/
+ * param3d 
+ *---------------------------------------------------------------------------*/
 
-int LoadParam3D()
+static int LoadParam3D()
 {
   struct param3d_rec *lplot;
   lplot= ((struct param3d_rec *) MALLOC(sizeof(struct param3d_rec)));
@@ -216,8 +300,7 @@ int LoadParam3D()
   return(1);
 }
 
-
-int LoadParam3D1()
+static int LoadParam3D1()
 {
   struct param3d1_rec *lplot;
   lplot= ((struct param3d1_rec *) MALLOC(sizeof(struct param3d1_rec)));
@@ -248,12 +331,11 @@ int LoadParam3D1()
   return(1);
 }
 
-
 /*---------------------------------------------------------------------
-  \encadre{Le cas des plot3d}
-  ---------------------------------------------------------------------------*/
+ * plot3d 
+ *---------------------------------------------------------------------------*/
 
-int LoadPlot3D()
+static int LoadPlot3D()
 {
   struct plot3d_rec *lplot ;
   lplot= ((struct plot3d_rec *) MALLOC(sizeof(struct plot3d_rec)));
@@ -282,10 +364,10 @@ int LoadPlot3D()
 }
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas des fac3d}
-  ---------------------------------------------------------------------------*/
+ * fac3d 
+ *---------------------------------------------------------------------------*/
 
-int LoadFac3D()
+static int LoadFac3D()
 {
   struct fac3d_rec *lplot;
   lplot= ((struct fac3d_rec *) MALLOC(sizeof(struct fac3d_rec)));
@@ -298,6 +380,8 @@ int LoadFac3D()
       if (LoadVectC(&(lplot->name)) == 0) return(0);
       if ( strcmp(lplot->name,"fac3d2")==0) 
 	if ( LoadVectLI(&(lplot->cvect)) == 0) return(0);
+      if ( strcmp(lplot->name,"fac3d3")==0) /****** added by polpoth 4/5/2000 ******/ 
+	if ( LoadVectLI(&(lplot->cvect)) == 0) return(0); /****** entry added by polpoth 4/5/2000 ******/
       if (LoadVectF(&(lplot->x)) == 0) return(0);
       if (LoadVectF(&(lplot->y)) == 0) return(0);
       if (LoadVectF(&(lplot->z)) == 0) return(0);
@@ -316,10 +400,11 @@ int LoadFac3D()
 }
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas des fec}
-  ---------------------------------------------------------------------------*/
+ * fec 
+ *---------------------------------------------------------------------------*/
 
-int LoadFec()
+/* pas compatibility */
+static int LoadFec()
 {
   struct fec_rec *lplot;
   lplot= ((struct fec_rec *) MALLOC(sizeof(struct fec_rec)));
@@ -339,6 +424,47 @@ int LoadFec()
       if (LoadVectC(&(lplot->strflag)) == 0) return(0);
       if (LoadVectC(&(lplot->strflag_kp)) == 0) return(0);
       if (LoadVectC(&(lplot->legend))  == 0) return(0);
+      if ((lplot->zminmax = (double *)  MALLOC(2*sizeof(double))) == 0) return (0);
+      if ((lplot->colminmax = (int *)  MALLOC(2*sizeof(int))) == 0) return (0);
+      if ((lplot->colout = (int *)  MALLOC(2*sizeof(int))) == 0) return (0);
+      lplot->zminmax[0] = lplot->zminmax[1]= 0.0;
+      lplot->colminmax[0] = lplot->colminmax[1]=1;
+      lplot->colout[0] = lplot->colout[1]=-1;
+      lplot->with_mesh = 0;
+      if (Store(lplot->name,(char *) lplot) == 0) return(0);
+    }
+  else 
+    {
+      Scistring("\nLoad Plot (fec): No more place \n");
+      return(0);
+    }
+  return(1);
+}
+
+static int LoadFec_n()
+{
+  struct fec_rec *lplot;
+  lplot= ((struct fec_rec *) MALLOC(sizeof(struct fec_rec)));
+  if (lplot != NULL)
+    {
+      if (LoadLI(&lplot->Nnode) == 0) return(0);
+      if (LoadLI(&lplot->Ntr) == 0) return(0);
+      if (LoadVectC(&(lplot->name)) == 0) return(0);
+      if (LoadVectF(&(lplot->x)) == 0) return(0);
+      if (LoadVectF(&(lplot->y)) == 0) return(0);
+      if (LoadVectF(&(lplot->triangles)) == 0) return(0);
+      if (LoadVectF(&(lplot->func) ) == 0) return(0);
+      if (LoadVectF(&(lplot->brect)) == 0) return(0);
+      if (LoadVectF(&(lplot->brect_kp)) == 0) return(0);
+      if (LoadVectF(&(lplot->zminmax)) == 0) return(0);    /* added by bruno */
+      if (LoadVectLI(&(lplot->colminmax)) == 0) return(0); /* added by bruno */
+      if (LoadVectLI(&(lplot->colout)) == 0) return(0); /* added by bruno */
+      if (LoadLI(&lplot->with_mesh) == 0) return(0);
+      if (LoadVectLI(&(lplot->aaint)) == 0) return(0);
+      if (LoadVectLI(&(lplot->aaint_kp)) == 0) return(0);
+      if (LoadVectC(&(lplot->strflag)) == 0) return(0);
+      if (LoadVectC(&(lplot->strflag_kp)) == 0) return(0);
+      if (LoadVectC(&(lplot->legend))  == 0) return(0);
       if (Store(lplot->name,(char *) lplot) == 0) return(0);
       
     }
@@ -352,10 +478,10 @@ int LoadFec()
 
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas des contours}
-  ---------------------------------------------------------------------------*/
+ * contour 
+ *---------------------------------------------------------------------------*/
 
-int LoadContour()
+static int LoadContour()
 {
   struct contour_rec *lplot;
   lplot= ((struct contour_rec *) MALLOC(sizeof(struct contour_rec)));
@@ -391,7 +517,7 @@ int LoadContour()
 }
 
 
-int LoadContour2D()
+static int LoadContour2D()
 {
   struct contour2d_rec *lplot;
   lplot= ((struct contour2d_rec *) MALLOC(sizeof(struct contour2d_rec)));
@@ -430,13 +556,10 @@ int LoadContour2D()
 
 
 /*---------------------------------------------------------------------
-  \encadre{Le cas des niveaux de gris }
-  ---------------------------------------------------------------------------*/
+ * gray plot 
+ * ---------------------------------------------------------------------------*/
 
-/* Load the plot in the plot list 
-   same arguments as gray */
-
-int LoadGray()
+static int LoadGray()
 {
  
   struct gray_rec *lplot;
@@ -466,13 +589,14 @@ int LoadGray()
   return(1);
 }
 
-int LoadGray1()
+static int LoadGray1()
 {
  
   struct gray_rec *lplot;
   lplot= ((struct gray_rec *) MALLOC(sizeof(struct gray_rec)));
   if (lplot != NULL)
     {
+      lplot->x = lplot->y = NULL; /* unused in Gray1 */
       if (LoadLI(&lplot->n1) == 0) return(0);
       if (LoadLI(&lplot->n2) == 0) return(0);
       if (LoadVectC(&(lplot->name)) == 0) return(0);
@@ -494,7 +618,7 @@ int LoadGray1()
   return(1);
 }
 
-int LoadGray2()
+static int LoadGray2()
 {
  
   struct gray_rec_2 *lplot;
@@ -518,12 +642,11 @@ int LoadGray2()
 }
 
 
-
 /*---------------------------------------------------------------------
-  \encadre{Le cas des champs de vecteurs}
-  ---------------------------------------------------------------------------*/
+ * champ 
+ * ---------------------------------------------------------------------------*/
 
-int LoadChamp()
+static int LoadChamp()
 {
   struct champ_rec *lplot;
   lplot= ((struct champ_rec *) MALLOC(sizeof(struct champ_rec)));
@@ -555,8 +678,8 @@ int LoadChamp()
 
 
 /*---------------------------------------------------------------------
-  Relecture 
-  ---------------------------------------------------------------------------*/
+ *  code for reload operations 
+ *---------------------------------------------------------------------------*/
 
 static char RFname[128];
 static FILE *RF ;
@@ -566,10 +689,11 @@ static u_int rszof ;
 
 /** special case for colormap **/
 
-int LoadColormap()
+static int LoadColormap()
 {
   double *table; double x;
   int m,i,trois=3;
+  int succeed;
   if (LoadLI(&m) == 0) return(0);
   table  = (double *)  MALLOC(3*m*sizeof(double));
   if ( table == NULL) return(0);
@@ -583,7 +707,7 @@ int LoadColormap()
       table[i]=x;
       /** sciprint("loading %f\r\n",table[i]); **/
     }
-  C2F(dr)("xset","colormap",&m,&trois,PI0,PI0,PI0,PI0,table,PD0,PD0,PD0,0L,0L);
+  C2F(dr)("xset","colormap",&m,&trois,&succeed,PI0,PI0,PI0,table,PD0,PD0,PD0,0L,0L);
   FREE(table);
   return(1);
 }
@@ -594,6 +718,7 @@ typedef  struct  {
   int  (*Load)();} LoadTable;
 
 static LoadTable LoadCTable[] ={
+  {"axis",LoadSciAxis},
   {"champ",LoadChamp},
   {"colormap",LoadColormap},
   {"contour",LoadContour},
@@ -601,10 +726,13 @@ static LoadTable LoadCTable[] ={
   {"fac3d",LoadFac3D},
   {"fac3d1",LoadFac3D},
   {"fac3d2",LoadFac3D},
+  {"fac3d3",LoadFac3D}, /****** entry added by polpoth 4/5/2000 ******/
   {"fec",LoadFec},
+  {"fec_n",LoadFec_n},
   {"gray",LoadGray},
   {"gray1",LoadGray1}, 
   {"gray2",LoadGray2}, 
+  {"nscale",LoadNEch},
   {"param3d",LoadParam3D},
   {"param3d1",LoadParam3D1},
   {"plot2d",LoadPlot},
@@ -619,7 +747,6 @@ static LoadTable LoadCTable[] ={
 #define __STDC__
 #endif 
 
-
 int C2F(xloadplots)(fname1,lvx)
      char *fname1;
      integer lvx;
@@ -638,16 +765,36 @@ int C2F(xloadplots)(fname1,lvx)
       return(0);
     }
   xdrstdio_create(rxdrs, RF, XDR_DECODE) ;
-  if ( LoadVectC(&SciF_version) == 0 ) 
+
+  /*if ( LoadVectC(&SciF_version) == 0 )  replaced by the following line to avoid error message*/
+  xdr_vector(rxdrs,(char *) &rszof,(u_int)1,(u_int) sizeof(u_int), (xdrproc_t) xdr_u_int);
+  SciF_version = (char *)  MALLOC(rszof);
+  if (( version_flag() == 0||SciF_version == NULL) || (xdr_opaque(rxdrs, SciF_version,rszof)==0))
     {
-      sciprint("Wrong plot file : %s\n\n",fname1);
+      char temp[256];
+      integer ierr,seq=1;
+      fclose(RF);
+      sprintf(temp,"%%xload('%s')",fname1);
+      na=strlen(temp);
+      C2F(syncexec)(temp,&na,&ierr,&seq);
+      if(ierr != 0)  sciprint("Wrong plot file : %s\n\n",fname1);
       return(0);
     }
-  if ( strcmp(SciF_version,"SciG1.1") != 0 && strcmp(SciF_version,"SciG1.0")) 
+
+  if ( strncmp(SciF_version,"SciG",4) != 0 )
     {
-      sciprint("Wrong plot file : %s\n\n",fname1);
+      sciprint("Not a save graphics file: %s\n\n",fname1);
       return(0);
     }
+  if ( strcmp(SciF_version,"SciG1.0") != 0 && 
+       strcmp(SciF_version,"SciG1.1") != 0 )
+    {
+      sciprint("Wrong version of saved graphics %s : %s\n\n",
+	       SciF_version,fname1);
+      return(0);
+    }
+
+
   while (LoadVectC(&type) != 0 && strcmp(type,"endplots") != 0) 
     {
       if (LoadTPlot(type) == 0) break;
@@ -763,3 +910,21 @@ static int LoadVectC(nx)
   assert( xdr_opaque(rxdrs, *nx,rszof));
   return(1);
 }
+
+static int LoadVectS(nx) 
+     char ***nx ;
+{
+  int i;
+  char **loc;
+  assert( xdr_vector(rxdrs,(char *) &rcount,(u_int)1,(u_int) sizeof(u_int), (xdrproc_t) xdr_u_int)) ;
+  if ( rcount == 0) { *nx = 0 ; return 1;} 
+  if (( loc = (char **) MALLOC( (rcount+1)* sizeof(char*))) == NULL) return 0;
+  for ( i=0; i < (int) rcount ; i++) 
+    {
+      if ( LoadVectC(&loc[i]) == 0 ) return 0;
+    }
+  loc[rcount]= 0;
+  *nx = loc ;
+  return 1;
+}
+
