@@ -1,30 +1,50 @@
 /*------------------------------------------------------------------------
-    Graphic library for 2D and 3D plotting 
-    Copyright (C) 1998 Chancelier Jean-Philippe
-    jpc@cergrene.enpc.fr 
+ *    Graphic library
+ *    Copyright (C) 1998-2001 Enpc/Jean-Philippe Chancelier
+ *    jpc@cermics.enpc.fr 
  --------------------------------------------------------------------------*/
+/*------------------------------------------------------------------------
+ * save graphic code in a file  
+ * --------------------------------------------------------------------------*/
 
 #include <string.h> /* in case of dbmalloc use */
-#ifdef __STDC__
-#include <stdlib.h>
-#else
-#include <malloc.h>
-#endif
-
 #include <stdio.h>
 #include "Math.h"
-
 #include "Rec.h"
 
-static int SaveTPlot(), SaveD(), SaveLI(),SaveC();
-static int SaveVectLI(), SaveVectF(), SaveVectC();
+extern void C2F(syncexec)(char * str, int *ns, int *ierr, int *seq);
+extern void  xdrstdio_create();
+
+static int SaveTPlot __PARAMS((char *type, char *plot));
+static int SaveD  __PARAMS((double x));
+static int SaveLI __PARAMS((integer ix));
+static int SaveC __PARAMS((char *c, integer lc));
+static int SaveVectLI __PARAMS((integer *nx, integer l));
+static int SaveVectF __PARAMS((double *nx, integer l)); 
+static int SaveVectC __PARAMS((char *nx, int l));
+static int SaveVectS   __PARAMS((char **nx));
+static int SaveColormap __PARAMS((void));
+static int SaveEch  __PARAMS((char *)); 
+static int SavePlot  __PARAMS((char *)); 
+static int SaveSciAxis  __PARAMS((char *)); 
+static int SaveGrid  __PARAMS((char *)); 
+static int SaveParam3D  __PARAMS((char *)); 
+static int SaveParam3D1  __PARAMS((char *)); 
+static int SavePlot3D  __PARAMS((char *)); 
+static int SaveFac3D  __PARAMS((char *)); 
+static int SaveFec  __PARAMS((char *)); 
+static int SaveContour  __PARAMS((char *)); 
+static int SaveXcall1  __PARAMS((char *)); 
+static int SaveContour2D  __PARAMS((char *)); 
+static int SaveGray  __PARAMS((char *)); 
+static int SaveChamp  __PARAMS((char *)); 
 
 /*---------------------------------------------------------------------
-Partie save/load en utilisant xdr 
----------------------------------------------------------------------------*/
+ * save plots using xdr 
+ * exproted function int C2F(xsaveplots) __PARAMS((integer *winnumber,char *,integer lxv)); 
+ *---------------------------------------------------------------------------*/
 
-int SaveXcall1(plot)
-     char *plot;
+static int SaveXcall1(char *plot)
 {
   struct xcall1_rec *lplot = (struct xcall1_rec *) plot;
   if ( SaveVectC((lplot->fname),((int)strlen(lplot->fname))+1) == 0) return(0);
@@ -53,11 +73,10 @@ int SaveXcall1(plot)
 }
 
 /*---------------------------------------------------------------------
-\encadre{Le cas des changement d'echelle}
----------------------------------------------------------------------------*/
+ * scale 
+ * ---------------------------------------------------------------------------*/
 
-int SaveEch(plot)
-     char *plot;
+static int SaveEch(char *plot)
 {
   struct scale_rec *lplot = (struct scale_rec *) plot;
   if ( SaveC(lplot->logflag,2L)== 0) return(0);
@@ -68,16 +87,24 @@ int SaveEch(plot)
   return(1);
 }
 
+static int SaveNEch(char *plot)
+{
+  struct nscale_rec *lplot = (struct nscale_rec *) plot;
+  if ( SaveC(lplot->logflag,2L)== 0) return(0);
+  if ( SaveVectC((lplot->name),((int)strlen(lplot->name))+1) == 0) return(0);
+  if ( SaveVectC((lplot->flag),((int)strlen(lplot->flag))+1) == 0) return(0);
+  if ( SaveVectF((lplot->Wrect),4L) == 0) return(0);
+  if ( SaveVectF((lplot->Frect),4L) == 0) return(0);
+  if ( SaveVectF((lplot->Arect),4L) == 0) return(0);
+  if ( SaveVectF((lplot->Frect_kp),4L) == 0) return(0);
+  return(1);
+}
   
 /*---------------------------------------------------------------------
-\encadre{Le cas des plot2d}
----------------------------------------------------------------------------*/
+ * plot2d 
+ *---------------------------------------------------------------------------*/
 
-/* Save the plot2d  in the plot list 
-   same arguments as plot2d */
-
-int SavePlot(plot)
-     char *plot;
+static int SavePlot(char *plot)
 {
   integer n=0, nstyle;
   struct plot2d_rec *lplot = (struct plot2d_rec *) plot;
@@ -104,14 +131,40 @@ int SavePlot(plot)
   if ( SaveVectLI((lplot->aint_kp),4L) == 0) return(0);
   return(1);
 }
-
   
 /*---------------------------------------------------------------------
-\encadre{Le cas de xgrid}
----------------------------------------------------------------------------*/
+ * axis 
+ *---------------------------------------------------------------------------*/
 
-int SaveGrid(plot)
-     char *plot;
+static int SaveSciAxis(char *plot)
+{
+  struct sciaxis_rec *lplot = (struct sciaxis_rec *) plot;
+  char type[3] ;
+  type[0] = lplot->pos;
+  type[1] = lplot->xy_type;
+  type[2] = lplot->logflag;
+  if ( SaveC(type,3L)== 0) return(0);
+  if ( SaveLI(lplot->nx)==0) return(0);
+  if ( SaveLI(lplot->ny)==0) return(0);
+  if ( SaveLI(lplot->subtics)==0) return(0);
+  if ( SaveLI(lplot->fontsize)==0) return(0);
+  if ( SaveLI(lplot->textcolor)==0) return(0);
+  if ( SaveLI(lplot->ticscolor)==0) return(0);
+  if ( SaveLI(lplot->seg_flag)==0) return(0);
+  if ( SaveLI(lplot->f_l)==0) return(0);
+  if ( SaveVectC((lplot->name),((int)strlen(lplot->name))+1) == 0) return(0);
+  if ( lplot->f_l == 1) { if ( SaveVectC((lplot->format),((int)strlen(lplot->format))+1) == 0) return(0);} 
+  if ( SaveVectF(lplot->x,lplot->nx) == 0) return(0);
+  if ( SaveVectF(lplot->y,lplot->ny) == 0) return(0);
+  if ( SaveVectS(lplot->str) == 0) return(0);
+  return(1);
+}
+
+/*---------------------------------------------------------------------
+ * xgrid 
+ *---------------------------------------------------------------------------*/
+
+static int SaveGrid(char *plot)
 {
   struct xgrid_rec *lplot = (struct xgrid_rec *) plot;
   if ( SaveLI(lplot->style)== 0) return(0);
@@ -121,11 +174,10 @@ int SaveGrid(plot)
 
 
 /*---------------------------------------------------------------------
-\encadre{Le cas du param3d}
----------------------------------------------------------------------------*/
+ * param3d 
+ *---------------------------------------------------------------------------*/
 
-int SaveParam3D(plot)
-     char *plot;
+static int SaveParam3D(char *plot)
 {
   struct param3d_rec *lplot = (struct param3d_rec *) plot;
   if ( SaveLI(lplot->n)== 0) return(0);
@@ -141,8 +193,7 @@ int SaveParam3D(plot)
   return(1);
 }
 
-int SaveParam3D1(plot)
-     char *plot;
+static int SaveParam3D1(char *plot)
 {
   struct param3d1_rec *lplot = (struct param3d1_rec *) plot;
   if ( SaveLI(lplot->m)== 0) return(0);
@@ -164,11 +215,10 @@ int SaveParam3D1(plot)
 
 
 /*---------------------------------------------------------------------
-\encadre{Le cas des plot3d}
----------------------------------------------------------------------------*/
+ * plot3d  
+ *---------------------------------------------------------------------------*/
 
-int SavePlot3D(plot)
-     char *plot;
+static int SavePlot3D(char *plot)
 {
   struct plot3d_rec *lplot = (struct plot3d_rec *) plot;
   if ( SaveLI(lplot->p)== 0) return(0);
@@ -186,11 +236,10 @@ int SavePlot3D(plot)
 }
 
 /*---------------------------------------------------------------------
-\encadre{Le cas des fac3d}
----------------------------------------------------------------------------*/
+ * fac3d 
+ *---------------------------------------------------------------------------*/
 
-int SaveFac3D(plot)
-     char *plot;
+static int SaveFac3D(char *plot)
 {
   struct fac3d_rec *lplot = (struct fac3d_rec *) plot;
   if ( SaveLI(lplot->p)== 0) return(0);
@@ -200,6 +249,9 @@ int SaveFac3D(plot)
   if ( SaveVectC((lplot->name),((int)strlen(lplot->name))+1) == 0) return(0);
   if ( strcmp(lplot->name,"fac3d2")==0) 
     if ( SaveVectLI((lplot->cvect),(lplot->q)) == 0) return(0);
+  if ( strcmp(lplot->name,"fac3d3")==0) { /****** added by polpoth 4/5/2000 ******/
+    if ( SaveVectLI((lplot->cvect),(lplot->p)*(lplot->q)) == 0) return(0); /****** added by polpoth 4/5/2000 ******/
+  }
   if ( SaveVectF((lplot->x),(lplot->p)*(lplot->q)) == 0) return(0);
   if ( SaveVectF((lplot->y),(lplot->p)*(lplot->q)) == 0) return(0);
   if ( SaveVectF((lplot->z),(lplot->p)*(lplot->q)) == 0) return(0);
@@ -210,11 +262,10 @@ int SaveFac3D(plot)
 }
 
 /*---------------------------------------------------------------------
-\encadre{Le cas des fec}
----------------------------------------------------------------------------*/
+ * fec 
+ *---------------------------------------------------------------------------*/
 
-int SaveFec(plot)
-     char *plot;
+static int SaveFec(char *plot)
 {
   struct fec_rec *lplot = (struct fec_rec *) plot;
   if ( SaveLI(lplot->Nnode)== 0) return(0);
@@ -226,6 +277,10 @@ int SaveFec(plot)
   if ( SaveVectF((lplot->func),lplot->Nnode ) == 0) return(0);
   if ( SaveVectF((lplot->brect),4L) == 0) return(0);
   if ( SaveVectF((lplot->brect_kp),4L) == 0) return(0);
+  if ( SaveVectF((lplot->zminmax),2L) == 0) return(0);   /* added by bruno */
+  if ( SaveVectLI((lplot->colminmax),2L) == 0) return(0);/* ============== */
+  if ( SaveVectLI((lplot->colout),2L) == 0) return(0);/* ============== */
+  if ( SaveLI(lplot->with_mesh)== 0) return(0);             /* ============== */
   if ( SaveVectLI((lplot->aaint),4L) == 0) return(0);
   if ( SaveVectLI((lplot->aaint_kp),4L) == 0) return(0);
   if ( SaveVectC((lplot->strflag),((int)strlen(lplot->strflag))+1) == 0) return(0);
@@ -234,13 +289,11 @@ int SaveFec(plot)
   return(1);
 }
 
-
 /*---------------------------------------------------------------------
-\encadre{Le cas des contours}
----------------------------------------------------------------------------*/
+ * contour 
+ *---------------------------------------------------------------------------*/
 
-int SaveContour(plot)
-     char *plot;
+static int SaveContour(char *plot)
 {
   struct contour_rec *lplot = (struct contour_rec *) plot;
   if ( SaveLI(lplot->n1)== 0) return(0);
@@ -262,8 +315,7 @@ int SaveContour(plot)
   return(1);
 }
 
-int SaveContour2D(plot)
-     char *plot;
+static int SaveContour2D(char *plot)
 {
   integer nstyle;
   struct contour2d_rec *lplot = (struct contour2d_rec *) plot;
@@ -290,14 +342,10 @@ int SaveContour2D(plot)
 }
 
 /*---------------------------------------------------------------------
-\encadre{Le cas des niveaux de gris }
----------------------------------------------------------------------------*/
+ * xgray 
+ *---------------------------------------------------------------------------*/
 
-/* Save the plot in the plot list
-   same arguments as gray */
-
-int SaveGray(plot)
-     char *plot;
+static int SaveGray(char *plot)
 {
   struct gray_rec *lplot = (struct gray_rec *) plot;
   if ( SaveLI(lplot->n1)== 0) return(0);
@@ -315,8 +363,7 @@ int SaveGray(plot)
   return(1);
 }
 
-int SaveGray1(plot)
-     char *plot;
+static int SaveGray1(char *plot)
 {
   struct gray_rec *lplot = (struct gray_rec *) plot;
   if ( SaveLI(lplot->n1)== 0) return(0);
@@ -332,9 +379,7 @@ int SaveGray1(plot)
   return(1);
 }
 
-
-int SaveGray2(plot)
-     char *plot;
+static int SaveGray2(char *plot)
 {
   struct gray_rec_2 *lplot = (struct gray_rec_2 *) plot;
   if ( SaveLI(lplot->n1)== 0) return(0);
@@ -345,13 +390,11 @@ int SaveGray2(plot)
   return(1);
 }
 
-
 /*---------------------------------------------------------------------
-\encadre{Le cas des champs de vecteurs}
----------------------------------------------------------------------------*/
+ * champ 
+ *---------------------------------------------------------------------------*/
 
-int SaveChamp(plot)
-     char *plot;
+static int SaveChamp(char *plot)
 {
   struct champ_rec *lplot = (struct champ_rec *) plot;
   if ( SaveLI(lplot->n1)== 0) return(0);
@@ -370,8 +413,8 @@ int SaveChamp(plot)
 }
 
 /*---------------------------------------------------------------------
-Sauvegarde 
----------------------------------------------------------------------------*/
+ * Saving  
+ *---------------------------------------------------------------------------*/
 
 #ifdef macintosh
 #	include "types.h"
@@ -380,7 +423,7 @@ Sauvegarde
 #	ifndef __ABSC__
 #   	include <sys/types.h>	/* for <netinet/in.h> on some systems */
 #	endif
-#   	if (!defined __MSC__) && !(defined __ABSC__)
+#   	if (!defined __MSC__) && !(defined __ABSC__) && !(defined __MINGW32__) 
 #          include <netinet/in.h>	/* for htonl() */
 #   	endif
 #	endif
@@ -403,45 +446,59 @@ static XDR xdrs[1] ;
 static u_int count ;
 static u_int szof ;
 
-
 typedef  struct  {
   char *name;
   int  (*Save)();} SaveTable;
 
 static SaveTable SaveCTable[] ={
-    {"champ",SaveChamp},
-    {"contour",SaveContour},
-    {"contour2",SaveContour2D},
-    {"fac3d",SaveFac3D},
-    {"fac3d1",SaveFac3D},
-    {"fac3d2",SaveFac3D},
-    {"fec",SaveFec},
-    {"gray",SaveGray},
-    {"gray1",SaveGray1},
-    {"gray2",SaveGray2},
-    {"param3d",SaveParam3D},
-    {"param3d1",SaveParam3D1},
-    {"plot2d",SavePlot},
-    {"plot3d",SavePlot3D},
-    {"plot3d1",SavePlot3D},
-    {"scale",SaveEch},
-    {"xcall1",SaveXcall1},
-    {"xgrid",SaveGrid},
-    {(char *)NULL,NULL}};
-
+  {"axis", SaveSciAxis},
+  {"champ",SaveChamp},
+  {"contour",SaveContour},
+  {"contour2",SaveContour2D},
+  {"fac3d",SaveFac3D},
+  {"fac3d1",SaveFac3D},
+  {"fac3d2",SaveFac3D},
+  {"fac3d3",SaveFac3D},/****** entry added by polpoth 4/5/2000 ******/
+  {"fec_n",SaveFec},
+  {"gray",SaveGray},
+  {"gray1",SaveGray1},
+  {"gray2",SaveGray2},
+  {"nscale",SaveNEch},
+  {"param3d",SaveParam3D},
+  {"param3d1",SaveParam3D1},
+  {"plot2d",SavePlot},
+  {"plot3d",SavePlot3D},
+  {"plot3d1",SavePlot3D},
+  {"scale",SaveEch},
+  {"xcall1",SaveXcall1},
+  {"xgrid",SaveGrid},
+  {(char *)NULL,NULL}};
 
 #ifdef __MSC__
 #define __STDC__
 #endif 
 
-int C2F(xsaveplots)(winnumber, fname1, lxv)
-     integer *winnumber;
-     char *fname1;
-     integer lxv; 
+extern int version_flag(void); /* NG */
+
+int C2F(xsaveplots)(integer *winnumber, char *fname1, integer lxv)
 {
   static char endplots[]={"endplots"};
   static char scig[]={"SciG1.1"};
   struct listplot *list;
+  int verb=0,cur,na;
+
+  if (version_flag() == 0) {
+    char temp[256];
+    integer ierr,seq=1;
+    sprintf(temp,"%%xsave('%s',%d)",fname1,*winnumber);
+    na=strlen(temp);
+    C2F(syncexec)(temp,&na,&ierr,&seq);
+    return 0;
+  }
+  C2F(dr)("xget","window",&verb,&cur,&na,PI0,PI0,PI0,PD0,PD0,PD0,PD0,0L,0L);
+  C2F(dr)("xset","window",winnumber,PI0,PI0,PI0,PI0,PI0,PD0,PD0,PD0,PD0,0L,0L);
+
+
 #ifdef lint 
   *unused;
 #endif
@@ -477,9 +534,7 @@ int C2F(xsaveplots)(winnumber, fname1, lxv)
   return(0);
 }
 
-static int SaveTPlot(type, plot)
-     char *type;
-     char *plot;
+static int SaveTPlot(char *type, char *plot)
 {
   int i=0;
   while (SaveCTable[i].name != (char *) NULL)
@@ -503,8 +558,7 @@ static int SaveTPlot(type, plot)
   return(0);
 }
 
-static int SaveD(x)
-     double x;
+static int SaveD(double x)
 {
   szof = sizeof(double) ;
   count = 1;
@@ -512,8 +566,7 @@ static int SaveD(x)
   return(1);
 }
 
-static int SaveF(x)
-     float x;
+static int SaveF(float x)
 {
   double z=x;
   SaveD(z);
@@ -527,8 +580,7 @@ static int SaveF(x)
   return(1);
 }
 
-static int SaveLI(ix)
-     integer ix;
+static int SaveLI(integer ix)
 {
   szof = sizeof(int) ;
   count = 1;
@@ -536,9 +588,7 @@ static int SaveLI(ix)
   return(1);
 }
 
-static int SaveC(c, lc)
-     char *c;
-     integer lc;
+static int SaveC(char *c, integer lc)
 {
   szof = lc*sizeof(char);
   assert( xdr_vector(xdrs,(char *) &szof,(unsigned)1,(unsigned) sizeof(unsigned), (xdrproc_t) xdr_u_int)) ;
@@ -569,9 +619,7 @@ static int SaveVectI(nx,l)
 
 **/
 
-static int SaveVectLI(nx, l)
-     integer *nx;
-     integer l;
+static int SaveVectLI(integer *nx, integer l)
 { 
   integer nx1=1;
   /** Attention integer peut etre un long int **/
@@ -589,9 +637,7 @@ static int SaveVectLI(nx, l)
   return(1);
 }
 
-static int SaveVectF(nx, l)
-     double *nx;
-     integer l;
+static int SaveVectF(double *nx, integer l)
 {
   double nx1=0.0;
   szof = sizeof(double) ;
@@ -604,10 +650,8 @@ static int SaveVectF(nx, l)
   return(1);
 }
 
-static int SaveVectC(nx, l)
-     char *nx;
-     int l;
-{
+static int SaveVectC(char *nx, int l)
+{ 
   char nx1='1';
   szof = l*sizeof(char);
   assert( xdr_vector(xdrs,(char *) &szof,(unsigned)1,(unsigned) sizeof(unsigned), (xdrproc_t) xdr_u_int)) ;
@@ -618,9 +662,24 @@ static int SaveVectC(nx, l)
   return(1);
 }
 
+static int SaveVectS(char **nx)
+{
+  int scount = 0,i;
+  if (nx != NULL) { while ( nx[scount] != NULL) scount++;} 
+  /* save the number of strings */
+  count = scount;
+  assert( xdr_vector(xdrs,(char *) &count,(unsigned)1,(unsigned) sizeof(unsigned), (xdrproc_t) xdr_u_int)) ;
+  /* now save each string */ 
+  for ( i=0; i < scount ; i++) 
+    {
+      if ( SaveVectC( nx[i], strlen(nx[i])+1) == 0 ) return 0;
+    }
+  return 1;
+}
+
 /** save the colormap if necessary **/
 
-int SaveColormap()
+static int SaveColormap(void)
 {
   int m;
   /** If the X window exists we check its colormap **/
