@@ -1,4 +1,12 @@
       subroutine convert(it)
+C     ---------------------------------------------
+C     converts the first argument 
+C     (on top of stack)
+C     it: is the new requested type 
+C     0,1,2,4,11,12,14 : 
+C     (double,char, short,int,uchar, ushort,uint)
+C     ----------------------------------------------
+
       include '../stack.h'
       logical checkrhs,checklhs
       integer iadr,sadr,memused
@@ -39,12 +47,30 @@ c     intn to intm or double
             istk(il)=8
             istk(il+3)=it
             lr=l
-            call tpconv(is1,it,mn,istk(l),-1,istk(lr),-1)
+            inc=-1
+            if (mod(is1,10).gt.(mod(it,10)))  inc=1
+            call tpconv(is1,it,mn,istk(l),inc,istk(lr),inc)
             lstk(top+1)=sadr(lr+memused(it,mn))
          endif
       else
-         err=1
-         call error(53)
+         top=top+1
+         il=iadr(lstk(top))
+         l=sadr(il+4)
+         err=l+1-lstk(bot)
+         if(err.gt.0) then
+            call error(17)
+            return
+         endif
+         istk(il)=1
+         istk(il+1)=1
+         istk(il+2)=1
+         istk(il+3)=0
+         stk(l)=it
+         lstk(top+1)=l+1
+         rhs=2
+         call funnam(ids(1,pt+1),'iconvert',iadr(lstk(top-1)))
+         fun=-1
+         return
       endif
       return
       end
