@@ -7,12 +7,14 @@ c     Copyright INRIA
       include '../stack.h'
 c
 c
-      integer insert,extrac
+      logical	ishm
+      integer insert,extrac, toto, gettype
       data insert/2/,extrac/3/
+      
 
 c     handle recursion
       if(rstk(pt).eq.403.or.rstk(pt).eq.405) goto 10
-      if(rstk(pt).eq.406) goto 50
+      if(rstk(pt).eq.406.or.rstk(pt).eq.407) goto 50
 c
       if (ddt .eq. 4) then
          write(buf(1:4),'(i4)') fin
@@ -31,14 +33,28 @@ c     undefined operations (look for function-defined operation)
 c
 c     extraction
  10   continue
-      call intl_e()
+      toto = gettype(top-1)
+c$$$      print *, ' type de top =', gettype(top), ' type de top-1 =', toto
+      if ( ishm() .and. toto.ne.10 .and. toto.ne.15) then
+         call intehm()
+      else               
+         call intl_e()
+      endif
       return
 c
 c     insertion
  50   continue
-      call intl_i()
+      toto = gettype(top-2)  ! a priori
+c$$$      print *, ' type de top =',   gettype(top), 
+c$$$     $         ' type de top-1 =', gettype(top-1),
+c$$$     $         ' type de top-2 =', toto
+      if ( ishm() .and. toto.ne.10 .and. toto.ne.15) then
+         call intihm()
+      else               
+         call intl_i()
+      endif
       if(err.gt.0) return
-      if(rstk(pt).eq.407) goto 50
+c      if(rstk(pt).eq.407) goto 50
       return
 c
       end
@@ -59,6 +75,3 @@ c
       strpos=0
       return
       end
-
-
-
