@@ -26,7 +26,7 @@ c     --------------------------------
 c     
       fun = 0
 c     
-c        cconc  extrac insert rconc
+c        cconc   insert extrac rconc
       goto(75  ,  95  ,  78   ,76) op
 c     
 c           :  +  -  * /  \  =          '
@@ -217,7 +217,7 @@ c     .     Matlab like []+a=[]
          else
 c     .     []+a=a
             call icopy(4,istk(il2),1,istk(il1),1)
-            call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+            call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
             lstk(top+1)=l1+mn2*(it2+1)
          endif
       elseif (mn2.eq.0) then
@@ -238,16 +238,27 @@ c     .  vector+eye
          go to 41
       elseif (mn2.eq.1) then
 c     .   vector+const
+         err=l1+mn1*(itr+1)-lstk(bot)
+         if(err.gt.0) then
+            call error(17)
+            return
+         endif
          call dadd(mn1,stk(l2),0,stk(l1),1)
-         if(it2+2*it1.eq.1) call dcopy(mn1,stk(l2+mn2),0,stk(l1+mn1),1)
+         if(it2+2*it1.eq.1) call unsfdcopy(mn1,stk(l2+mn2),0,
+     $        stk(l1+mn1),1)
          if(it1*it2.eq.1) call dadd(mn1,stk(l2+mn2),0,stk(l1+mn1),1)
          lstk(top+1)=l1+mn1*(itr+1)
          istk(il1+3)=itr
       elseif (mn1.eq.1) then
 c     .  cst+vector
+         err=l1+mn2*(itr+1)-lstk(bot)
+         if(err.gt.0) then
+            call error(17)
+            return
+         endif
          cstr=stk(l1)
          csti=stk(l1+1)
-         call dcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
+         call unsfdcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
          if(it1.eq.1.and.it2.eq.0) call dset(mn2,0.d0,stk(l1+mn2),1)
          call dadd(mn2,cstr,0,stk(l1),1)
          if(it1.eq.1) call dadd(mn2,csti,0,stk(l1+mn2),1)
@@ -261,9 +272,14 @@ c     .  vector+vector
             call error(8)
             return
          endif
+         err=l1+mn1*(itr+1)-lstk(bot)
+         if(err.gt.0) then
+            call error(17)
+            return
+         endif
          call dadd(mn1,stk(l2),1,stk(l1),1)
          if(it2+2*it1.eq.1) then
-            call dcopy(mn1,stk(l2+mn1),1,stk(l1+mn1),1)
+            call unsfdcopy(mn1,stk(l2+mn1),1,stk(l1+mn1),1)
          endif
          if(it1*it2.eq.1) then
             call dadd(mn1,stk(l2+mn1),1,stk(l1+mn1),1)
@@ -276,7 +292,7 @@ c     addition et soustraction d'un scalaire fois l'identite
  40   sr=stk(l1)
       si=0.0d+0
       if(it1.eq.1) si=stk(l1+1)
-      call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+      call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
       m1=m2
       n1=n2
       m2=it2
@@ -377,7 +393,7 @@ c     .     Matlab like []-a=[]
          else
 c     .     []-a=-a
             call icopy(4,istk(il2),1,istk(il1),1)
-            call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+            call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
             call dscal(mn2*(it2+1),-1.0d0,stk(l1),1)
             lstk(top+1)=l1+mn2*(it2+1)
          endif
@@ -400,7 +416,8 @@ c     .  vector-a*eye
       elseif (mn2.eq.1) then
 c     .   vector-const
          call dadd(mn1,-stk(l2),0,stk(l1),1)
-         if(it2+2*it1.eq.1) call dcopy(mn1,-stk(l2+mn2),0,stk(l1+mn1),1)
+         if(it2+2*it1.eq.1) call unsfdcopy(mn1,-stk(l2+mn2),0,
+     $        stk(l1+mn1),1)
          if(it1*it2.eq.1) call dadd(mn1,-stk(l2+mn2),0,stk(l1+mn1),1)
          lstk(top+1)=l1+mn1*(itr+1)
          istk(il1+3)=itr
@@ -409,7 +426,7 @@ c     .  cst-vector
          cstr=stk(l1)
          csti=stk(l1+1)
          call dscal((it2+1)*mn2,-1.0d0,stk(l2),1)
-         call dcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
+         call unsfdcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
          if(it1.eq.1.and.it2.eq.0) call dset(mn2,0.d0,stk(l1+mn2),1)
          call dadd(mn2,cstr,0,stk(l1),1)
          if(it1.eq.1) call dadd(mn2,csti,0,stk(l1+mn2),1)
@@ -427,7 +444,7 @@ c     .  vector-vector
          if(itr.eq.0) return
          if(it1.eq.0) then
             call dscal (mn1,-1.0d+0,stk(l2+mn1),1)
-            call dcopy(mn1,stk(l2+mn1),1,stk(l1+mn1),1)
+            call unsfdcopy(mn1,stk(l2+mn1),1,stk(l1+mn1),1)
          endif
          if(it1*it2.eq.1) call ddif(mn1,stk(l2+mn1),1,stk(l1+mn1),1)
          lstk(top+1)=l1+mn1*(itr+1)
@@ -439,7 +456,7 @@ c     a*eye-b
  42   sr=stk(l1)
       si=0.0d+0
       if(it1.eq.1) si=stk(l1+1)
- 43   call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+ 43   call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
       call dscal(mn2*(it2+1),-1.0d+0,stk(l1),1)
       mn1=mn2
       m1=m2
@@ -535,7 +552,7 @@ c     .     eye*cst
             istk(il1+2)=n2
             istk(il1+3)=itr
          endif
-         call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+         call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
          it21=it2+2*it1
          if(it21.eq.0) then
 c     .     le scalaire et la matrice sont reel
@@ -552,7 +569,7 @@ c     .     la matrice est reelle, le scalaire est complexe
                call error(17)
                return
             endif
-            call dcopy(mn2,stk(l1),1,stk(l1+mn2),1)
+            call unsfdcopy(mn2,stk(l1),1,stk(l1+mn2),1)
             call dscal(mn2,sr,stk(l1),1)
             call dscal(mn2,si,stk(l1+mn2),1)
          elseif(it21.eq.3) then
@@ -580,7 +597,7 @@ c     .     la matrice est reelle le scalaire est complexe
                call error(17)
                return
             endif
-            call dcopy(mn1,stk(l1),1,stk(l1+mn1),1)
+            call unsfdcopy(mn1,stk(l1),1,stk(l1+mn1),1)
             call dscal(mn1,si,stk(l1+mn1),1)
             call dscal(mn1,sr,stk(l1),1)
             istk(il1+3)=itr
@@ -602,23 +619,27 @@ c     .  matrix*matrix
             return
          endif
          lr=l2+mn2*(it2+1)
-         err=lr+m1*n2*(itr+1)-lstk(bot)
-         if(err.gt.0) then
+c     .  m1*n2 may overflow
+         temp=float(lr)+float(m1)*n2*(itr+1)-lstk(bot)
+         if(temp.gt.0.0d0) then
+            err=int(temp)
             call error(17)
             return
          endif
          if(it1*it2.ne.1) then
-            call dmmul(stk(l1),m1,stk(l2),m2,stk(lr),m1,m1,n1,n2)
-            if(it1.eq.1) call dmmul(stk(l1+mn1),m1,stk(l2),m2
-     $           ,stk(lr+m1*n2),m1,m1,n1,n2)
-            if(it2.eq.1) call dmmul(stk(l1),m1,stk(l2+mn2),m2
-     $           ,stk(lr+m1*n2),m1,m1,n1,n2)
+*           remplacement de dmmul par dgemm (Bruno le 31/10/2001)
+            call dgemm('n','n',m1,n2,n1,1.d0,stk(l1),m1,stk(l2),m2,
+     $           0.d0,stk(lr),m1)
+            if(it1.eq.1) call dgemm('n','n',m1,n2,n1,1.d0,stk(l1+mn1),
+     $           m1,stk(l2),m2,0.d0,stk(lr+m1*n2),m1)
+            if(it2.eq.1) call dgemm('n','n',m1,n2,n1,1.d0,stk(l1),m1,
+     $           stk(l2+mn2),m2,0.d0,stk(lr+m1*n2),m1)      
          else
 c     .     a et a2 sont complexes
             call wmmul(stk(l1),stk(l1+mn1),m1,stk(l2),stk(l2
      $           +mn2),m2,stk(lr),stk(lr+m1*n2),m1,m1,n1,n2)
          endif
-         call dcopy(m1*n2*(itr+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(m1*n2*(itr+1),stk(lr),1,stk(l1),1)
          istk(il1+2)=n2
          istk(il1+3)=itr
          lstk(top+1)=l1+m1*n2*(itr+1)
@@ -673,11 +694,13 @@ c     .  a/[] or []/a
             call error(14)
             return
          endif
-         if (m2 .eq. n2) fun = 1
-         if (m2 .ne. n2) fun = 4
-         fin = -1
          top = top+1
-         rhs = 2
+         rhs = 2        
+         call intslash('slash')
+         if (fin.ge.0) call putlhsvar()
+c         if (m2 .eq. n2) fun = 1
+c         if (m2 .ne. n2) fun = 4
+c         fin = -1
       else
 c     .  vector / cst
          if(m2.lt.0.and.mn1.ne.1) then 
@@ -771,11 +794,13 @@ c     .  a\[] or []\a
             call error(14)
             return
          endif
-         if (m1 .eq. n1) fun = 1
-         if (m1 .ne. n1) fun = 4
          top = top+1
-         fin = -2
-         rhs = 2
+         rhs = 2 
+         call intbackslash('backslash')
+         if (fin.ge.0) call putlhsvar()
+c         if (m1 .eq. n1) fun = 1
+c         if (m1 .ne. n1) fun = 4
+c         fin = -2
       else
 c     .  cst \ vector
          if(m1.lt.0.and.mn2.ne.1) then 
@@ -802,17 +827,17 @@ c     .     real \ real
 c     .     real \ complex = complex/real
             call wdrdiv(stk(l2),stk(l2+mn2),1,sr,0,stk(l2)
      $           ,stk(l2+mn2),1,mn2,ierr)
-            call dcopy(2*mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy(2*mn2,stk(l2),1,stk(l1),1)
          elseif(it21.eq.2) then 
 c     .     complex \ real =real / complex
             si=stk(l1+1)
-            call dcopy(mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy(mn2,stk(l2),1,stk(l1),1)
             call dwrdiv(stk(l1),1,sr,si,0,stk(l1),stk(l1+mn2),1
      $           ,mn2,ierr)
          elseif(it21.eq.3) then  
 c     .     complex \ complex
             si=stk(l1+1)
-            call dcopy(2*mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy(2*mn2,stk(l2),1,stk(l1),1)
             call wwrdiv(stk(l1),stk(l1+mn2),1,sr,si,0,stk(l1)
      $           ,stk(l1+mn2),1,mn2,ierr)
          endif
@@ -898,25 +923,25 @@ c
       if(it2.eq.0) then
          if(it1.eq.0) then
             call ddpow1(mn,stk(l1),inc1,stk(l2),inc2,
-     $           stk(lw),stk(lw+mn),1,err,itr)
+     $           stk(lw),stk(lw+mn),1,ierr,itr)
          else
             call wdpow1(mn,stk(l1),stk(l1+mn1),inc1,stk(l2),inc2,
-     $           stk(lw),stk(lw+mn),1,err)
+     $           stk(lw),stk(lw+mn),1,ierr)
          endif
       else
          if(it1.eq.0) then
             call dwpow1(mn,stk(l1),inc1,stk(l2),stk(l2+mn2),inc2,
-     &           stk(lw),stk(lw+mn),1,err)
+     &           stk(lw),stk(lw+mn),1,ierr)
          else
             call wwpow1(mn,stk(l1),stk(l1+mn1),inc1,stk(l2),stk(l2+mn2),
-     &           inc2,stk(lw),stk(lw+mn),1,err)
+     &           inc2,stk(lw),stk(lw+mn),1,ierr)
          endif
       endif
-      if(err.eq.1) then
+      if(ierr.eq.1) then
          call error(30)
          return
       endif
-      if(err.eq.2) then
+      if(ierr.eq.2) then
          if(ieee.eq.0) then
             call error(27)
             return
@@ -928,7 +953,7 @@ c
       istk(il1+1)=m
       istk(il1+2)=n
       istk(il1+3)=itr
-      call dcopy(mn*(itr+1),stk(lw),1,stk(l1),1)
+      call unsfdcopy(mn*(itr+1),stk(lw),1,stk(l1),1)
       lstk(top+1)=l1+mn*(itr+1)
       return
       end
@@ -1019,7 +1044,7 @@ c     scalar^matrix treated as scalar.^matrix (see matxpow)
          istk(il1+1)=m2
          istk(il1+2)=n2
          istk(il1+3)=itr
-         call dcopy(mn2*(itr+1),stk(lw),1,stk(l1),1)
+         call unsfdcopy(mn2*(itr+1),stk(lw),1,stk(l1),1)
          lstk(top+1)=l1+mn2*(itr+1)
          return
       endif
@@ -1098,13 +1123,12 @@ c     .     vect^scalar treated as  vect.^scalar (see matxpow)
       endif
 c     
       if (nexp.le.0) then
-         fun=10
-         fin=1
          rhs=1
-         call matlu
-         if(err.gt.0) return
+         call intinv('pow')
+         call putlhsvar()
+c         call matlu
+         if(err.gt.0.or.err1.gt.0) return
          nexp=-nexp
-         fun=0
       endif
       l2=l1+mn1*(it1+1)
 c     
@@ -1117,13 +1141,13 @@ c
       lstk(top+1)=l1+mn1*(itr+1)
       istk(il1+3)=itr
 c     
-      call dcopy(mn1*(itr+1),stk(l1),1,stk(l2),1)
+      call unsfdcopy(mn1*(itr+1),stk(l1),1,stk(l2),1)
       if(it1.eq.1) goto 35
 c     la matrice est reelle
       do 34 kexp=2,nexp
          do 33 j=1,n1
             ls=l1+(j-1)*n1
-            call dcopy(n1,stk(ls),1,stk(l3),1)
+            call unsfdcopy(n1,stk(ls),1,stk(l3),1)
             do 32 i=1,n1
                ls=l2+(i-1)
                ll=l1+(i-1)+(j-1)*n1
@@ -1138,8 +1162,8 @@ c     la matrice est complexe
       do 38 kexp=2,nexp
          do 37 j=1,n1
             ls=l1+(j-1)*n1
-            call dcopy(n1,stk(ls),1,stk(l3),1)
-            call dcopy(n1,stk(ls+mn1),1,stk(l3+n1),1)
+            call unsfdcopy(n1,stk(ls),1,stk(l3),1)
+            call unsfdcopy(n1,stk(ls+mn1),1,stk(l3+n1),1)
             do 36 i=1,n1
                ls=l2+(i-1)
                ll=l1+(i-1)+(j-1)*n1
@@ -1246,18 +1270,46 @@ c     : in compiled for, next line to differentiate from extraction
          if(istk(pstk(pt)).eq.16) goto  54
       endif
 c
+      if (st .eq. 0.0d+0) then
+         istk(il1+1)=1
+         istk(il1+1)=0
+         istk(il1+2)=0
+         istk(il1+3)=0
+         lstk(top+1)=l1
+         return
+      endif
+
+      if(isanan(e1).eq.1.or.isanan(st).eq.1.or.isanan(e2).eq.1) then
+         stk(l1)=e1+st+e2
+         istk(il1+1)=1
+         istk(il1+2)=1
+         istk(il1+3)=0
+         lstk(top+1)=l1+1
+         return
+      endif
+
 c     floating point used to avoid integer overflow
       e1r=dble(l1) + max(3.0d0,(e2-e1)/st) - dble(lstk(bot))
+      if(isanan(e1r).eq.1) then
+        stk(l1)=eiR
+         istk(il1+1)=1
+         istk(il1+2)=1
+         istk(il1+3)=0
+         lstk(top+1)=l1+1
+       return
+      endif
+
       if (e1r .gt. 0.0d0) then
          err=e1r
          call error(17)
          return
       endif
 c
+      e1r=2.0d0*max(abs(e1),abs(e2))*dlamch('p')
       n = 0
       l=l1
  52   if (st*(stk(l)-e2).gt.0.0d+0) then
-         if (abs(stk(l)-e2).lt.abs(st*dlamch('p'))) n=n+1
+         if (abs(stk(l)-e2).lt.e1r) n=n+1
          go to 53
       endif
       n = n+1
@@ -1273,12 +1325,19 @@ c
       return
 c     
 c     for clause
- 54   stk(l1) = e1
-      stk(l1+1) = st
-      stk(l1+2) = e2
-      istk(il1+1)=-3
-      istk(il1+2)=-1
-      lstk(top+1)=l1+3
+ 54   if(isanan(e1).eq.1.or.isanan(st).eq.1.or.isanan(e2).eq.1) then
+         stk(l1) = e1+st+e2
+         istk(il1+1)=1
+         istk(il1+2)=1
+         lstk(top+1)=l1+3
+      else
+         stk(l1) = e1
+         stk(l1+1) = st
+         stk(l1+2) = e2
+         istk(il1+1)=-3
+         istk(il1+2)=-1
+         lstk(top+1)=l1+3
+      endif
       return
       end
 
@@ -1367,7 +1426,7 @@ c     .     eye.*cst
             istk(il1+2)=n2
             istk(il1+3)=itr
          endif
-         call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+         call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
          it21=it2+2*it1
          if(it21.eq.0) then
 c     .     vector and cst are real
@@ -1384,7 +1443,7 @@ c     .     real vector, complex cst
                call error(17)
                return
             endif
-            call dcopy(mn2,stk(l1),1,stk(l1+mn2),1)
+            call unsfdcopy(mn2,stk(l1),1,stk(l1+mn2),1)
             call dscal(mn2,sr,stk(l1),1)
             call dscal(mn2,si,stk(l1+mn2),1)
          elseif(it21.eq.3) then
@@ -1412,7 +1471,7 @@ c     .     real vector, complex cst
                call error(17)
                return
             endif
-            call dcopy(mn1,stk(l1),1,stk(l1+mn1),1)
+            call unsfdcopy(mn1,stk(l1),1,stk(l1+mn1),1)
             call dscal(mn1,si,stk(l1+mn1),1)
             call dscal(mn1,sr,stk(l1),1)
             istk(il1+3)=itr
@@ -1435,7 +1494,7 @@ c     .  vector.*vector
             if(it2.eq.1) call dvmul(mn1,stk(l1),i1,stk(l2
      $           +mn2),i2)
             call dvmul(mn1,stk(l2),i2,stk(l1),i1)
-            if(it2.eq.1) call dcopy(mn1,stk(l2+mn2),i2
+            if(it2.eq.1) call unsfdcopy(mn1,stk(l2+mn2),i2
      $           ,stk(l1+mn1),i1)
          else
             call wvmul(mn1,stk(l2),stk(l2+mn2),i2,stk(l1)
@@ -1555,19 +1614,19 @@ c     .     real ./ real
             call ddrdiv(sr,0,stk(l2),1,stk(l1),1,mn2,ierr)
          elseif(it21.eq.1) then
 c     .     real ./ complex
-            call dcopy(2*mn2,stk(l2),1, stk(l1),1)
+            call unsfdcopy(2*mn2,stk(l2),1, stk(l1),1)
             call dwrdiv(sr,0,stk(l1),stk(l1+mn2),1,stk(l1)
      $           ,stk(l1+mn2),1,mn2,ierr)
          elseif(it21.eq.2) then 
 c     .     complex ./ real
             si=stk(l1+mn1)
-            call dcopy(mn2,stk(l2),1, stk(l1),1)
+            call unsfdcopy(mn2,stk(l2),1, stk(l1),1)
             call wdrdiv(sr,si,0,stk(l1),1,stk(l1),stk(l1+mn2)
      $           ,1,mn2,ierr)
          elseif(it21.eq.3) then 
 c     .     complex ./ complex
             si=stk(l1+mn1)
-            call dcopy(2*mn2,stk(l2),1, stk(l1),1)
+            call unsfdcopy(2*mn2,stk(l2),1, stk(l1),1)
             call wwrdiv(sr,si,0,stk(l1),stk(l1+mn2),1,stk(l1)
      $           ,stk(l1+mn2),1,mn2,ierr)
          endif
@@ -1581,7 +1640,7 @@ c     .     real ./ real
 c     .     real ./ complex
             call dwrdiv(stk(l1),1,stk(l2),stk(l2+mn1),1
      $           ,stk(l1),stk(l2),1,mn2,ierr)
-            call dcopy(mn2,stk(l2),1,stk(l1+mn2),1)
+            call unsfdcopy(mn2,stk(l2),1,stk(l1+mn2),1)
          elseif(it21.eq.2) then 
 c     .     complex ./ real
             call wdrdiv(stk(l1),stk(l1+mn1),1,stk(l2),1
@@ -1696,17 +1755,17 @@ c     .     real \ real
 c     .     real \ complex = complex/real
             call wdrdiv(stk(l2),stk(l2+mn2),1,sr,0,stk(l2)
      $           ,stk(l2+mn2),1,mn2,ierr)
-            call dcopy(2*mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy(2*mn2,stk(l2),1,stk(l1),1)
          elseif(it21.eq.2) then 
 c     .     complex \ real =real / complex
             si=stk(l1+1)
-            call dcopy(mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy(mn2,stk(l2),1,stk(l1),1)
             call dwrdiv(stk(l1),1,sr,si,0,stk(l1),stk(l1+mn2),1
      $           ,mn2,ierr)
          elseif(it21.eq.3) then  
 c     .     complex \ complex
             si=stk(l1+1)
-            call dcopy(2*mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy(2*mn2,stk(l2),1,stk(l1),1)
             call wwrdiv(stk(l1),stk(l1+mn2),1,sr,si,0,stk(l1)
      $           ,stk(l1+mn2),1,mn2,ierr)
          endif
@@ -1745,7 +1804,7 @@ c     .     complex .\ real
 c     .     real .\ complex = complex /. real
             call wdrdiv(stk(l2),stk(l2+mn2),1,stk(l1),1
      $           ,stk(l1),stk(l2),1,mn1,ierr)
-            call dcopy(mn1,stk(l2),1,stk(l1+mn1),1)
+            call unsfdcopy(mn1,stk(l2),1,stk(l1+mn1),1)
          elseif(it21.eq.3) then 
 c     .     complex .\ complex
             call wwrdiv(stk(l2),stk(l2+mn2),1,stk(l1)
@@ -1799,7 +1858,7 @@ c
             call error(17)
             return
          endif
-         call dcopy(vol,stk(l1),1,stk(ll),1)
+         call unsfdcopy(vol,stk(l1),1,stk(ll),1)
          call mtran(stk(ll),m1,stk(l1),n1,m1,n1)
          if(it1.eq.1) then
             call mtran(stk(ll+mn1),m1,stk(l1+mn1),n1,m1,n1)
@@ -1843,7 +1902,7 @@ c
             call error(17)
             return
          endif
-         call dcopy(vol,stk(l1),1,stk(ll),1)
+         call unsfdcopy(vol,stk(l1),1,stk(ll),1)
          call mtran(stk(ll),m1,stk(l1),n1,m1,n1)
          if(it1.eq.1) then
             call mtran(stk(ll+mn1),m1,stk(l1+mn1),n1,m1,n1)
@@ -1894,7 +1953,7 @@ c
       if(m2.eq.0) then
          return
       elseif(m1.eq.0)then
-         call dcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1))
+         call unsfdcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1))
      $        ,1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+lstk(top+2)-lstk(top+1)
          return
@@ -1904,7 +1963,7 @@ c
       endif
 c     
       if(itr.eq.0) then
-         call dcopy(mn2,stk(l2),1,stk(l1+mn1),1)
+         call unsfdcopy(mn2,stk(l2),1,stk(l1+mn1),1)
       else
          lw=l1+(itr+1)*(mn1+mn2)
          if(lw.gt.l2) then
@@ -1913,20 +1972,20 @@ c
                call error(17)
                return
             endif
-            call dcopy(mn2*(it2+1),stk(l2),-1,stk(lw),-1)
+            call unsfdcopy(mn2*(it2+1),stk(l2),-1,stk(lw),-1)
             l2=lw
          endif
-         if(it1.eq.1) call dcopy(mn1,stk(l1+mn1),-1,stk(l1
+         if(it1.eq.1) call unsfdcopy(mn1,stk(l1+mn1),-1,stk(l1
      $        +mn1+mn2),-1)
-         call dcopy(mn2,stk(l2),1,stk(l1+mn1),1)
+         call unsfdcopy(mn2,stk(l2),1,stk(l1+mn1),1)
          if(it1.eq.0) then
             call dset(mn1,0.0d+0,stk(l1+mn1+mn2),1)
-            call dcopy(mn2,stk(l2+mn2),1,stk(l1+2*mn1+mn2),1)
+            call unsfdcopy(mn2,stk(l2+mn2),1,stk(l1+2*mn1+mn2),1)
          endif
          if(it2.eq.0) then
             call dset(mn2,0.0d+0,stk(l1+2*mn1+mn2),1)
          else
-            call dcopy(mn2,stk(l2+mn2),1,stk(l1+2*mn1+mn2),1)
+            call unsfdcopy(mn2,stk(l2+mn2),1,stk(l1+2*mn1+mn2),1)
          endif
       endif
       n=n1+n2
@@ -1978,7 +2037,7 @@ c     .  [a;[]]
          return
       elseif(n1.eq.0)then
 c     .  [[];b]
-         call dcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1))
+         call unsfdcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1))
      $        ,1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+lstk(top+2)-lstk(top+1)
          return
@@ -1989,7 +2048,7 @@ c     .  [[];b]
       m=m1+m2
       mn=m*n1
       if(n1.eq.1.and.itr.eq.0) then
-         call dcopy(mn2,stk(l2),1,stk(l1+mn1),1)
+         call unsfdcopy(mn2,stk(l2),1,stk(l1+mn1),1)
          istk(il1+1)=m
          istk(il1+3)=itr
          lstk(top+1)=l1+mn*(itr+1)
@@ -2003,8 +2062,8 @@ c      lw1=l1+(itr+1)*mn
          call error(17)
          return
       endif
-      call dcopy(mn2*(it2+1),stk(l2),1,stk(lw2),1)
-      call dcopy(mn1*(it1+1),stk(l1),1,stk(lw1),1)
+      call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(lw2),1)
+      call unsfdcopy(mn1*(it1+1),stk(l1),1,stk(lw1),1)
 c     
       if(itr.eq.1) call dset(mn,0.0d+0,stk(l1+(mn1+mn2)),1)
       call dmcopy(stk(lw1),m1,stk(l1),m,m1,n1)
@@ -2065,13 +2124,27 @@ c     implied polynomials vector extraction
 
       if(mn2.eq.0) then 
 c     .  arg2=[]
-         il1=iadr(lstk(top))
-         istk(il1)=1
-         istk(il1+1)=0
-         istk(il1+2)=0
-         istk(il1+3)=0
-         lstk(top+1)=sadr(il1+4)
+         if(m1.le.0) then
+c            arg2(:) or arg2([])
+            il1=iadr(lstk(top))
+            istk(il1)=1
+            istk(il1+1)=0
+            istk(il1+2)=0
+            istk(il1+3)=0
+            lstk(top+1)=sadr(il1+4)
+         else
+c     should be an error but a lot of code uses this feature
+c     we will change it just after 3.1 release  see also matext2
+c            call error(21)
+            il1=iadr(lstk(top))
+            istk(il1)=1
+            istk(il1+1)=0
+            istk(il1+2)=0
+            istk(il1+3)=0
+            lstk(top+1)=sadr(il1+4)
+         endif
          return
+
       elseif(m2.lt.0) then
 c     .  arg2=eye
          call error(14)
@@ -2083,8 +2156,9 @@ c     .  arg2(:), just reshape to column vector
          istk(il1+1)=mn2
          istk(il1+2)=1
          istk(il1+3)=istk(il2+3)
-         call dcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
-         lstk(top+1)=sadr(il1+4)+mn2*(it2+1)
+         l1=sadr(il1+4)
+         call unsfdcopy(mn2*(it2+1),stk(l2),1,stk(l1),1)
+         lstk(top+1)=l1+mn2*(it2+1)
          return
       endif
 c     check and convert indices variable
@@ -2141,7 +2215,7 @@ c     form resulting variable
       istk(il1+1)=m
       istk(il1+2)=n
       istk(il1+3)=it2
-      if(lr.ne.l1) call dcopy(mi*(it2+1),stk(lr),1,stk(l1),1)
+      if(lr.ne.l1) call unsfdcopy(mi*(it2+1),stk(lr),1,stk(l1),1)
       lstk(top+1)=l1+mi*(it2+1)
       return
       end
@@ -2193,12 +2267,26 @@ c     arg3(arg1,arg2)
       endif
       if(mn3.eq.0) then 
 c     .  arg3=[]
-         il1=iadr(lstk(top))
-         istk(il1)=1
-         istk(il1+1)=0
-         istk(il1+2)=0
-         istk(il1+3)=0
-         lstk(top+1)=sadr(il1+4)
+         if(m1.le.0.or.m2.le.0) then
+            il1=iadr(lstk(top))
+            istk(il1)=1
+            istk(il1+1)=0
+            istk(il1+2)=0
+            istk(il1+3)=0
+            lstk(top+1)=sadr(il1+4)
+         else
+c     should be an error but a lot of code uses this feature
+c     we will change it just after 3.1 release see also matext1
+
+c           call error(21)
+            il1=iadr(lstk(top))
+            istk(il1)=1
+            istk(il1+1)=0
+            istk(il1+2)=0
+            istk(il1+3)=0
+            lstk(top+1)=sadr(il1+4)
+
+         endif
          return
       elseif(m3.lt.0) then
 c     .arg3=eye
@@ -2260,7 +2348,7 @@ c     form the resulting variable
       istk(il1+1)=mi
       istk(il1+2)=nj
       istk(il1+3)=it3
-      if(lr.ne.l1) call dcopy(mn*(it3+1),stk(lr),1,stk(l1),1)
+      if(lr.ne.l1) call unsfdcopy(mn*(it3+1),stk(lr),1,stk(l1),1)
       lstk(top+1)=l1+mn*(it3+1)
       return
       end
@@ -2271,6 +2359,7 @@ c     A(i)=B
 c     
 c     Copyright INRIA
       include '../stack.h'
+      common /mtlbc/ mmode
 c     
       logical isany
       integer top0
@@ -2293,6 +2382,7 @@ c
 c     
       il2=iadr(lstk(top))
       if(istk(il2).lt.0) il2=iadr(istk(il2+1))
+  
       m2=istk(il2+1)
       n2=istk(il2+2)
       it2=istk(il2+3)
@@ -2303,13 +2393,16 @@ c
       il1=iadr(lstk(top))
       ilrs=il1
       if(istk(il1).lt.0) il1=iadr(istk(il1+1))
+      if (istk(il1).eq.10.or.istk(il1).eq.15) then
+         top=top0
+         fin=-fin
+         return
+      endif
       m1=istk(il1+1)
       n1=istk(il1+2)
       it1=istk(il1+3)
       l1=sadr(il1+4)
       mn1=m1*n1
-
-
 c     arg3(arg1)=arg2
 c     
       if (istk(il2)*istk(il1).eq.0) then
@@ -2330,7 +2423,7 @@ c     .    arg3(:)=[]
 c     .     arg3([])=[]  --> arg3
             call icopy(4,istk(il3),1,istk(ilrs),1)
             l=sadr(ilrs+4)
-            call dcopy(mn3*(it3+1),stk(l3),1,stk(l),1)
+            call unsfdcopy(mn3*(it3+1),stk(l3),1,stk(l),1)
             lstk(top+1)=l+mn3*(it3+1)
             return
          else
@@ -2340,7 +2433,7 @@ c     .     arg3(arg1)=[]
 c     .           arg3([])=[]  --> arg3
                   call icopy(4,istk(il3),1,istk(ilrs),1)
                   l=sadr(ilrs+4)
-                  call dcopy(mn3*(it3+1),stk(l3),1,stk(l),1)
+                  call unsfdcopy(mn3*(it3+1),stk(l3),1,stk(l),1)
                   lstk(top+1)=l+mn3*(it3+1)
                   return
                endif
@@ -2368,7 +2461,16 @@ c     .  arg3(:)=arg2
             istk(ilrs+2)=n3
             istk(ilrs+3)=it2
             l1=sadr(ilrs+4)
-            call dcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
+            call unsfdcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
+            lstk(top+1)=l1+mn2*(it2+1)
+            return
+         elseif(mn3.eq.0) then
+            istk(ilrs)=1
+            istk(ilrs+1)=m2
+            istk(ilrs+2)=n2
+            istk(ilrs+3)=it2
+            l1=sadr(ilrs+4)
+            call unsfdcopy((it2+1)*mn2,stk(l2),1,stk(l1),1)
             lstk(top+1)=l1+mn2*(it2+1)
             return
          elseif(mn2.eq.1) then
@@ -2394,7 +2496,7 @@ c     .  arg3([])=arg2
 c     .  arg3([])=c  --> arg3
             call icopy(4,istk(il3),1,istk(ilrs),1)
             l=sadr(ilrs+4)
-            call dcopy(mn3*(it3+1),stk(l3),1,stk(l),1)
+            call unsfdcopy(mn3*(it3+1),stk(l3),1,stk(l),1)
             lstk(top+1)=l+mn3*(it3+1)
             return
          else
@@ -2492,10 +2594,15 @@ c     write arg2 in r
 c     
       if(lr.ne.l3) then
          l1=sadr(ilrs+4)
-         call dcopy(mnr*(itr+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(mnr*(itr+1),stk(lr),1,stk(l1),1)
          istk(ilrs)=1
+         if(mmode.eq.1.and.nr.eq.1.and.m3.eq.0) then
+         istk(ilrs+1)=nr
+         istk(ilrs+2)=mr
+         else
          istk(ilrs+1)=mr
          istk(ilrs+2)=nr
+         endif
          istk(ilrs+3)=itr
          lstk(top+1)=l1+mnr*(itr+1)
       else
@@ -2551,7 +2658,7 @@ c     form resulting variable
       istk(ilrs+1)=m
       istk(ilrs+2)=n
       istk(ilrs+3)=it2
-      if(lr.ne.l1) call dcopy(mi*(it2+1),stk(lr),1,stk(l1),1)
+      if(lr.ne.l1) call unsfdcopy(mi*(it2+1),stk(lr),1,stk(l1),1)
       lstk(top+1)=l1+mi*(it2+1)
       return
 
@@ -2628,7 +2735,7 @@ c     .    arg4(:,:)=[] -->[]
 c     .     arg4([],arg2)=[],  arg4(arg1,[])=[] --> arg4
             call icopy(4,istk(il4),1,istk(ilrs),1)
             l=sadr(ilrs+4)
-            call dcopy(mn4*(it4+1),stk(l4),1,stk(l),1)
+            call unsfdcopy(mn4*(it4+1),stk(l4),1,stk(l),1)
             lstk(top+1)=l+mn4*(it4+1)
             return
          elseif(m2.eq.-1) then
@@ -2664,8 +2771,8 @@ c     .     arg4(arg1,arg2)=[]
             if(err.gt.0) return
             if(nj.eq.0) then
 c     .        arg4(arg1,1:n4)=[] 
-               lw2=lw
                call indxgc(il1,m4,ili,mi,mxi,lw)
+               lw2=lw
                if(err.gt.0) return
 
                if(mi.eq.0) then
@@ -2729,7 +2836,7 @@ c     .     reshape arg3 according to arg4
             istk(ilrs+2)=n4
             istk(ilrs+3)=it3
             l1=sadr(ilrs+4)
-            call dcopy((it3+1)*mn4,stk(l3),1,stk(l1),1)
+            call unsfdcopy((it3+1)*mn4,stk(l3),1,stk(l1),1)
             lstk(top+1)=l1+mn4*(it3+1)
             return
          elseif(mn3.eq.1) then
@@ -2747,8 +2854,36 @@ c     .     set all elements of arg4 to arg3
             call error(15)
             return
          endif
-      endif
 
+      endif
+      init4=0
+      if(m1.eq.-1.and.m4.eq.0) then
+c     .  arg4(:,i)=arg3
+         m3=m3*n3
+         n3=1
+         n4=1
+         m4=m3
+         init4=1
+         
+      elseif(m2.eq.-1.and.m4.eq.0) then
+c     .  arg4(i,:)=arg3
+         n3=m3*n3
+         m3=1
+         m4=1
+         n4=n3
+         init4=1
+      endif
+      if(init4.eq.1) then
+         err=lw-lstk(bot)
+         if(err.gt.0) then
+            call error(17)
+            return
+         endif
+         mn4=m4*n4
+         l4=lw
+         lw=l4+ mn4
+         call dset(mn4,0.0d0,stk(l4),1)
+      endif
       call indxg(il1,m4,ili,mi,mxi,lw,1)
       if(err.gt.0) return
       call indxg(il2,n4,ilj,mj,mxj,lw,1)
@@ -2760,7 +2895,7 @@ c     .  sizes of arg1 or arg2 dont agree with arg3 sizes
             if(mi.eq.0.or.mj.eq.0) then
                call icopy(4,istk(il4),1,istk(ilrs),1)
                l=sadr(ilrs+4)
-               call dcopy(mn4*(it4+1),stk(l4),1,stk(l),1)
+               call unsfdcopy(mn4*(it4+1),stk(l4),1,stk(l),1)
                lstk(top+1)=l+mn4*(it4+1)
                return
             endif
@@ -2817,9 +2952,9 @@ c     copy arg3 elements in r
  114     continue
  115  continue
 c     
-      if(lr.ne.l4) then
+      if(lr.ne.l4.or.init4.ne.0) then
          l1=sadr(ilrs+4)
-         call dcopy(mnr*(itr+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(mnr*(itr+1),stk(lr),1,stk(l1),1)
          istk(ilrs)=1
          istk(ilrs+1)=mr
          istk(ilrs+2)=nr
@@ -2876,7 +3011,7 @@ c     form the resulting variable
       istk(ilrs+2)=nj
       istk(ilrs+3)=it3
       l1=sadr(ilrs+4)
-      if(lr.ne.l1) call dcopy(mn*(it3+1),stk(lr),1,stk(l1),1)
+      if(lr.ne.l1) call unsfdcopy(mn*(it3+1),stk(lr),1,stk(l1),1)
       lstk(top+1)=l1+mn*(it3+1)
       return
 
@@ -3037,8 +3172,8 @@ c     .  eye op b
             top=top0
             return
          endif
-         itrue=1
-         if(op.eq.less+great) itrue=0
+c         itrue=1
+c         if(op.eq.less+great) itrue=0
          istk(il1)=4
          do 131 i=0,mn1-1
             e1r=stk(l1+i)
@@ -3047,11 +3182,22 @@ c     .  eye op b
             e2i=0.0d+0
             if(it1.eq.1) e1i=stk(l1+mn1+i)
             if(it2.eq.1) e2i=stk(l2+mn2+i)
-            if(e1r.eq.e2r.and.e1i.eq.e2i) then
-               istk(il1+3+i)=itrue
+            call idcmp(e1r,e2r,1,ir,op)
+            call idcmp(e1i,e2i,1,ii,op)
+            if (op.eq.less+great) then
+               if(ir.eq.1.or.ii.eq.1) then
+                  istk(il1+3+i)=1
+               else
+                  istk(il1+3+i)=0
+               endif
             else
-               istk(il1+3+i)=1-itrue
+               if(ir.eq.1.and.ii.eq.1) then
+                  istk(il1+3+i)=1
+               else
+                  istk(il1+3+i)=0
+               endif
             endif
+
  131     continue
          lstk(top+1)=sadr(il1+3+mn1)
       else
@@ -3069,6 +3215,10 @@ c     .  eye op b
             endif
             return
          endif
+         if (.true.) then 
+c           add explicit nan tests when requested 
+            call idcmp(stk(l1),stk(l2),mn1,istk(il1+3),op)
+         else
          do 132 i=0,mn1-1
             e1=stk(l1+i)
             e2=stk(l2+i)
@@ -3090,6 +3240,7 @@ c     for vc++ we add an explicit test for nan
                istk(il1+3+i)=0
             endif
  132     continue
+         endif
          lstk(top+1)=sadr(il1+3+mn1)
       endif
       return
@@ -3155,7 +3306,7 @@ c
  10         continue
             lstk(top+1)=sadr(il1+3+mn1)
          endif
-      elseif(fin.eq.et) then
+      else
          if(mn1.eq.0.or.mn2.eq.0) then
             istk(il1)=1
             istk(il1+1)=0
@@ -3176,48 +3327,31 @@ c
          else
             i2=1
          endif
-         istk(il1)=4
-         do 20 i=0,mn1-1
-            e1=stk(l1+i)
-            e2=stk(l2+i)
-            if(e1.ne.0.0d0.and.e2.ne.0.0d0) then
-               istk(il1+3+i)=1
-            else
-               istk(il1+3+i)=0
-            endif
- 20      continue
-         lstk(top+1)=sadr(il1+3+mn1)
-         return
-      elseif(fin.eq.ou) then
-         if(mn1.eq.0.or.mn2.eq.0) then
-            istk(il1)=1
-            istk(il1+1)=0
-            istk(il1+2)=0
-            istk(il1+3)=0
-            lstk(top+1)=sadr(il1+4)
+         if(mn1.ne.mn2) then
+            call error(60)
             return
          endif
-         if(mn1.eq.1) then
-            i1=0
-            mn1=mn2
+         if(fin.eq.ou) then
+            do 30 i=0,mn1-1
+               e1=stk(l1+i*i1)
+               e2=stk(l2+i*i2)
+               if(e1.ne.0.0d0.or.e2.ne.0.0d0) then
+                  istk(il1+3+i)=1
+               else
+                  istk(il1+3+i)=0
+               endif
+ 30         continue
          else
-            i1=1
+            do 31 i=0,mn1-1
+               e1=stk(l1+i*i1)
+               e2=stk(l2+i*i2)
+               if(e1.ne.0.0d0.and.e2.ne.0.0d0) then
+                  istk(il1+3+i)=1
+               else
+                  istk(il1+3+i)=0
+               endif
+ 31         continue
          endif
-         if(mn2.eq.1) then
-            i2=0
-            mn2=mn1
-         else
-            i2=1
-         endif
-         do 30 i=0,mn1-1
-            e1=stk(l1+i)
-            e2=stk(l2+i)
-            if(e1.ne.0.0d0.or.e2.ne.0.0d0) then
-               istk(il1+3+i)=1
-            else
-               istk(il1+3+i)=0
-            endif
- 30      continue
          istk(il1)=4
          istk(il1+1)=max(m1,m2)
          istk(il1+2)=max(n1,n2)
