@@ -178,10 +178,16 @@ c     formal variable
          return
       endif
       n=istk(il+5)-1
+      if(n.gt.4) call msgs(109,0)
       do 12 i=1,4
          id(i)=blank
          if(i.le.n) id(i)=istk(il+5+i)
  12   continue
+      if(abs(id(1)).lt.10.or.abs(id(1)).ge.blank) then
+         err=2
+         call error(116)
+         return
+      endif
 c
 c     first argument
       top=top-1
@@ -190,6 +196,7 @@ c     first argument
       it1=istk(il1+3)
       l1=sadr(il1+4)
       mn1=m1*n1
+      itr=it1
 
       if(.not.roots) goto 17
  13   if(mn1.eq.1.or.m1.ne.n1) goto 14
@@ -204,22 +211,19 @@ c     necessary while matdsr and matdsc cannot be called by reference
             return
          endif
          call icopy(4,istk(il1),1,istk(ilr),1)
-         call dcopy(mn1*(it1+1),stk(l1),1,stk(lr),1)
+         call unsfdcopy(mn1*(it1+1),stk(l1),1,stk(lr),1)
          lstk(top+1)=lr+mn1*(it1+1)
       endif
-      fin=3
       rhs=1
-      if(it1.eq.0) then
-         call matdsr
-      else
-         call matdsc
-      endif
+      call inteig('spec')
+      call putlhsvar
       if(err.gt.0) return
       il1=iadr(lstk(top))
       l1=sadr(il1+4)
       mn1=m1
       m1=1
       n1=mn1
+      itr=min(it1,istk(il1+3))
       it1=istk(il1+3)
       lw=l1+mn1*(it1+1)
       ref=.false.
@@ -242,17 +246,17 @@ c     polynome defini par ses racines
             call error(17)
             return
          endif
-         call dcopy(mn1*(it1+1),stk(l1),1,stk(lr),1)
+         call unsfdcopy(mn1*(it1+1),stk(l1),1,stk(lr),1)
          l1=lr
       endif
       istk(ilr)=2
       istk(ilr+1)=1
       istk(ilr+2)=1
-      istk(ilr+3)=it1
+      istk(ilr+3)=itr
       call icopy(4,id,1,istk(ilr+4),1)
       istk(ild)=1
       istk(ild+1)=mn1+2
-      lstk(top+1)=lc+(mn1+1)*(it1+1)
+      lstk(top+1)=lc+(mn1+1)*(itr+1)
 
       if(it1.eq.0) then
          call dprxc(mn1,stk(l1),stk(lc))
@@ -273,22 +277,21 @@ c     polynome defini par ses coefficients
          call error(53)
          return
       endif
+      if(mn1.eq.0) return
+
       if(n1.ne.1.and.m1.ne.1) then
          err=1
          call error(89)
          return
       endif
-      if(mn1.le.0) then
-         call error(42)
-         return
-      endif
+
       lr=sadr(ilr+10)
       err=lr+mn1*(it1+1)-lstk(bot)
       if(err.gt.0) then
          call error(17)
          return
       endif
-      call dcopy(mn1*(it1+1),stk(l1),-1,stk(lr),-1)
+      call unsfdcopy(mn1*(it1+1),stk(l1),-1,stk(lr),-1)
       istk(ilr)=2
       istk(ilr+1)=1
       istk(ilr+2)=1
@@ -336,6 +339,8 @@ c
 c
       m1=istk(il1+1)
       n1=istk(il1+2)
+      if(m1*n1.eq.0) return
+
       it1=istk(il1+3)
       mn1=m1*n1
       if(mn1.ne.1) then
@@ -367,7 +372,7 @@ c     real polynomial
             call error(17)
             return
          endif
-         call dcopy(n+1,stk(lc),1,stk(lp),1)
+         call unsfdcopy(n+1,stk(lc),1,stk(lp),1)
          call dtild(n+1,stk(lp),1)
          call rpoly(stk(lp),n,stk(l1),stk(l1+n),fail)
          if(fail.eq.1) then
@@ -398,7 +403,7 @@ c     complex polynomial
          return
       endif
       sr=stk(lc+n)
-      call dcopy(n,stk(lc),-1,stk(lw),1)
+      call unsfdcopy(n,stk(lc),-1,stk(lw),1)
       if(it1.eq.0) then
          call dscal(n,-1.0d+0/sr,stk(lw),1)
       else
@@ -406,13 +411,13 @@ c     complex polynomial
          t=sr*sr+si*si
          sr=-sr/t
          si=si/t
-         call dcopy(n,stk(lc+vol),-1,stk(lw+n),1)
+         call unsfdcopy(n,stk(lc+vol),-1,stk(lw+n),1)
          call wscal(n,sr,si,stk(lw),stk(lw+n),1)
       endif
       call dset(n*n*(it1+1),0.0d+0,stk(l1),1)
       call dset(n-1,1.0d+0,stk(l1+n),n+1)
-      call dcopy(n,stk(lw),1,stk(l1),1)
-      if(it1.eq.1) call dcopy(n,stk(lw+n),1,stk(l1+n*n),1)
+      call unsfdcopy(n,stk(lw),1,stk(l1),1)
+      if(it1.eq.1) call unsfdcopy(n,stk(lw+n),1,stk(l1+n*n),1)
       lstk(top+1)=l1+n*n*(it1+1)
       istk(ilr)=1
       istk(ilr+1)=n
@@ -546,7 +551,7 @@ c
       else
          call icopy(mr*nr+1,istk(idr),1,istk(id1),1)
          l1=sadr(id1+mr*nr+1)
-         call dcopy(volr*(it1+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(volr*(it1+1),stk(lr),1,stk(l1),1)
       endif
       istk(ilr+1)=mr
       istk(ilr+2)=nr
@@ -656,7 +661,7 @@ c
       else
          call icopy(mn1+1,istk(idr),1,istk(id1),1)
          l1=sadr(id1+mn1+1)
-         call dcopy(volr*(it1+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(volr*(it1+1),stk(lr),1,stk(l1),1)
       endif
       istk(ilr+1)=m1
       istk(ilr+2)=n1
@@ -734,7 +739,7 @@ c
          else
             istk(ilr)=1
             istk(ilr+3)=0
-            call dcopy(mn1,stk(l1),1,stk(l),1)
+            call unsfdcopy(mn1,stk(l1),1,stk(l),1)
          endif
          lstk(top+1)=l+mn1
       elseif(istk(il1).eq.1) then
@@ -855,14 +860,14 @@ c     coeff of a matrix of polynomials
             do 30 ij=1,mn1
                nij=istk(id1+ij)-istk(id1+ij-1)
                lij=l1-1+istk(id1+ij-1)
-               call dcopy(nij,stk(lij),1,stk(lr-1+ij),mn1)
+               call unsfdcopy(nij,stk(lij),1,stk(lr-1+ij),mn1)
                if(it1.eq.1) then
-                  call dcopy(nij,stk(lij+vol),1,stk(lr+v2-1+ij),mn1)
+                  call unsfdcopy(nij,stk(lij+vol),1,stk(lr+v2-1+ij),mn1)
                endif
  30         continue
             if(.not.ref) then
                l=sadr(ilr+4)
-               call dcopy(v2*(it1+1),stk(lr),1,stk(l),1)
+               call unsfdcopy(v2*(it1+1),stk(lr),1,stk(l),1)
                lr=l
             endif
             istk(ilr)=1
@@ -877,7 +882,7 @@ c     coeff of a matrix of polynomials
                call error(17)
                return
             endif
-            call dcopy(nv,stk(lv),1,stk(lw),1)
+            call unsfdcopy(nv,stk(lv),1,stk(lw),1)
             lv=lw
             call dset(mn1*nv*(it1+1),0.0d+0,stk(lr),1)
             do 41 ij=1,mn1
@@ -891,7 +896,8 @@ c     coeff of a matrix of polynomials
  40            continue
  41         continue
             l=sadr(ilr+4)
-            if(.not.ref) call dcopy(mn1*nv*(it1+1),stk(lr),1,stk(l),1)
+            if(.not.ref) call unsfdcopy(mn1*nv*(it1+1),stk(lr),1,
+     $           stk(l),1)
             istk(ilr)=1
             istk(ilr+1)=m1
             istk(ilr+2)=n1*nv
@@ -910,7 +916,7 @@ c     .      copy matrix on the top of the stack
                   call error(17)
                   return
                endif
-               call dcopy(lstk(k+1)-lstk(k),stk(lstk(k)),1
+               call unsfdcopy(lstk(k+1)-lstk(k),stk(lstk(k)),1
      $              ,stk(lstk(top)),1)
                lstk(top+1)=lstk(top)+lstk(k+1)-lstk(k)
             endif
@@ -926,19 +932,19 @@ c     .      copy matrix on the top of the stack
             return
          endif
          if(lv.le.lw) then
-            call dcopy(nv,stk(lv),1,stk(lw),1)
+            call unsfdcopy(nv,stk(lv),-1,stk(lw),-1)
             lv=lw
          endif
          call dset(mn1*nv*(it1+1),0.0d+0,stk(lr),1)
          do 50 k=1,nv
             if(stk(lv-1+k).eq.0.0d0) then
-               call dcopy(mn1,stk(l1),1,stk(lr+(k-1)*mn1),1)
-               if(it1.eq.1) call dcopy(mn1,stk(l1+mn1),1,
+               call unsfdcopy(mn1,stk(l1),1,stk(lr+(k-1)*mn1),1)
+               if(it1.eq.1) call unsfdcopy(mn1,stk(l1+mn1),1,
      $              stk(lr+(k-1)*mn1+mn1*nv),1)
             endif
  50      continue
          l=sadr(ilr+4)
-         if(.not.ref) call dcopy(mn1*nv*(it1+1),stk(lr),1,stk(l),1)
+         if(.not.ref) call unsfdcopy(mn1*nv*(it1+1),stk(lr),1,stk(l),1)
          istk(ilr)=1
          istk(ilr+1)=m1
          istk(ilr+2)=n1*nv
@@ -1028,7 +1034,7 @@ c
          istk(ilr+2)=1
          istk(ilr+9)=1+maxd
          l1=sadr(il1+10)
-         call dcopy(maxd*(it1+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(maxd*(it1+1),stk(lr),1,stk(l1),1)
          lr=l1
       endif
       lstk(top+1)=lr+maxd*(it1+1)
@@ -1115,7 +1121,7 @@ c
          istk(ilr+2)=1
          istk(ilr+9)=1+maxd
          l1=sadr(il1+10)
-         call dcopy(maxd*(it1+1),stk(lr),1,stk(l1),1)
+         call unsfdcopy(maxd*(it1+1),stk(lr),1,stk(l1),1)
          lr=l1
       endif
       lstk(top+1)=lr+maxd*(it1+1)
@@ -1166,8 +1172,8 @@ c     move d and copy value of n in place of its reference
             call error(17)
             return
          endif
-         call dcopy(vold,stk(lstk(top)),-1,stk(lstk(top-1)+voln),-1)
-         call dcopy(voln,stk(lstk(k)),1,stk(lstk(top-1)),1)
+         call unsfdcopy(vold,stk(lstk(top)),-1,stk(lstk(top-1)+voln),-1)
+         call unsfdcopy(voln,stk(lstk(k)),1,stk(lstk(top-1)),1)
          lstk(top)=lstk(top-1)+voln
          ild=iadr(lstk(top))
          lstk(top+1)=lstk(top)+vold
@@ -1182,7 +1188,7 @@ c      copy value of b in place of its reference
             call error(17)
             return
          endif
-         call dcopy(vold,stk(lstk(k)),1,stk(lstk(top)),1)
+         call unsfdcopy(vold,stk(lstk(k)),1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+vold
       endif
       lw=lstk(top+1)
@@ -1314,8 +1320,8 @@ c     .  set type
 c     .  set length
          istk(il1+9)=nq+2
 c     .  install real and imaginary part
-         call dcopy(nq+1,stk(l1+nd),1,stk(l1),1)
-         if(itr.eq.1) call dcopy(nq+1,stk(l1i+nd),1,stk(l1+nq+1),1)
+         call unsfdcopy(nq+1,stk(l1+nd),1,stk(l1),1)
+         if(itr.eq.1) call unsfdcopy(nq+1,stk(l1i+nd),1,stk(l1+nq+1),1)
       else
 c     .  computes remainder degree
          nr=nd
@@ -1349,7 +1355,7 @@ c????               stk(lr+1)=stk(l1i)
             lq=sadr(ilq+10)
             inc=1 
             if (l1+nd.lt.lq) inc=-1
-            call dcopy(nq+1,stk(l1+nd),inc,stk(lq),inc)
+            call unsfdcopy(nq+1,stk(l1+nd),inc,stk(lq),inc)
             if(nr.lt.0) stk(lr)=0.0d+0
             istk(ilq)=2
             istk(ilq+1)=1
@@ -1371,13 +1377,13 @@ c     .     preserve quotient coeff
                call error(17)
                return
             endif
-            call dcopy(nq+1,stk(l1+nd),1,stk(lqs),1)
-            call dcopy(nq+1,stk(l1i+nd),1,stk(lqs+nq+1),1)
+            call unsfdcopy(nq+1,stk(l1+nd),1,stk(lqs),1)
+            call unsfdcopy(nq+1,stk(l1i+nd),1,stk(lqs+nq+1),1)
 
 c     .     set remainder
             if(nr.ge.0) then
                istk(il1+3)=1
-               call dcopy(nr+1,stk(l1i),1,stk(l1+nr+1),1)
+               call unsfdcopy(nr+1,stk(l1i),1,stk(l1+nr+1),1)
                istk(id1+1)=nr+2
                lstk(top)=l1+(nr+1)*2
             else
@@ -1397,7 +1403,7 @@ c     .     set quotient
             istk(idq)=1
             istk(idq+1)=nq+2
             lq=sadr(ilq+10)
-            call dcopy(2*(nq+1),stk(lqs),1,stk(lq),1)
+            call unsfdcopy(2*(nq+1),stk(lqs),1,stk(lq),1)
             lstk(top+1)=lq+(nq+1)*2
          endif
       endif
@@ -1514,7 +1520,7 @@ c     pgcd
       istk(ilr+8)=1
       np=ipb(2)-ipb(1)-1
       istk(ilr+9)=2+np
-      call dcopy(np+1,stk(lf+ipb(1)-1),1,stk(l1),1)
+      call unsfdcopy(np+1,stk(lf+ipb(1)-1),1,stk(l1),1)
       lstk(top)=l1+np+1
 
 c     matrice q 
@@ -1530,7 +1536,8 @@ c     matrice q
       do 71 i=1,4
          ii=i+1
          istk(il+1)=istk(il)+ipb(ii+1)-ipb(ii)
-         call dcopy(istk(il+1)-istk(il),stk(lf+ipb(ii)-1),1,stk(l),1)
+         call unsfdcopy(istk(il+1)-istk(il),stk(lf+ipb(ii)-1),1,
+     $        stk(l),1)
          l=l+istk(il+1)-istk(il)
          il=il+1
  71   continue
@@ -1621,8 +1628,8 @@ c     move b and copy value of a in place of its reference
             call error(17)
             return
          endif
-         call dcopy(volb,stk(lstk(top)),-1,stk(lstk(top-1)+vola),-1)
-         call dcopy(vola,stk(lstk(k)),1,stk(lstk(top-1)),1)
+         call unsfdcopy(volb,stk(lstk(top)),-1,stk(lstk(top-1)+vola),-1)
+         call unsfdcopy(vola,stk(lstk(k)),1,stk(lstk(top-1)),1)
          lstk(top)=lstk(top-1)+vola
          ilb=iadr(lstk(top))
          lstk(top+1)=lstk(top)+volb
@@ -1637,7 +1644,7 @@ c      copy value of b in place of its reference
             call error(17)
             return
          endif
-         call dcopy(volb,stk(lstk(k)),1,stk(lstk(top)),1)
+         call unsfdcopy(volb,stk(lstk(k)),1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+volb
       endif
       lw=lstk(top+1)
@@ -1708,8 +1715,13 @@ c     determine max of the degrees
          nb=max(nb,istk(idb+i)-istk(idb-1+i))
  76   continue
 c     preserve adress of the beginning of a and b coefficients
+
       lar=la
       lbr=lb
+c     allocate memory for intermediate results 
+      law=lw
+      lbw=law+na+1
+      lw=lbw+nb+1
 
 c     simplify
       la1=la
@@ -1718,14 +1730,18 @@ c     simplify
          na=istk(ida+i)-istk(ida-1+i)-1
          nb=istk(idb+i)-istk(idb-1+i)-1
          ierr=lstk(bot)-lw
-         call  dpsimp(stk(la),na,stk(lb),nb,stk(la1),nnum,
-     $        stk(lb1),nden,stk(lw),ierr)
+         call  dpsimp(stk(la),na,stk(lb),nb,stk(law),nnum,
+     $        stk(lbw),nden,stk(lw),ierr)
          if(ierr.eq.1) then
             call error(27)
             return
          elseif(ierr.eq.2) then
             call msgs(43,i)
          endif
+c     .  copy overwrite initial polynomials with simplified ones
+         call dcopy(nnum,stk(law),1,stk(la1),1)
+         call dcopy(nden,stk(lbw),1,stk(lb1),1)
+
          la=la+na+1
          lb=lb+nb+1
          la1=la1+nnum
@@ -1756,13 +1772,13 @@ c     b matrice de polynome
          l=sadr(il+9+mna)
 c     .  move b data up
          call icopy(9+mna,istk(ilb),1,istk(il),1)
-         call dcopy(istk(il+8+mna),stk(lbr),1,stk(l),1)
+         call unsfdcopy(istk(il+8+mna),stk(lbr),1,stk(l),1)
          l=l+istk(il+8+mna)-1
       else
 c     b matrice de scalaires
          call icopy(4,istk(ilb),1,istk(il),1)
          l=sadr(il+4)
-         call dcopy(mna,stk(lbr),1,stk(l),1)
+         call unsfdcopy(mna,stk(lbr),1,stk(l),1)
          l=l+mna
       endif
       lstk(top+1)=l
@@ -1812,7 +1828,7 @@ c
             call error(17)
             return
          endif
-         call dcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
+         call unsfdcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+vol
          lw=lstk(top+1)
          il1=ilr
@@ -1885,6 +1901,11 @@ c     set adress where to put the results
       lb1=lb
       ida1=ida
       idb1=idb
+c
+c     allocate memory for intermediate results 
+      law=lw
+      lbw=law+na+1
+      lw=lbw+nb+1
 
 c     beginning of numerator in resulting structure
       l0=ll+istk(ilr+4)-1
@@ -1899,14 +1920,19 @@ c     simplify
          na=istk(ida+i)-istk(ida-1+i)-1
          nb=istk(idb+i)-istk(idb-1+i)-1
          ierr=lstk(bot)-lw
-         call  dpsimp(stk(la),na,stk(lb),nb,stk(la1),nnum,
-     $        stk(lb1),nden,stk(lw),ierr)
+         call  dpsimp(stk(la),na,stk(lb),nb,stk(law),nnum,
+     $        stk(lbw),nden,stk(lw),ierr)
          if(ierr.eq.1) then
             call error(27)
             return
          elseif(ierr.eq.2) then
             call msgs(43,i)
          endif
+
+c     .  copy overwrite initial polynomials with simplified ones
+         call dcopy(nnum,stk(law),1,stk(la1),1)
+         call dcopy(nden,stk(lbw),1,stk(lb1),1)
+
          la=la+na+1
          lb=lb+nb+1
          la1=la1+nnum
@@ -1941,20 +1967,20 @@ c     b matrice de polynome
          call icopy(9+mna,istk(ilb),1,istk(il),1)
          idb1=il+8
          l=sadr(il+9+mna)
-         call dcopy(istk(idb1+mna)-1,stk(lbr),1,stk(l),1)
+         call unsfdcopy(istk(idb1+mna)-1,stk(lbr),1,stk(l),1)
          l=l+istk(idb1+mna)-1
       else
 c     b matrice de scalaires
          call icopy(4,istk(ilb),1,istk(il),1)
          l=sadr(il+4)
-         call dcopy(mna,stk(lbr),1,stk(l),1)
+         call unsfdcopy(mna,stk(lbr),1,stk(l),1)
          l=l+mna
       endif
       volb=l-ll0
 
 c     copy 4th entry of the list 
       mb=istk(ilr+6)-istk(ilr+5)
-      call dcopy(mb,stk(ll+istk(ilr+5)-1),1,stk(l),1)
+      call unsfdcopy(mb,stk(ll+istk(ilr+5)-1),1,stk(l),1)
       l=l+mb
       lstk(top+1)=l
 c     adjust components sizes
@@ -2023,7 +2049,7 @@ c     . sfact1 modifies its input argument
                return
             endif
             call icopy(8+mn1+1,istk(il1),1,istk(ilr),1)
-            call dcopy(vol,stk(l1),1,stk(lr),1)
+            call unsfdcopy(vol,stk(l1),1,stk(lr),1)
             lstk(top+1)=lr+vol
          endif
 c
@@ -2090,7 +2116,8 @@ c
          do 88 i1=0,mn1-1
             lij=l1-1+istk(id1+i1)
             mij=2+istk(id1+i1+1)-istk(id1+i1)-1-n1
-            if(mij.gt.0) call dcopy(mij,stk(lij+n1-1),1,stk(l2+i1),mn1)
+            if(mij.gt.0) call unsfdcopy(mij,stk(lij+n1-1),1,
+     $           stk(l2+i1),mn1)
  88      continue
 c     
          maxit=maxit+n1
@@ -2108,7 +2135,7 @@ c     .  convert polynomial matrix to matrix of polynomials
          lr=sadr(id1+mn1+1)
          l1=lr
          do 89 i=0,mn1-1
-            call dcopy(n1,stk(l2+i),mn1,stk(l1),1)
+            call unsfdcopy(n1,stk(l2+i),mn1,stk(l1),1)
             l1=l1+n1
  89      continue
          do 90 i=0,mn1
@@ -2152,7 +2179,7 @@ c
                   call error(17)
                   return
                endif
-               call dcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
+               call unsfdcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
                lstk(top+1)=lstk(top)+vol
             endif
             return
@@ -2177,9 +2204,15 @@ c     extraction du nom de la variable muette
          istk(ilr+1)=1
          istk(ilr+2)=1
          istk(ilr+3)=0
-         call icopy(4,istk(il1+4),-1,istk(ilr+6),-1)
+         do 10 ii=4,1,-1
+            if(istk(il1+3+ii).ne.blank) then
+               nv=ii
+               goto 11
+            endif
+ 10      continue
+ 11      call icopy(nv,istk(il1+4),-1,istk(ilr+6),-1)
          istk(ilr+4)=1
-         istk(ilr+5)=5
+         istk(ilr+5)=nv+1
          lstk(top+1)=sadr(ilr+10)
       elseif(rhs.eq.2) then
 c     change formal variable name
@@ -2211,7 +2244,7 @@ c           stack
                call error(17)
                return
             endif
-            call dcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
+            call unsfdcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
             lstk(top+1)=lstk(top)+vol
          endif
          ilr=iadr(lstk(top))
@@ -2309,7 +2342,7 @@ c     get relative and absolute tolerances
             call error(17)
             return
          endif
-         call dcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
+         call unsfdcopy(vol,stk(lstk(k)),1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+vol
          il1=ilr
       endif
