@@ -25,12 +25,18 @@ c
 c     functions/fin
 c     1       2       3       4       5           6       7       8   
 c     length    part   string  convstr  emptystr str2code code2str sort
-c     9         10       11        12
-c     strcat    strindex strsubst ascii
+c     9         10       11        12    13    14
+c     strcat    strindex strsubst ascii  grep tokens
 
 c     
+c      if ( fin.eq.8)  then 
+c         call intssort(id)
+c      else 
+c         call cstrelm()
+c      endif
+c      return 
 
-      goto (10,20,25,50,60,70,80,90,100,110,120,125) fin
+      goto (10,20,25,50,60,70,80,90,100,110,120,125,130, 135) fin
 c     
 c     length
 c     
@@ -96,6 +102,16 @@ c
  125  continue
       call intascii(id)
       goto 900
+c     
+c     grep(str1,str2) 
+ 130  continue
+      call intgrep()
+      goto 900
+c     
+c     tokens(str [,sep])
+ 135  continue
+      call inttokens(id)
+      goto 900
 
 c     
  900  return
@@ -160,6 +176,11 @@ c
 c
 c     strcat(tr [,ins])
 c
+      if (rhs.eq.3) then 
+         call cstrelm() 
+         return 
+      endif
+
       if(rhs.ne.1.and.rhs.ne.2) then
          call error(39)
          return
@@ -225,7 +246,7 @@ c
          istk(ilr)=10
          istk(ilr+1)=1
          istk(ilr+2)=1
-         istk(ilr+3)=1
+         istk(ilr+3)=0
          istk(ilr+4)=1
          istk(ilr+5)=1+vol
          l=ilr+6
@@ -252,7 +273,7 @@ c
          istk(ilr)=10
          istk(ilr+1)=1
          istk(ilr+2)=1
-         istk(ilr+3)=1
+         istk(ilr+3)=0
          istk(ilr+4)=1
          istk(ilr+5)=1+vol
          l=ilr+6
@@ -280,7 +301,7 @@ c
          call error(39)
          return
       endif
-      if (lhs .ne. 1) then
+      if (lhs .gt. 2) then
          call error(41)
          return
       endif
@@ -294,11 +315,6 @@ c
          return
       endif
       mn2=istk(il2+1)*istk(il2+2)
-c      if(istk(il2+1)*istk(il2+2).ne.1) then
-c         err=2
-c         call error(36)
-c         return
-c      endif
       l2=il2+5+mn2
       do 01 i=1,mn2
          if(istk(il2+4+i)-istk(il2+3+i).eq.0) then
@@ -315,6 +331,14 @@ c
       ref=ilr.ne.il1
       if(istk(il1).eq.1) then
          if(istk(il1+1)*istk(il1+2).eq.0) then
+            istk(ilr)=1
+            istk(ilr+1)=0
+            istk(ilr+2)=0
+            istk(ilr+3)=0
+            lstk(top+1)=sadr(ilr+4)
+            if(lhs.eq.1) return
+            top=top+1
+            ilr=iadr(lstk(top))
             istk(ilr)=1
             istk(ilr+1)=0
             istk(ilr+2)=0
@@ -370,7 +394,7 @@ c     first character matches
       do 16 ii=0,nok-1
          i=istk(ilm+ii)
          if(k.ge.istk(il2+4+i)-istk(il2+3+i)) then
-c     .     a match found or ith string
+c     .     a match found for ith string
             nfound=i
             goto 30
          elseif(istk(ll1+k).eq.istk(l2+istk(il2+3+i)-1+k)) then
@@ -386,14 +410,14 @@ c     .     a match found or ith string
 c     a match found
       mc=mc+1
       if(mod(mc,10).eq.0) then
-         err=lpos+mc+10-lstk(bot)
+         err=lpos+2*(mc+10)-lstk(bot)
          if(err.gt.0) then
             call error(17)
             return
          endif
       endif
-      stk(lpos+mc-1)=ll1-l1+1
-      ll1=ll1+k-1
+      stk(lpos+2*(mc-1)) = dble(ll1-l1+1)
+      stk(lpos+2*(mc-1)+1)     = dble(nfound)
       goto 10
  35   continue
 c     end of string str1 reached
@@ -403,18 +427,55 @@ c     end of string str1 reached
          istk(ilr+2)=0
          istk(ilr+3)=0
          lstk(top+1)=sadr(ilr+4)
-      else
-         l=sadr(ilr+4)
-         if(l.gt.lpos) then
-            call dcopy(mc,stk(lpos),-1,stk(l),-1)
-         else
-            call dcopy(mc,stk(lpos),1,stk(l),1)
-         endif
+         if (lhs.eq.1) return
+         top=top+1
+         ilr=iadr(lstk(top))
          istk(ilr)=1
-         istk(ilr+1)=1
-         istk(ilr+2)=mc
+         istk(ilr+1)=0
+         istk(ilr+2)=0
          istk(ilr+3)=0
-         lstk(top+1)=l+mc
+         lstk(top+1)=sadr(ilr+4)
+      else
+         if(lhs.eq.1) then
+            l=sadr(ilr+4)
+            call unsfdcopy(mc,stk(lpos),2,stk(l),1)
+            istk(ilr)=1
+            istk(ilr+1)=1
+            istk(ilr+2)=mc
+            istk(ilr+3)=0
+            lstk(top+1)=l+mc
+         else
+            lw=lpos+2*mc
+            ilw=iadr(lw)
+            err=lw+2*sadr(4)+2*mc-lstk(bot)
+            if(err.gt.0) then
+               call error(17)
+               return
+            endif
+            istk(ilw)=1
+            istk(ilw+1)=1
+            istk(ilw+2)=mc
+            istk(ilw+3)=0
+            l=sadr(ilw+4)
+            call dcopy(mc,stk(lpos),2,stk(l),1)
+            l=l+mc
+            l1=l
+            ilw=iadr(l)
+            istk(ilw)=1
+            istk(ilw+1)=1
+            istk(ilw+2)=mc
+            istk(ilw+3)=0
+            l=sadr(ilw+4)
+            call dcopy(mc,stk(lpos+1),2,stk(l),1)
+            l=l+mc
+c     .     copy the variables at the top of the stack
+            call unsfdcopy(l-lw,stk(lw),1,stk(lstk(top)),1)
+            lstk(top+1)=lstk(top)+l1-lw
+            top=top+1
+            lstk(top+1)=lstk(top)+l-l1
+
+         endif
+
       endif
       return
       end
@@ -642,7 +703,7 @@ c
             stk(ll-1+k)=dble(istk(id1+k)-istk(id1+k-1))
  11      continue
          l=sadr(ilr+4)
-         if(.not.ref) call dcopy(mn1,stk(ll),1,stk(l),1)
+         if(.not.ref) call unsfdcopy(mn1,stk(ll),1,stk(l),1)
          istk(ilr)=1
          istk(ilr+1)=m1
          istk(ilr+2)=n1
@@ -932,16 +993,16 @@ c
 c     update stack
          if(lhs.eq.3) then
             n=llin-lout
-            call dcopy(n,stk(lout),1,stk(lstk(top)),1)
+            call unsfdcopy(n,stk(lout),1,stk(lstk(top)),1)
             lstk(top+1)=lstk(top)+n+1
             top=top+1
             n=ltxt-llin
-            call dcopy(n,stk(llin),1,stk(lstk(top)),1)
+            call unsfdcopy(n,stk(llin),1,stk(lstk(top)),1)
             lstk(top+1)=lstk(top)+n+1
             top=top+1
          endif
          n=lw-ltxt
-         call dcopy(n,stk(ltxt),1,stk(lstk(top)),1)
+         call unsfdcopy(n,stk(ltxt),1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+n+1
          goto 999
 
@@ -996,7 +1057,7 @@ c
       INCLUDE '../stack.h'
 c
       integer id(nsiz)
-      logical ref,lgq
+      logical ref,lgq,cremat
       integer tops
       integer iadr,sadr
 c
@@ -1042,6 +1103,14 @@ c
    51 continue
       il1=iadr(lstk(top))
       if(abs(istk(il1)).ne.10) then
+         if(abs(istk(il1)).eq.1) then
+            if(istk(il1).lt.0) il1=iadr(istk(il1+1))
+            if(istk(il1+1)*istk(il1+2).eq.0) then
+c              convstr([])
+               lgq=cremat('convstr', top, 0, 0, 0, lr, lc)
+               return
+            endif
+         endif
          top=tops
          fun=-1
          call funnam(ids(1,pt+1),'convstr',iadr(lstk(top+1-rhs)))
@@ -1150,11 +1219,6 @@ c
          ilr=il
          if(istk(il).lt.0) il=iadr(istk(il+1))
          ref=ilr.ne.il
-         if(istk(il+3).ne.0) then
-            err=1
-            call error(52)
-            return
-         endif
          m=istk(il+1)
          n=istk(il+2)
          if(m*n.eq.0) then
@@ -1176,6 +1240,8 @@ c
       istk(il+3)=0
       call iset(m*n+1,1,istk(il+4),1)
       lstk(top+1)=sadr(il+6+m*n)
+c     The correct assignation should be 
+c     lstk(top+1)=sadr(il+5+m*n)
       goto 999 
  999  return
       end
@@ -1272,7 +1338,7 @@ c
       n1 = istk(il1+1)*istk(il1+2)
       l1 = sadr(il1+4)
       if(.not.ref) then
-         call dcopy(n1,stk(l1),-1,stk(l1+2),-1)
+         call unsfdcopy(n1,stk(l1),-1,stk(l1+2),-1)
          l1=l1+2
       endif
       istk(ilr+4)=1
@@ -1327,11 +1393,11 @@ c     select type of sort to perform
          call funnam(ids(1,pt+1),'sort',iadr(lstk(top+1-rhs)))
          return
       endif
+      if(rhs.eq.2) rhs=rhs-1
+      call ref2val
 c
       il1=iadr(lstk(top))
       ilr=il1
-      if(istk(il1).lt.0) il1=iadr(istk(il1+1))
-      ref=il1.ne.ilr
 
       m=istk(il1+1)
       n=istk(il1+2)
@@ -1341,14 +1407,10 @@ c
       vol=istk(id1+mn)-1
 
 c
-      if(ref) then
-         id1r=ilr+4
-         lsz=id1r+mn+1+vol
-      else
-         id1r=id1
-         ls=iadr(lstk(top+1))
-         lsz=ls+vol
-      endif
+      id1r=id1
+      ls=iadr(lstk(top+1))
+      lsz=ls+vol
+
 
       lind=lsz+mn
       lw=lind+mn
@@ -1358,13 +1420,8 @@ c
          return
       endif
 
-      if(ref) then
-         call icopy(4+mn+1,istk(il1),1,istk(ilr),1)
-         ls=id1+mn+1
-         id1=id1r
-      else
-         call icopy(vol,istk(l1),1,istk(ls),1)
-      endif
+      call icopy(vol,istk(l1),1,istk(ls),1)
+
       do 91 i=0,mn-1
          istk(lsz+i)=istk(id1+i+1)-istk(id1+i)
  91   continue
@@ -1419,12 +1476,13 @@ c     sort(a,'r')  <=>  sort(a,1)
       subroutine intascii(id)
       include '../stack.h'
 c
-      integer id(nsiz),tops,vol,sel
+      integer id(nsiz),tops,vol,sel,eol
       logical ref
-      external strord
-      integer strord
+      external strord,getfastcode
+      integer strord,getfastcode
       character*1 c
       integer iadr,sadr
+      data eol/99/
 c
       iadr(l)=l+l-1
       sadr(l)=(l/2)+1
@@ -1444,7 +1502,7 @@ c
       ref=il.ne.ilr
 
       if(istk(il).eq.1) then
-c     argument is a vector of ascii codes
+c     argument is a vector of ascii codes return a scilab string
          n=istk(il+1)*istk(il+2)
          l=sadr(il+4)
          if(.not.ref) then
@@ -1453,7 +1511,7 @@ c     argument is a vector of ascii codes
                call error(17)
                return
             endif
-            call dcopy(n,stk(l),1,stk(l+n),1) 
+            call unsfdcopy(n,stk(l),1,stk(l+n),1) 
             l=l+n
          else
             err=sadr(ilr+6+n)-lstk(bot)
@@ -1471,24 +1529,15 @@ c     argument is a vector of ascii codes
          lr=ilr+6
          do 20 i=0,n-1
             c=char(int(stk(l+i)))
-            istk(lr+i)=99
-            do 15 k=1,csiz
-               if(c.eq.alfa(k)) then
-                  istk(lr+i)=k-1
-                  goto 20
-               elseif(c.eq.alfb(k)) then
-                  istk(lr+i)=-(k-1)
-                  goto 20
-               endif
- 15         continue
+            istk(lr+i)=getfastcode(c)
  20      continue
          lstk(top+1)=sadr(lr+n)
       elseif(istk(il).eq.10) then
-c     argument is a string
+c     argument is a scilab string return a vector of ascii codes
          n=istk(il+4+istk(il+1)*istk(il+2))-1
          l=il+5+istk(il+1)*istk(il+2)
          if(.not.ref) then
-            lw=iadr(sadr(ilr+4)+n)
+            lw=l+n
             err=sadr(lw+n)-lstk(bot)
             if(err.gt.0) then
                call error(17)
@@ -1510,8 +1559,12 @@ c     argument is a string
          lr=sadr(ilr+4)
          do 30 i=0,n-1
             mc=istk(l+i)
-            if(mc.lt.0) then
+            if(mc.gt.eol) then
+               stk(lr+i)=ichar(char(mc-(eol+1)))
+            elseif(mc.lt.0) then
                stk(lr+i)=ichar(alfb(abs(mc)+1))
+            elseif(mc.eq.eol) then
+               stk(lr+i)=10.
             else
                stk(lr+i)=ichar(alfa(mc+1))
             endif
@@ -1522,3 +1575,391 @@ c     argument is a string
          call funnam(ids(1,pt+1),'ascii',iadr(lstk(top)))
       endif
       end
+
+      subroutine intgrep()
+      INCLUDE '../stack.h'
+c
+      integer vol,vol2,tops
+      logical ref
+c
+      integer iadr,sadr
+c
+      iadr(l)=l+l-1
+      sadr(l)=(l/2)+1
+c
+c     strindex(str1,str2)
+c
+      tops=top
+
+      if(rhs.ne.2) then
+         call error(39)
+         return
+      endif
+      if (lhs .gt. 2) then
+         call error(41)
+         return
+      endif
+c     
+      il2=iadr(lstk(top))
+      if(istk(il2).lt.0) il2=iadr(istk(il2+1))
+      if(istk(il2).ne.10) then 
+         fun=-1
+         top=tops
+         call funnam(ids(1,pt+1),'grep',iadr(lstk(top)))
+         return
+      endif
+      mn2=istk(il2+1)*istk(il2+2)
+      l2=il2+5+mn2
+      do 01 i=1,mn2
+         if(istk(il2+4+i)-istk(il2+3+i).eq.0) then
+            err=2
+            call error(249)
+            return
+         endif
+ 01   continue
+      top=top-1
+c     
+      il1=iadr(lstk(top))
+      ilr=il1
+      if(istk(il1).lt.0) il1=iadr(istk(il1+1))
+      ref=ilr.ne.il1
+      if(istk(il1).eq.1) then
+         if(istk(il1+1)*istk(il1+2).eq.0) then
+            istk(ilr)=1
+            istk(ilr+1)=0
+            istk(ilr+2)=0
+            istk(ilr+3)=0
+            lstk(top+1)=sadr(ilr+4)
+            return
+         endif
+      endif
+      if(istk(il1).ne.10) then
+         fun=-1
+         top=tops
+         call funnam(ids(1,pt+1),'grep',iadr(lstk(top+1-rhs)))
+         return
+      endif
+c
+      mn1=istk(il1+1)*istk(il1+2)
+      id1=il1+4
+      l1=id1+mn1+1
+      vol=istk(id1+mn1)-1
+      lw = max(lstk(tops+1),lstk(top)+mn1+2)
+      ilrow=lw
+      lw=ilrow+mn1
+      if(lhs.eq.2) then
+         ilm=lw
+         lw=ilm+mn1
+      endif
+      err=lw-lstk(bot)
+      if(err.gt.0) then
+         call error(17)
+         return
+      endif
+c
+c     loops on arg1 entries
+      mc=0
+      do 36 k1=1,mn1
+         vol=istk(id1+k1)-istk(id1+k1-1)
+         ll1=l1-1
+ 10      ll1=ll1+1
+         if(ll1-l1.ge.vol) goto 35 
+c     .  loop on arg2 strings
+         do 32 k2=1,mn2
+            ll2=l2+istk(il2+3+k2)-1
+            vol2=istk(il2+4+k2)-istk(il2+3+k2)
+c     .     check for a match on  characters  arg1(k1)(ll1) and arg2(k2)(1)
+            if(istk(ll1).eq.istk(ll2))  then
+c     .        they match
+               i2=k2
+            else
+c     .        characters arg1(k1)(ll1) and arg2(k2)(1) doesnt match
+               goto 32
+            endif
+c           first character matches check if next chars match
+ 12         k=0
+ 15         k=k+1
+            if(k.ge.vol2) then
+c     .        end of arg2(k2) reached a match found or ith string
+               nfound=k2
+               goto 30
+            elseif(ll1+k.ge.l1+vol) then
+c     .        not enough chars in arg1(k1)
+               goto 32
+            elseif(istk(ll1+k).eq.istk(ll2+k)) then
+c     .        next characters matches go ahaead
+               goto 15
+            else
+c     .        next characters doesnt match, chech for an other string in arg2
+               goto 32
+            endif
+
+ 30         continue
+c     a match found
+            mc=mc+1
+
+            stk(ilrow+mc-1)=k1
+            if(lhs.eq.2) stk(ilm+mc-1)=nfound
+            goto 35
+ 32      continue
+c     .  arg1(k1)(ll1) doesnt match with any string in arg2
+         goto 10
+
+c     end of loop skip to next entry
+ 35      l1=l1+vol
+ 36   continue
+c
+c     end of string str1 reached
+       if(mc.eq.0) then
+            istk(ilr)=1
+            istk(ilr+1)=0
+            istk(ilr+2)=0
+            istk(ilr+3)=0
+            lstk(top+1)=sadr(ilr+4)
+            if(lhs.eq.2) then
+               top=top+1
+               ilr=iadr(lstk(top))
+               istk(ilr)=1
+               istk(ilr+1)=0
+               istk(ilr+2)=0
+               istk(ilr+3)=0
+               lstk(top+1)=sadr(ilr+4)
+            endif
+         else
+            lrow=sadr(ilr+4)
+            lstk(top+1)=lrow+mc
+            if(lhs.eq.2) then
+               top=top+1
+               il2=iadr(lstk(top))
+               lm=sadr(il2+4)
+               lstk(top+1)=lm+mc
+            endif
+            err=lstk(top+1)-lstk(bot)
+            if(err.gt.0) then
+               call error(17)
+               return
+            endif
+            istk(ilr)=1
+            istk(ilr+1)=1
+            istk(ilr+2)=mc
+            istk(ilr+3)=0
+            inc=1
+            if(lrow.gt.ilrow) inc=-1
+            call unsfdcopy(mc,stk(ilrow),inc,stk(lrow),inc)
+            if(lhs.eq.2) then
+               istk(il2)=1
+               istk(il2+1)=1
+               istk(il2+2)=mc
+               istk(il2+3)=0
+               inc=1
+               if(lm.gt.ilm) inc=-1
+               call unsfdcopy(mc,stk(ilm),inc,stk(lm),inc)
+            endif
+         endif
+      return
+      end
+
+      subroutine inttokens(id)
+c
+c     Interface code for tokens :
+c
+c         words = tokens( str [, sep])
+c
+c     which cut the string str into words (as a vector column of words)
+c     the words being sub-strings of str between separators chars.
+c
+c     where str must be a simple string 
+c      and  sep a vector (or matrix) of characters (not string of chars)
+c           sep is optionnal and if it is is omitted then sep = [ " ", TAB ]
+c
+c     Added by Bruno      
+c        (Initially tokens was coded as a scilab macro but was very inefficient).
+c
+      implicit none
+      INCLUDE '../stack.h'
+
+      integer id(nsiz)
+
+      integer lw, tops, il2, m2, n2, p2, i_sep, nb_sep, il1, l1, nv1, 
+     $        ilr, nb_chars, nb_words, i, c, ipt, size, lword, iwd
+      logical  in_sep
+
+      logical is_a_sep
+      external error, funnam, is_a_sep, icopy
+
+      integer BLANK, TAB
+      parameter (BLANK = 40, TAB = -40)
+
+      integer iadr,sadr, l
+      iadr(l)=l+l-1
+      sadr(l)=(l/2)+1
+
+      if(rhs.lt.1 .or. rhs.gt.2) then
+         call error(39)
+         return
+      endif
+      if (lhs .ne. 1) then
+         call error(41)
+         return
+      endif
+
+      tops=top
+     
+      lw = iadr(lstk(top+1))
+     
+      if (rhs .eq. 2) then  
+c        get the separators sep
+         il2 = iadr(lstk(top))
+         if (istk(il2).lt.0) il2 = iadr(istk(il2+1))
+         if (istk(il2).ne.10) then 
+            fun=-1
+            top=tops
+            call funnam(ids(1,pt+1),'tokens',iadr(lstk(top)))
+            return
+         endif
+         m2 = istk(il2+1)
+         n2 = istk(il2+2)
+c        verify that each element of sep is a simple character
+         p2 = il2 + 4
+         do i = 1, m2*n2
+            if (istk(p2+i)-istk(p2+i-1) .ne. 1) then
+               buf='a separator must be formed by only one character'
+               call error(9999)
+               return
+            endif
+         enddo
+         nb_sep = m2*n2           
+c     !  there are nb_sep separators, and the first is located at istk(i_sep)
+         i_sep = p2 + 1 + nb_sep  
+         top = top - 1
+      else
+c        no separators given => uses the 2 default separators (blank and tab)
+         i_sep = lw
+         nb_sep = 2
+         lw = lw + 2
+         err=sadr(lw)-lstk(bot)
+         if ( err.gt.0 ) then
+            call error(17)
+            return
+         endif
+         istk(i_sep) = BLANK
+         istk(i_sep+1) = TAB
+      endif
+
+c     get the string str
+      il1 = iadr(lstk(top)) 
+      if(istk(il1).lt.0) il1 = iadr(istk(il1+1))
+      if(istk(il1).ne.10) then 
+         fun = -1
+         top = tops
+         call funnam(ids(1,pt+1),'tokens',iadr(lstk(top-1)))
+         return
+      endif
+      if(istk(il1+1)*istk(il1+2).ne.1) then 
+c     !  str must be a simple string not a vector|matrix of strings
+         call error(36)
+         return
+      endif
+      l1 = il1+6
+      nv1 = istk(il1+5)-1   
+c     ! nv1 is the number of chars of str
+      if(nv1.eq.0) then     
+c     !   str is an empty string => return a void matrix
+         il1=iadr(lstk(top)) 
+         istk(il1)   = 1
+         istk(il1+1) = 0
+         istk(il1+2) = 0
+         istk(il1+3) = 0
+         return
+      endif
+
+c     count the number of words (and characters)
+      nb_words = 0
+      nb_chars = 0
+      in_sep = .true.
+      do i = 0, nv1-1
+         c = istk(l1+i)
+         if ( is_a_sep(c, istk(i_sep), nb_sep) ) then
+            in_sep = .true.
+         else
+            nb_chars = nb_chars + 1
+            if (in_sep) then
+               nb_words = nb_words + 1
+               in_sep = .false.
+            endif
+         endif
+      enddo
+
+      if ( nb_words .eq. 0 ) then 
+c     !   return a void matrix
+         il1=iadr(lstk(top)) 
+         istk(il1)   = 1
+         istk(il1+1) = 0
+         istk(il1+2) = 0
+         istk(il1+3) = 0
+         return
+      endif
+
+c     prepare memory for the result
+      ilr = lw
+      size =  5 + nb_words + nb_chars  
+c     ! size of the resulting scilab object
+      lw = lw + size                   
+      err = sadr(lw) - lstk(bot)
+      if ( err.gt.0 ) then
+         call error(17)
+         return
+      endif
+
+c     form the output
+      istk(ilr)   = 10
+      istk(ilr+1) = nb_words
+      istk(ilr+2) = 1
+      istk(ilr+3) = 0
+      istk(ilr+4) = 1
+      ipt = ilr + 5
+      iwd = ipt + nb_words
+      istk(iwd-1) = nb_chars + 1 
+c     dernier pointeur (necessaire lorsque la chaine ne se termine pas 
+C     par un separateur)
+      lword = 0
+      do i = 0, nv1-1
+         c = istk(l1+i)
+         if ( is_a_sep(c, istk(i_sep), nb_sep) ) then
+            if (lword .gt. 0) then 
+c     !        on vient donc de terminer un mot
+               istk(ipt) = istk(ipt-1) + lword
+               ipt = ipt + 1
+               lword = 0
+            endif
+         else
+            istk(iwd) = c
+            iwd = iwd + 1
+            lword = lword + 1
+         endif
+      enddo
+
+c     copy result at top
+      il1=iadr(lstk(top)) 
+      call icopy(size, istk(ilr), 1, istk(il1), 1)       
+      lstk(top+1)=sadr(il1 + size)
+
+      end 
+
+
+      logical function is_a_sep(c, sep, nb_sep)
+c
+c     utility function used by inttokens (Bruno)
+c      
+      implicit none
+      integer c, nb_sep, sep(nb_sep)
+      integer i
+      is_a_sep = .false.
+      do i = 1, nb_sep
+         if (c .eq. sep(i)) then
+            is_a_sep = .true.
+            return
+         endif
+      enddo
+
+      end 
