@@ -7,6 +7,7 @@ c ====================================================================
 c
 c     Copyright INRIA
       include '../stack.h'
+      common /mtlbc/ mmode
 c
       integer plus,quote,equal,less,great,insert,extrac
       integer top0,iadr,sadr,op,vol,volr,rhs1
@@ -124,7 +125,7 @@ c
       if(m2.eq.0) then
          return
       elseif(m1.eq.0)then
-         call dcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1)),1,
+         call unsfdcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1)),1,
      &        stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+lstk(top+2)-lstk(top+1)
          return
@@ -162,7 +163,7 @@ c     concatenation [a;b]
       if(n2.eq.0) then
          return
       elseif(n1.eq.0)then
-         call dcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1)),1,
+         call unsfdcopy(lstk(top+2)-lstk(top+1),stk(lstk(top+1)),1,
      &        stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+lstk(top+2)-lstk(top+1)
          return
@@ -256,7 +257,13 @@ c     get arg1
       il1=iadr(lstk(top))
       ilrs=il1
       if(istk(il1).lt.0) il1=iadr(istk(il1+1))
+      if (istk(il1).eq.10.or.istk(il1).eq.15) then
+         top=top0
+         fin=-fin
+         return
+      endif
       m1=istk(il1+1)
+      n1=istk(il1+2)
 c
       if (m2.eq.0) then
 c     .  arg3(arg1)=[] 
@@ -332,13 +339,21 @@ c     .  reshape arg2 according to arg3
 c     .  arg3([])=arg2
          if(mn2.eq.1) then
 c     .  arg3([])=c  --> arg3 
-            istk(ilrs)=10
-            istk(ilrs+1)=m3
-            istk(ilrs+2)=n3
-            istk(ilrs+3)=0
-            volr=istk(id3+mn3)-1
-            call icopy(mn3+1+volr,istk(id3),1,istk(ilrs+4),1)
-            lstk(top+1)=sadr(ilrs+5+mn3+volr)
+            if(mn3.eq.0) then
+               istk(ilrs)=1
+               istk(ilrs+1)=0
+               istk(ilrs+2)=0
+               istk(ilrs+3)=0
+               lstk(top+1)=sadr(ilrs+4)
+            else
+               istk(ilrs)=10
+               istk(ilrs+1)=m3
+               istk(ilrs+2)=n3
+               istk(ilrs+3)=0
+               volr=istk(id3+mn3)-1
+               call icopy(mn3+1+volr,istk(id3),1,istk(ilrs+4),1)
+               lstk(top+1)=sadr(ilrs+5+mn3+volr)
+            endif
             goto 999
          else
             call error(15)
@@ -408,8 +423,13 @@ c
 c     set output variable
       ilrs=iadr(lstk(top))
       istk(ilrs)=10
-      istk(ilrs+1)=mr
-      istk(ilrs+2)=nr
+      if(mmode.eq.1.and.nr.eq.1.and.m3.eq.0) then
+         istk(ilrs+1)=nr
+         istk(ilrs+2)=mr
+      else
+         istk(ilrs+1)=mr
+         istk(ilrs+2)=nr
+      endif
       istk(ilrs+3)=0
       call icopy(mnr+1+volr,istk(idr),1,istk(ilrs+4),1)
       lstk(top+1)=sadr(ilrs+5+mnr+volr)
@@ -437,11 +457,14 @@ c     get arg3
       il3=iadr(lstk(top))
       if(istk(il3).lt.0) il3=iadr(istk(il3+1))
       if(istk(il3).ne.10) then
+         if(istk(il3).eq.1) then
+            if(istk(il3+1)*istk(il3+2).eq.0) goto 126
+         endif
          top=top0
          fin=-fin
          return
       endif
-      m3=istk(il3+1)
+ 126  m3=istk(il3+1)
       n3=istk(il3+2)
       mn3=m3*n3
       id3=il3+4
@@ -475,7 +498,7 @@ c     .     arg4([],arg2)=[],  arg4(arg1,[])=[] --> arg4
             istk(ilrs+2)=n4
             istk(ilrs+3)=0
             volr=istk(id4+mn4)-1
-            call icopy(mn4+volr,istk(id4),1,istk(ilrs+4),1)
+            call icopy(mn4+1+volr,istk(id4),1,istk(ilrs+4),1)
             lstk(top+1)=sadr(ilrs+5+mn4+volr)
             goto 999
          elseif(m2.eq.-1) then
@@ -511,8 +534,8 @@ c     .     arg4(arg1,arg2)=[]
             if(err.gt.0) return
             if(nj.eq.0) then
 c     .        arg4(arg1,1:n4)=[] 
-               lw2=lw
                call indxgc(il1,m4,ili,mi,mxi,lw)
+               lw2=lw
                if(err.gt.0) return
 c     .        arg2=1:n4
                if(mi.eq.0) then
@@ -571,7 +594,7 @@ c     .  arg3=eye , arg4=eye
       elseif(m1.eq.-1.and.m2.eq.-1) then
 c     .  arg4(:,:)=arg3
          if(mn3.ne.mn4) then
-            if(mn3.eq.1) goto 126
+            if(mn3.eq.1) goto 127
             call error(15)
             return
          endif
@@ -586,7 +609,7 @@ c     .  reshape arg3 according to arg4
          goto 999
       endif
 
- 126  call indxg(il1,m4,ili,mi,mxi,lw,1)
+ 127  call indxg(il1,m4,ili,mi,mxi,lw,1)
       if(err.gt.0) return
       call indxg(il2,n4,ilj,mj,mxj,lw,1)
       if(err.gt.0) return
