@@ -40,6 +40,9 @@ c
       if (fin.eq.12) then
          call intsxgetfile("xgetfile")
       endif
+      if (fin.eq.13) then
+         call scimess1("x_message_modeless")
+      endif
  9999 return
       end
 
@@ -60,13 +63,14 @@ c      implicit undefined (a-z)
       sadr(l)=(l/2)+1
 
       call xscion(inxsci)
-      if (inxsci.eq.0) then
-         buf='synchronous actions are not supported with -nw option'
-         call error(1020)
-         return
-      endif
+c      if (inxsci.eq.0) then
+c         buf='synchronous actions are not supported with -nw option'
+c         call error(1020)
+c         return
+c      endif
       if (.not.checkrhs(fname,1,4)) return
       if (.not.checklhs(fname,1,1)) return
+      topk = top
 c     checking last variable
       ill=iadr(lstk(top))
       if(istk(ill).eq.15) then
@@ -109,7 +113,13 @@ c     in a graphic window
          if(.not.getscalar(fname,topk,top-rhs+1,lr))return
          iwin = int(stk(lr))
          iskip = 0
-      elseif (istk(il1).eq.10) then
+         elseif (istk(il1).eq.10) then
+c      if -nw mode main window does not exists - error
+        if (inxsci.eq.0) then
+         buf='No main graphic window in -nw mode, specify 1-th arg...'
+          call error(1020)
+          return
+         endif
 c     in main window
          iskip =1
          iwin =-1
@@ -177,6 +187,11 @@ c        checking variable submenu names (number 3-iskip)
       end
 
 
+
+
+
+
+
       subroutine scidellm(fname)
 C     ================================================================
 C     SCILAB function : dellmenu 
@@ -187,9 +202,9 @@ c      implicit undefined (a-z)
       logical checkrhs,getsmat,getscalar
       integer topk,m,n,lr,nlr
       call xscion(inxsci)
-      if (inxsci.eq.0) then
-         return
-      endif
+c      if (inxsci.eq.0) then
+c         return
+c      endif
       if (.not.checkrhs(fname,1,2)) return
       topk = top
       if (.not.getsmat(fname,topk,top,m,n,1,1,lr,nlr)) return
@@ -200,8 +215,18 @@ c      implicit undefined (a-z)
          if(.not.getscalar(fname,topk,top,lr))return
          iwin=int(stk(lr))
       else
-         iwin=-1
+c      if -nw mode main window does not exists - error
+        if (inxsci.eq.0) then
+         buf='No main graphic window in -nw mode, specify 1-th arg...'
+          call error(1020)
+          return
+         endif
+       iwin=-1
       endif
+c     next two lines for compatibility "2D Zoom" button renamed in "Zoom"
+      if (buf(1:7).eq.'2D Zoom') buf='Zoom'//char(0)
+      if (buf(1:8).eq.'2D &Zoom') buf='&Zoom'//char(0)
+
       call delbtn(iwin,buf)
       call objvide(fname,top)
       return
@@ -218,9 +243,9 @@ c      implicit undefined (a-z)
       integer topk,m,n,lr,nlr,gettype
       external func
       call xscion(inxsci)
-      if (inxsci.eq.0) then
-         return
-      endif
+c      if (inxsci.eq.0) then
+c         return
+c      endif
       if (.not.checkrhs(fname,1,3)) return
       topk = top
       nsub=0
@@ -240,6 +265,12 @@ c      implicit undefined (a-z)
          iwin = int(stk(lr))
       else
          if (.not.checkrhs(fname,1,2)) return
+c      if -nw mode main window does not exists - error
+        if (inxsci.eq.0) then
+         buf='No main graphic window in -nw mode, specify 1-th arg...'
+          call error(1020)
+          return
+         endif
          iwin = -1
          if ( rhs.eq.2 ) then 
             if(.not.getscalar(fname,topk,top,lr))return
@@ -418,7 +449,7 @@ cc      implicit undefined (a-z)
       endif
       if (.not.getwsmat(fname,topk,top,m3,n3,il3,ild3)) return
       if (m3.ne.1.and.n3.ne.1) then 
-         buf= fname// ': second argument must be a vector'
+         buf= fname// ': third argument must be a vector'
          call error(999)
          return
       endif
@@ -561,6 +592,53 @@ C     first argument is a string vector
       return
       end
 
+      subroutine scimess1(fname) 
+C     ================================================================
+c     SCILAB function : xmessage 
+C     ================================================================
+      character*(*) fname
+cc      implicit undefined (a-z)
+      include '../stack.h'
+      integer topk,iadr,sadr
+      integer m2,n2,il2,ild2,m1,n1,il1,ild1
+      logical getwsmat,checkrhs,cresmat2
+      logical cremat
+      iadr(l)=l+l-1
+      sadr(l)=(l/2)+1
+      if (.not.checkrhs(fname,1,1)) return
+      topk=top
+C     creation of extra arguments
+C     add button name
+      if (rhs.lt.2) then 
+         top  = top+1
+         nlr2 = 2
+         if(.not.cresmat2(fname,top,nlr2,lr2)) return
+         call cvstr(nlr2,istk(lr2),'Ok',0)
+      endif
+C     second argument is a string vector
+      if (.not.getwsmat(fname,topk,top,m2,n2,il2,ild2)) return
+      if (m2.ne.1.or.n2.ne.1) then 
+         buf= fname// ': Second argument must be a string'
+         call error(999)
+         return
+      endif
+      top=top-1
+C     first argument is a string vector 
+      if (.not.getwsmat(fname,topk,top,m1,n1,il1,ild1)) return
+      if (m1.ne.1.and.n1.ne.1) then 
+         buf= fname// ': First argument must be a vector'
+         call error(999)
+         return
+      endif
+      call xmsg1(istk(il1),istk(ild1),m1*n1,
+     $     istk(il2),istk(ild2),m2*n2,ierr)
+      if(ierr.eq.1) then 
+         call error(112)
+         return
+      endif
+      call objvide(fname,top)
+      end
+
       subroutine scichoose(fname) 
 C     ================================================================
 c     SCILAB function : x_choose
@@ -785,47 +863,3 @@ c     Putting in order the stack
        end
 
 
-      logical  function getwsmat(fname,topk,lw,m,n,ilr,ilrd)
-C     ================================================================
-C     SHOULD be in stack.f ou stack-more.f 
-C     ------
-C     renvoit .true. si l'argument en lw est une matrice
-C     de chaine de caractere
-C     sinon renvoit .false. et appelle error
-C     Entree :
-C       fname : nom de la routine appellante pour le message
-C       d'erreur
-C       lw : position dans la pile
-C     Sortie :
-C	[m,n] caracteristiques de la matrice
-C       ilr : pointe sur le premier caractere de a
-C            le caractere se recupere par abs(istk(ilr)) qui est
-C            un codage Scilab des chaines
-C       ilrd : pointe vers le descripteur des taille de chaque element 
-C            c'est un tableau d'entiers istk(ilrd)
-C      implicit undefined (a-z)
-C     ================================================================
-      integer topk,lw,m,n,ilr,ilrd,il
-      integer iadr,sadr
-      character fname*(*)
-      include '../stack.h'
-c     
-      iadr(l)=l+l-1
-      sadr(l)=(l/2)+1
-c
-      il=iadr(lstk(lw))
-      if(istk(il).ne.10) then
-         call cvname(ids(1,pt+1),fname,0)
-         err=rhs+(lw-topk)
-         call error(207)
-         getwsmat=.false.
-         return
-      endif
-      m=istk(il+1)
-      n=istk(il+2)
-      ilrd=il+4
-      ilr=il+5+m*n
-      getwsmat=.true.
-      return
-      end
-
