@@ -3,13 +3,15 @@ c SCILAB function : scicos
 c     Copyright INRIA
       subroutine intsscicos
       include '../stack.h'
-      integer start,run,finish,flag
-      parameter (start=28,run=27,finish=15)
+      integer start,run,finish,flag,linear
+      parameter (start=28,run=27,finish=15,linear=21)
       integer id(nsiz)
-      double precision simpar(4)
+      double precision simpar(7)
       integer pointi
       integer funnum
       external funnum
+c
+      integer solver
 
       common /dbcos/ idb
 
@@ -55,7 +57,7 @@ c       --   subvariable z0(state) --
         n1e3 = istk(il1e3+1)
         l1e3 = sadr(il1e3+4)
 c      
-c       --   subvariable iz0(state)  NO MORE USED --
+c       --   subvariable work(state) --
         il1e4=iadr(l1+istk(il1+5)-1)
         n1e4 = istk(il1e4+1)
         l1e4 = sadr(il1e4+4)
@@ -80,6 +82,8 @@ c       --   subvariable outtb(state) --
         n1e8 = istk(il1e8+1)
         nout = n1e8
         l1e8 = sadr(il1e8+4)
+
+c      
 c       checking variable tcur (number 2)
 c       
         il2 = iadr(lstk(top-rhs+2))
@@ -139,7 +143,7 @@ c       --   subvariable zptr(sim) --
         n4e4 = istk(il4e4+1)
         l4e4 = sadr(il4e4+4)
 c      
-c       --   subvariable izptr (sim) NO MORE USED --
+c       --   subvariable zcptr (sim) 
         il4e5=iadr(l4+istk(il4+6)-1)
         n4e5 = istk(il4e5+1)
         l4e5 = sadr(il4e5+4)
@@ -241,7 +245,7 @@ c       --   subvariable critev(sim) --
         n4e22 = istk(il4e22+1)
         l4e22 = sadr(il4e22+4)
 c      
-c       --   subvariable nblk(sim) --
+c       --   subvariable nb(sim) --
         il4e23=iadr(l4+istk(il4+24)-1)
         l4e23 = sadr(il4e23+4)
 c
@@ -301,6 +305,14 @@ c       --   subvariable labels(sim) --
            nlab=1
            call iset(n4e30,1,istk(labptr),1)
         endif
+
+c       --   subvariable modptr(sim) --
+        il4e31=iadr(l4+istk(il4+32)-1)
+        n4e31 = istk(il4e31+1)
+        l4e31 = sadr(il4e31+4)
+c      
+c 
+
 c
 c       checking variable flag (number 5)
         il5 = iadr(lstk(top-rhs+5))
@@ -320,6 +332,8 @@ c       checking variable flag (number 5)
            flag=2
         elseif(istk(il5+6).eq.finish) then 
            flag=3
+        elseif(istk(il5+6).eq.linear) then 
+           flag=4
         else
            err=5
            call error(44)
@@ -327,7 +341,8 @@ c       checking variable flag (number 5)
         endif
 
         if(rhs.ge.6) then
-c     checking variable simpar (number 6) [atol  rtol ttol, deltat]
+c     checking variable simpar (number 6) 
+c      [atol  rtol ttol, deltat, scale, impl, hmax]
 c     
            il6 = iadr(lstk(top-rhs+6))
            if (istk(il6) .ne. 1) then
@@ -336,7 +351,7 @@ c
               return
            endif
            m6 = istk(il6+1)*istk(il6+2)
-           if (m6 .ne. 4) then
+           if (m6 .ne. 5.and. m6.ne.4.and. m6.ne.6.and. m6.ne.7) then
               err = 6
               call error(89)
               return
@@ -354,6 +369,10 @@ c
           call error(42)
           return
         endif
+        if (n4e3 .ne. n4e5) then
+          call error(42)
+          return
+        endif
         if (n4e3 .ne. n4e6) then
           call error(42)
           return
@@ -411,8 +430,7 @@ c       cross equal output variable checking
 c       not implemented yet
         call entier(n4e3,stk(l4e3),istk(iadr(l4e3)))
         call entier(n4e4,stk(l4e4),istk(iadr(l4e4)))
-c        call entier(n1e4,stk(l1e4),istk(iadr(l1e4)))
-c        call entier(n4e5,stk(l4e5),istk(iadr(l4e5)))
+        call entier(n4e5,stk(l4e5),istk(iadr(l4e5)))
         call entier(n1e6,stk(l1e6),istk(iadr(l1e6)))
         call entier(n4e6,stk(l4e6),istk(iadr(l4e6)))
         call entier(n4e7,stk(l4e7),istk(iadr(l4e7)))
@@ -424,7 +442,7 @@ c        call entier(n4e5,stk(l4e5),istk(iadr(l4e5)))
         call entier(n4e14,stk(l4e14),istk(iadr(l4e14)))
         call entier(n4e15,stk(l4e15),istk(iadr(l4e15)))
         call entier(n4e16,stk(l4e16),istk(iadr(l4e16)))
-c        call entier(n4e17*m4e17,stk(l4e17),istk(iadr(l4e17)))
+        call entier(n4e31,stk(l4e31),istk(iadr(l4e31)))
         call entier(n4e18*m4e18,stk(l4e18),istk(iadr(l4e18)))
         call entier(n4e19*m4e19,stk(l4e19),istk(iadr(l4e19)))
         call entier(n4e20*m4e20,stk(l4e20),istk(iadr(l4e20)))
@@ -440,58 +458,35 @@ c        call entier(n4e17*m4e17,stk(l4e17),istk(iadr(l4e17)))
         ilfuntyp=iadr(l4e28)
         call entier(n4e29*m4e29,stk(l4e29),istk(iadr(l4e29)))
 c
-        if(rhs.eq.5) then
-           simpar(1) = 1.d-4
-           simpar(2) = 1.d-6
-           simpar(3) = 1.d-10
-           simpar(4) = stk(l3)+simpar(3)*2.0d0
-        else
+        if(m6.eq.4) then
            call dcopy(4,stk(l6),1,simpar,1)
+           simpar(5)=  0.d0
+           simpar(6)=  0.d0
+           simpar(7)=  0.d0
+        elseif(m6.eq.5) then
+           call dcopy(5,stk(l6),1,simpar,1)
+           simpar(6)=  0.d0
+           simpar(7)=  0.d0
+        elseif(m6.eq.6) then
+           call dcopy(6,stk(l6),1,simpar,1)
+           simpar(7)=  0.d0
+        else
+           call dcopy(7,stk(l6),1,simpar,1)
         endif
+        solver=simpar(6)
 c
         lfunpt=iadr(lw)
         lw=sadr(lfunpt+nblk)
 c
-        ncst = n1e2
-        ndst = n1e3
         linpptr=iadr(l4e6)
-        linplnk=iadr(l4e8)
         llnkptr=iadr(l4e10)
         iloutptr=iadr(l4e7)
 c
-c        niz = n1e4
-c        liz0=iadr(l1e4)
 
-        ng=0
-c        do 02 kfun = ncblk+ndblk+1,nblk
-        do 02 kfun = 1,nblk
-           if(istk(ilztyp+kfun-1).eq.1) then
-c     .    loop on block input ports
-              do 01 kport=istk(linpptr-1+kfun),istk(linpptr+kfun)-1
-c     .       get corresponding link pointer 
-                 klink=istk(linplnk-1+kport)
-                 ng=ng+istk(llnkptr+klink)-istk(llnkptr-1+klink)
- 01           continue
-           endif
- 02     continue
-c     
-c     maximum block state and input sizes      
-c        ilst=iadr(l4e3)
+c
+c        ng=istk(iadr(l4e5)+n4e5-1)-1
+
         ilinp=iadr(l4e4)
-c        ilout=iadr(l4e5)
-c        ilclk=iadr(l4e11)
-        
-
-        nn42=nout+22+ncst*max(16,ncst + 9)+3*ng 
-        lw42=lw
-        lw=lw+nn42
-        nn43= 20 + ncst + 2*ng
-        lw43=lw
-        lw=lw+nn43
-        nn44=2*nblk
-        lw44=lw
-        lw=lw+sadr(nn44)+1
-
         err=lw-lstk(bot)
         if (err .gt. 0) then
           call error(17)
@@ -540,7 +535,9 @@ C     Block is defined by a predefined scilab function
 c
         if(ddt.ne.0) idb=1
 c
-        call scicos(stk(l1e2),istk(iadr(l4e3)),stk(l1e3),istk(ilinp),
+        call scicos(stk(l1e2),istk(iadr(l4e3)),stk(l1e3),
+     $       stk(l1e4),istk(ilinp),
+     $       istk(iadr(l4e31)),
      &       istk(llab),istk(labptr),stk(l2),stk(l3),stk(l1e5),
      $       istk(iadr(l1e6)),n1e5,pointi,stk(l1e8),nout,
      $       istk(lfunpt),istk(iadr(l4e28)),istk(linpptr),
@@ -551,9 +548,8 @@ c
      $       istk(iadr(l4e18)),istk(iadr(l4e19)),n4e19,
      $       istk(iadr(l4e29)),n4e29,
      $       istk(iadr(l4e20)),n4e20,istk(iadr(l4e21)),n4e21,
-     $       istk(iadr(l4e22)),nblk,istk(iadr(l4e24)),ng,
-     $       istk(iadr(l4e27)),n4e27,simpar,stk(lw42),stk(lw43),
-     $       stk(lw44),flag,ierr)
+     $       istk(iadr(l4e22)),nblk,istk(iadr(l4e24)),istk(iadr(l4e5)),
+     $       istk(iadr(l4e27)),n4e27,simpar,flag,ierr)
         idb=0
         if (ierr .gt. 0 ) then
            if(ierr.eq.1) then
@@ -583,11 +579,21 @@ c
            elseif(ierr.eq.4) then
               buf='algrebraic loop detected'
               kfun=0
+           elseif(ierr.eq.5) then
+              buf='cannot allocate memory'
+              kfun=0
+           elseif(ierr.eq.21) then
+              write(buf,'(''cannot allocate memory in block='',i5)') 
+     $             kfun
+              kfun=0
+           elseif(ierr.eq.22) then
+              buf='sliding mode condition, cannot integrate'
+              kfun=0
            else
               buf='scicos unexpected error,please report...'
               kfun=0
            endif
-           call error(888)
+           if(.not.(err.gt.0.or.err1.gt.0)) call error(888)
            fun=0
            return
         endif
@@ -691,7 +697,7 @@ c
       logical putlhsvar,checkrhs,checklhs
       character*(nlgh) fname
 c
-      goto (1,2,3,4,5) fin 
+      goto (1,2,3,4,5,6,7,8,9) fin 
       return
  1    call intsscicos
       return
@@ -718,6 +724,23 @@ c     [r1,r2,ok]=tree4(vec,outoin,outoinptr,dep_ut,typ_r)
       if(.not.checkrhs(fname,5,5)) return
       call intree4(fname)
       goto 9988
+ 6    continue
+      fname='realtimeinit'
+      call intsrealtimeinit(fname)
+      goto 9988
+ 7    continue
+      fname='realtime'
+      call intsrealtime(fname) 
+      goto 9988
+ 8    continue
+c     scicos_debug(i)
+      fname='scicos_debug' 
+      call scicosdebug(fname)
+      goto 9988
+ 9    continue
+      fname='scicos_debug_count'
+      call scicosdebugcount(fname)
+      goto 9988
  9988 if(.not.putlhsvar())return
       end
 
@@ -730,6 +753,7 @@ c
       logical createvar,getrhsvar
       character*(*) fname
 c
+      iadr(l)=l+l-1
       if(.not.getrhsvar(1,'i',nvec,mvec,ipvec)) return
       if(.not.getrhsvar(2,'i',noin,moin,ipoin)) return
       if(.not.getrhsvar(3,'i',noinr,moinr,ipoinr)) return
@@ -743,7 +767,8 @@ c
      $     nord,istk(ipok))
       lhsvar(1)=5
       lhsvar(2)=6
-      nbrows(5)=nord
+c      nbrows(5)=nord
+      istk(iadr(iwhere(5))+1)=nord
       end
 
 
@@ -755,6 +780,7 @@ c
       logical createvar,getrhsvar
       character*(*) fname
 c
+      iadr(l)=l+l-1
       if(.not.getrhsvar(1,'i',nvec,mvec,ipvec)) return
       if(.not.getrhsvar(2,'i',ndep,mdep,ipdep)) return
       if(.not.getrhsvar(3,'i',ntyp,mtyp,iptyp)) return
@@ -774,7 +800,8 @@ c
      $     istk(ipok))
       lhsvar(1)=8
       lhsvar(2)=9
-      nbrows(8)=nord
+c      nbrows(8)=nord
+      istk(iadr(iwhere(8))+1)=nord
       end
 
 
@@ -786,6 +813,7 @@ c
       logical createvar,getrhsvar
       character*(*) fname
 c
+      iadr(l)=l+l-1
       if(.not.getrhsvar(1,'i',nvec,mvec,ipvec)) return
       if(.not.getrhsvar(2,'i',noin,moin,ipoin)) return
       if(.not.getrhsvar(3,'i',noinr,moinr,ipoinr)) return
@@ -801,8 +829,52 @@ c
      $     istk(ipr2),nr)
       lhsvar(1)=6
       lhsvar(2)=7
-      nbcols(6)=nr
-      nbcols(7)=nr
+c      nbcols(6)=nr
+      istk(iadr(iwhere(6))+2)=nr
+c      nbcols(7)=nr
+      istk(iadr(iwhere(7))+2)=nr
+      end
+
+c     ********************
+      subroutine  scicosdebug(fname)
+c     
+      include '../stack.h'
+c     
+      common /cosdebug/ cosd
+      logical getrhsvar, createvar,checklhs,checkrhs
+      integer cosd
+      character*(*) fname
+c
+      rhs = max(0,rhs)
+      if(.not.checklhs(fname,1,1)) return
+      if(.not.checkrhs(fname,-1,1)) return
+      if (rhs.eq.1) then
+         if(.not.getrhsvar(1,'i',n,m,i)) return
+         cosd=istk(i)
+         lhsvar(1)=0
+      else
+         if(.not.createvar(1,'d',1,1,ipr1)) return
+         stk(ipr1)=cosd
+         lhsvar(1)=1 
+      endif
+
       end
 
 
+c     ********************
+      subroutine scicosdebugcount(fname)
+c     
+      include '../stack.h'
+c     
+      common /cosdebugcounter/ counter 
+      logical getrhsvar, createvar,checklhs,checkrhs
+      integer counter
+      character*(*) fname
+c      
+      rhs = max(0,rhs)
+      if(.not.checklhs(fname,1,1)) return
+      if(.not.checkrhs(fname,-1,0)) return
+      if(.not.createvar(1,'d',1,1,ipr1)) return
+      stk(ipr1)=counter
+      lhsvar(1)=1 
+      end
