@@ -1,3 +1,6 @@
+#ifndef __SPDEFS_H__
+#define __SPDEFS_H__
+
 /*
  *  DATA STRUCTURE AND MACRO DEFINITIONS for Sparse.
  *
@@ -25,7 +28,7 @@
  *  software for any purpose.  It is provided `as is', without express
  *  or implied warranty.
  *
- *  $Date: 88/06/18 11:13:40 $
+ *  $Date: 2004/08/27 13:55:54 $
  *  $Revision: 1.2 $
  */
 
@@ -885,3 +888,5 @@ struct  MatrixFrame
     struct FillinListNodeStruct *LastFillinListNode;
 };
 typedef  struct MatrixFrame  *MatrixPtr;
+
+#endif /* __SPDEFS_H__*/
