@@ -3,24 +3,29 @@
 #ifndef STACK_DEF 
 #define STACK_DEF 
 
+/* XXXXX pour gcc -3.1 il faut un extern devant le 
+ * __declspec(dllimport) 
+ */ 
+
 #ifdef FORDLL 
-#define IMPORT  __declspec (dllimport)
+#define IMPORT extern  __declspec (dllimport)
 #else 
 #define IMPORT extern
 #endif
 
-
 /* typedef int integer ; */
 
 #define csiz 63  
 #define bsiz 4096  
-#define isizt 1200
-#define psiz 256  
+#define isizt 10000
+#define psiz 4096  
 #define nsiz 6  
-#define lsiz 16384 
+#define lsiz 65536
 #define nlgh nsiz*4  
 #define vsiz 2 
-#define intersiz 60
+#define intersiz 1024
+#define maxdb 100
+#define maxbpt 1000
 
 IMPORT struct {
     double Stk[vsiz];
@@ -35,7 +40,7 @@ IMPORT struct {
 
 IMPORT struct {
     int ids[nsiz*psiz], pstk[psiz], rstk[psiz], pt, niv, 
-	    macr, paus, icall;
+	    macr, paus, icall, krec;
 } C2F(recu);
 
 IMPORT struct {
@@ -43,7 +48,7 @@ IMPORT struct {
 } C2F(iop);
 
 IMPORT struct {
-    int err1, err2, errct, toperr, errpt, ieee;
+    int err1, err2, errct, toperr, errpt, ieee ,errcatch;
 } C2F(errgst); 
 
 IMPORT struct {
@@ -57,9 +62,17 @@ IMPORT struct {
 
 
 IMPORT struct {
-  int nbvars, nbrows[intersiz], nbcols[intersiz], itflag[intersiz],
-    ntypes[intersiz], lad[intersiz],ladc[intersiz],lhsvar[intersiz];
+  int nbvars, iwhere[intersiz], nbrows[intersiz], nbcols[intersiz], itflag[intersiz], ntypes[intersiz], lad[intersiz],ladc[intersiz],lhsvar[intersiz];
 } C2F(intersci);
 
+
+IMPORT struct {
+  char alfa[csiz], alfb[csiz], buf[bsiz];
+} C2F(cha1);
+
+IMPORT struct {
+    int wmac, lcntr, nmacs, macnms[nsiz*maxdb]	, lgptrs[maxdb+1], bptlg[maxbpt];
+} C2F(dbg);
+
 #endif /** STACK_DEF  **/
 
