@@ -1,18 +1,16 @@
       subroutine print(id,lk,lunit)
 c     ==================================================================
-c     ==
 c     print object of id id(nsiz) stored at position lk in the stack
 c     ==================================================================
-c     ==
 c     Copyright INRIA
       include '../stack.h'
-      integer id(nsiz),lk,id1(nsiz)
+      integer id(nsiz),lk,id1(nsiz),dname(nsiz)
 c     
       common / ptkeep / lwk
       integer itype,itypel,gettype
       integer fl,mode,m,n,it,lr,lc,nlr,lkeep,topk,lname,siz,vol,tops
       integer namef(nlgh),percent,under,pchar
-      logical getmat,ilog,getpoly,typer,clsave,getsimat
+      logical getmat,ilog,getpoly,typer,clsave,getsimat,gethmat
       logical crewimat ,islss,getilist,getbmat,eptover
       character*4 name
       character*10 form
@@ -24,6 +22,8 @@ c
       data left/54/,right/55/,rparen/42/,lparen/41/,equal/50/
       data eol/99/,nclas/29/
       data percen/56/,under/36/,pchar/25/
+      save ligne
+
 c     
       iadr(l)=l+l-1
       sadr(l)=(l/2)+1
@@ -32,18 +32,21 @@ c
       lineln=lct(5)
       mode=lct(6)
       ndgt=lct(7)
+
       if(rstk(pt).eq.1101) goto 96
       lkeep0=lstk(lk)
+      call putid(dname,id)
 c     
       if (lct(1) .lt. 0) then
          lk=0
          return
       endif
 c     
-      if(id(1).ne.0) call prntid(id,-1,lunit)
+      if(id(1).ne.0) call prntid(dname,-1,lunit)
  01   nlist=0
 C     topk : free stack zone for working areas 
       topk=top+1
+      lstk(topk+1)=lstk(topk)
       li2=1
       lkeep=lstk(lk)
       itype=gettype(lk)
@@ -51,12 +54,16 @@ C     topk : free stack zone for working areas
       mactop=0
       if (abs(itype).eq.11.or.abs(itype).eq.13) mactop=1
 c     
- 05   goto (20,10,06,55,25,26,06,06,06,30,60,06,60,70,40,40,40)
+
+
+ 05   goto (20,10,75,55,25,26,75,75,75,30,60,75,60,70,40,40,40)
      $     ,abs(itype)
- 06   goto 75
+      goto 75
 c     
 c     ----polynomial matrices
- 10   ilog=getpoly("print",lk,lk,it,m,n,name,namel,ilp,lr,lc)
+ 10   continue
+      if (lunit.eq.wte.and.intexmacs().ne.0) goto 75
+      ilog=getpoly("print",lk,lk,it,m,n,name,namel,ilp,lr,lc)
 C     working area (see dmpdsp)
       iwl = istk(ilp + m*n)-istk(ilp) + m*n+1
       if (.not.crewimat("print",topk,1,iwl,lw)) return
@@ -78,7 +85,9 @@ C     working area (see dmpdsp)
       goto 48
 c     
 c     -------scalar matrices 
- 20   ilog=getmat("print",lk,lk,it,m,n,lr,lc)
+ 20   continue
+      if (lunit.eq.wte.and.intexmacs().ne.0) goto 75
+      ilog=getmat("print",lk,lk,it,m,n,lr,lc)
 C     working area 
       if (.not.crewimat("print",topk,1,m*n+2*n,lw)) return
       if(m*n.eq.0) then
@@ -118,35 +127,43 @@ c     -------sparse boolean matrices
      $     lineln,lunit,buf)
       goto 48 
 c     -------matrices of string 
- 30   ilog=getsimat("print",lk,lk,m,n,1,1,lr,nlr)
+ 30   continue
+      if (lunit.eq.wte.and.intexmacs().ne.0) goto 75
+      ilog=getsimat("print",lk,lk,m,n,1,1,lr,nlr)
 C     working area 
       if (.not.crewimat("print",topk,1,n,lw)) return
       call strdsp(istk(lr),istk(lr-m*n-1),m,n,lineln,lunit,istk(lw),buf)
       goto 48
+c     -------matrices of handle 
+ 35   ilog=gethmat("print",lk,lk,m,n,lr)
+      if (.not.crewimat("print",topk,1,m*n+2*n,lw)) return
+      call dmdsp(stk(lr),m,m,n,ndgt,mode,lineln,lunit,buf,istk(lw))
+      goto 48 
+
 c     -------lists 
  40   continue
       itype=gettype(lk)
       call listtype(lk,itypel)
-      if (itypel.eq.1) goto 50
+      if (itypel.eq.1) then
+         if (lunit.eq.wte.and.intexmacs().ne.0) goto 41
+         goto 50
+      endif
       islss=itypel.eq.2
-      if(islss.or.itype.eq.15) goto 45
+      if (islss) then
+         if (lunit.eq.wte.and.intexmacs().ne.0) goto 41
+         goto 45
+      endif
+
  41   continue
       ilw=iadr(lstk(lk))
       if(istk(ilw).lt.0) ilw=iadr(istk(ilw+1))
       nw=istk(ilw+1)
       llw=sadr(ilw+3+nw)
 c     form the requested function name
-c     get tlist type
-      iltyp=iadr(llw)
-      nlt=min(nlgh-3,istk(iltyp+5)-1)
-      iltyp=iltyp+5+istk(iltyp+1)*istk(iltyp+2)
-      namef(1)=percen
-      call icopy(nlt,istk(iltyp),1,namef(2),1)
-      namef(2+nlt)=under
-      namef(3+nlt)=pchar
-      call namstr(id1,namef,nlt+3,0)
+      call funnam(id1,'p',ilw)
+
 c     look for the function 
-      fin=0
+ 42   fin=0
       tops=top
       top=topk
       llk=lstk(lk)
@@ -156,11 +173,14 @@ c     look for the function
       lstk(lk)=llk
       if(fun.gt.0) then
          top=tops
-         goto 42
+         goto 43
       endif
       if(fin.eq.0) then
          top=tops
-         goto 45
+         if(itype.eq.15) goto 45
+         call setfunnam(id1,'%l_p',4)
+         itype=15
+         goto 42
       endif
       fin=-1
       call stackg(id1)
@@ -169,7 +189,7 @@ c     look for the function
       endif
 
 c     copy tlist to top of stack
- 42   if(lk.ne.top) then
+ 43   if(lk.ne.top) then
          top=topk
          ilt1=iadr(lstk(top))
          lt1=sadr(ilt1+3+nw)
@@ -179,27 +199,28 @@ c     copy tlist to top of stack
             return
          endif
          call icopy(nw+3,istk(ilw),1,istk(ilt1),1)
-         call dcopy(istk(ilw+2+nw)-1,stk(llw),1,stk(lt1),1)
+         call unsfdcopy(istk(ilw+2+nw)-1,stk(llw),1,stk(lt1),1)
          lstk(top+1)=lt1+istk(ilw+2+nw)-1
       endif
       goto 95
 c     
 c     check for typed lists 
- 45   ilog=getilist("print",lk,lk,nl,1,ilt)
+ 45   continue
+      ilog=getilist("print",lk,lk,nl,1,ilt)
       illist=lstk(lk)
 C     list ( we must deal with recursion ) 
       nlist=nlist+1
       if(nlist.le.1) then
-         if(id(1).ne.0) then
-            call cvnamel(id,ligne,1,li1)
+         if(dname(1).ne.0) then
+            call cvnamel(dname,ligne,1,li1)
          else
             li1=li2
+            ligne(li1:li1)=' '
          endif
       else
          li1=li2
       endif
       ligne(li1+1:li1+1)='('  
-      if(islss) ligne(li1+1:li1+1)='('
       li1=li1+2
       kl=0
  47   continue
@@ -298,6 +319,7 @@ c     end for list
 c     
 c     -----fractions rationnelles <=> list('r',matpoly,matpoly)
  50   continue
+      if (lunit.eq.wte.and.intexmacs().ne.0) goto 75
 c     Numerateur ( 2ieme elt de la liste )
       ilog=getilist("print",lk,lk,nel,2,iln)
 c     ---local change of lstk(topk)
@@ -335,7 +357,7 @@ c     denominateur (3ieme elt )
 
 C     --wrong type argument or not same size 
       if (typer.or.m1.ne.m.or.n1.ne.n) then 
-         call cvname(id,buf(1:nlgh),1)
+         call cvname(dname,buf(1:nlgh),1)
          call error(103)
          return 
       endif 
@@ -366,7 +388,9 @@ C     of requested size iws
       goto 48
 c     
 c     -----------boolean matrix
- 55   ilog= getbmat("print",lk,lk,m,n,lr)
+ 55   continue
+      if (lunit.eq.wte.and.intexmacs().ne.0) goto 75
+      ilog= getbmat("print",lk,lk,m,n,lr)
       if(m*n.eq.0) then
          call basout(io,lunit,'     []')
       else
@@ -421,22 +445,6 @@ c     ------------macros---- a changer
       il=il+1
 C     cas ou l'object macro est au top ( pas ds une liste )
       if (mactop.eq.1) then 
-c$$$  c        compiled macro [ display of source code ]
-c$$$  if(istk(ilm).eq.13) then 
-c$$$  call cvnamel(idstk(1,lk),mmname,1,lname)
-c$$$  call xscion(iflag)
-c$$$  if(iflag.eq.0) then
-c$$$  buf='$SCI/bin/scilab -macro "'//mmname(1:lname)//'"  '
-c$$$  call bashos(buf,27+lname,ls,ierr)
-c$$$  else
-c$$$  buf='$SCI/bin/scilab -macro "'//mmname(1:lname)//
-c$$$  $              '" | $SCI/bin/xless &   '
-c$$$  call bashos(buf,45+lname,ls,ierr)
-c$$$  endif
-c$$$  goto 68 
-c$$$  endif
-C     we supress the display of the code of a non compiled macro with
-c     goto 68
          isncf=1
          if (isncf.eq.1) goto 68
          l=il
@@ -516,7 +524,7 @@ c     copy variable  to top of stack
  76   if(lk.ne.top) then
          top=topk
          vol=lstk(lk+1)-lstk(lk)
-         call dcopy(vol,stk(lstk(lk)),1,stk(lstk(top)),1)
+         call unsfdcopy(vol,stk(lstk(lk)),1,stk(lstk(top)),1)
          lstk(top+1)=lstk(top)+vol
       endif
       goto 95
@@ -525,14 +533,17 @@ c     -----------end
 
 c     overloaded print
  95   continue
-
 c     preserve data for recursion
 
-      if ( eptover(2,psiz)) return
+      if ( eptover(3,psiz)) return
+      rstk(pt-2)= 0
+      call putid(ids(1,pt-2),dname)
       rstk(pt-1)= 0
       ids(1,pt-1)= lhs
       ids(2,pt-1)= rhs
       ids(3,pt-1)= lstk(lk)
+      ids(4,pt-1)= li1
+      ids(5,pt-1)= lkeep0
       lstk(lk)   = lkeep
       pstk(pt)   = lk
       ids(1,pt)  = nlist
@@ -556,19 +567,21 @@ c     *call* matfn
 
  96   continue
 c     set back preserved data
+      call putid(dname,ids(1,pt-2))
       lhs     = ids(1,pt-1)
       rhs     = ids(2,pt-1)
       lk      = pstk(pt)
-      lkeep  = lstk(lk)
+      lkeep   = lstk(lk)
       lstk(lk)= ids(3,pt-1)
+      li1     = ids(4,pt-1)
+      lkeep0  = ids(5,pt-1)
       nlist   = ids(1,pt)
       kl      = ids(2,pt)
       nl      = ids(3,pt)
       illist  = ids(4,pt)
       topk    = ids(5,pt)
       top     = ids(6,pt)
-      pt=pt-2
-c      if(lk.ne.top) top=top-1
+      pt=pt-3
       goto 48
 
 
