@@ -18,14 +18,14 @@ c
       double precision x,xx
       integer op,ix(2),fptr
       equivalence (x,ix(1))
-      logical iflag
-      common /basbrk/ iflag
 c
 
       integer id(nsiz),iadr,sadr,cmplxt,pt0
 c
       integer for(3),while(5),iff(2),sel(6)
       integer ops(nops)
+      external getendian
+      integer getendian
 
       data for/15,24,27/
       data iff/18,15/
@@ -144,8 +144,10 @@ c     nouvelle 'operation'
       op=istk(lc)
 c     
       if(ddt.lt.-1) write(6,'(i7)') op
-      goto(20,25,40,42,30,41,45,50,50,60,15,90,90,90,90,100,
-     &     12,101,102,90,103,104,105) ,op
+      goto(20,25,40,42,30,41,45,50,50,60,
+     &     15,90,90,90,90,100,12,101,102,90,
+     &     103,104,105,106,107,108,110,90,115,116,
+     &     117) ,op
 c     
 c     
 c     matfns
@@ -331,9 +333,16 @@ c     num
 c     
  41   continue
       il=iadr(lr)
-      ix(1)=istk(lc+1)
-      ix(2)=istk(lc+2)
-      maxc=17
+      if(getendian().eq.1) then
+         ix(1)=istk(lc+1)
+         ix(2)=istk(lc+2)
+      else
+         ix(2)=istk(lc+1)
+         ix(1)=istk(lc+2)
+      endif
+
+
+      maxc=18
       isign=1
       if(x.lt.0)  isign=-1
       call fmt(abs(x),maxc,ifmt,n1,n2)
@@ -354,10 +363,14 @@ c     Nan
             if(ie.lt.0) ie=ie-1
             xx=x/(10.0d0**ie)
             nf=2
-            write(form,120) maxc,0
+            write(form,120) maxc,15
             write(strg,form) xx
             ls=lnblnk(strg)
-            write(strg(ls+1:),'(''D'',i4)') ie
+            if(abs(x).lt.1.d-99) then
+               write(strg(ls+1:),'(''D'',i4)') ie
+            else
+               write(strg(ls+1:),'(''D'',i3)') ie
+            endif
          else
             write(form,130) maxc,maxc-7
             write(strg,form) x
@@ -1045,7 +1058,7 @@ c     mkindx
       istk(il+3)=0
       istk(il+4)=1
       l=il+8
-c     type 2
+c     type 19
       istk(l)=1
       istk(l+1)=9
       istk(il+5)=3
@@ -1104,6 +1117,209 @@ c
 c     
       lc=lc+1+nsiz
       goto 10
+c
+106   continue
+      il=iadr(lr)
+      istk(il)=10
+      istk(il+1)=1
+      istk(il+2)=1
+      istk(il+3)=0
+      istk(il+4)=1
+      l=il+6
+c     type 24
+      istk(l)=2
+      istk(l+1)=4
+      istk(il+5)=3
+      l=l+2
+c
+      l=sadr(l)
+      istk(ilr)=istk(ilr-1)+l-lr
+      lr=l
+      ilr=ilr+1
+c
+      lc=lc+1
+      goto 10
+c
+107   continue
+c     profile
+      il=iadr(lr)
+      istk(il)=10
+      istk(il+1)=1
+      istk(il+2)=3
+      istk(il+3)=0
+      istk(il+4)=1
+      l=il+8
+c     type 25
+      istk(l)=2
+      istk(l+1)=5
+      istk(il+5)=3
+      l=l+2
+c
+c     n
+      call intstr(istk(lc+1),istk(l),ni,1)
+      istk(il+6)=istk(il+5)+ni
+      l=l+ni
+c     m
+      call intstr(istk(lc+2),istk(l),ni,1)
+      istk(il+7)=istk(il+6)+ni
+      l=l+ni
+
+      l=sadr(l)
+      istk(ilr)=istk(ilr-1)+l-lr
+      lr=l
+      ilr=ilr+1
+c
+      lc=lc+3
+      goto 10
+c
+ 108  continue
+c     vector of string
+      n=istk(lc+1)*istk(lc+2)
+      il=iadr(lr)
+      istk(il)=10
+      istk(il+1)=1
+      istk(il+2)=3+n
+      istk(il+3)=0
+      istk(il+4)=1
+      l=il+5+(3+n)
+c     type 26
+      istk(l)=2
+      istk(l+1)=6
+      istk(il+5)=3
+      l=l+2
+c
+c     n
+      call intstr(istk(lc+1),istk(l),ni,1)
+      istk(il+6)=istk(il+5)+ni
+      l=l+ni
+c     m
+      call intstr(istk(lc+2),istk(l),ni,1)
+      istk(il+7)=istk(il+6)+ni
+      l=l+ni
+
+      nc=istk(lc+4+n)-1
+      do 109 ii=1,n
+         istk(il+7+ii)=istk(il+6+ii)+istk(lc+4+ii)-istk(lc+3+ii)
+ 109  continue
+      call icopy(nc,istk(lc+5+n),1,istk(l),1)
+      l=l+nc
+
+      l=sadr(l)
+      istk(ilr)=istk(ilr-1)+l-lr
+      lr=l
+      ilr=ilr+1
+c
+      lc=lc+5+n+nc
+      goto 10
+
+ 110  continue
+c     varfunptr
+      il=iadr(lr)
+      istk(il)=10
+      istk(il+1)=1
+      istk(il+2)=4
+      istk(il+3)=0
+      istk(il+4)=1
+      l=il+9
+c     type 27
+      istk(l)=2
+      istk(l+1)=7
+      istk(il+5)=3
+      l=l+2
+
+c     fun
+      call intstr(istk(lc+1),istk(l),ni,1)
+      istk(il+6)=istk(il+5)+ni
+      l=l+ni
+c     fin
+      call intstr(istk(lc+2),istk(l),ni,1)
+      istk(il+7)=istk(il+6)+ni
+      l=l+ni
+c
+c     nom de la variable
+      call namstr(istk(lc+3),istk(l),ni,1)
+      istk(il+8)=istk(il+7)+ni
+      l=l+ni
+
+      l=sadr(l)
+      istk(ilr)=istk(ilr-1)+l-lr
+      lr=l
+      ilr=ilr+1
+c
+      lc=lc+3+nsiz
+      goto 10
+c
+ 115  continue
+c     affectation
+      nlhs=istk(lc+1)
+c      print *, 'affectation'
+      il=iadr(lr)
+      istk(il)=10
+      istk(il+1)=1
+      istk(il+2)=2+2*nlhs
+      istk(il+3)=0
+      istk(il+4)=1
+
+      l=il+5+istk(il+2)
+c     type 
+      call intstr(istk(lc),istk(l),ni,1)
+      istk(il+5)=istk(il+4)+ni
+      l=l+ni
+c     sym
+      call intstr(istk(lc+2),istk(l),ni,1)
+      istk(il+6)=istk(il+5)+ni
+      l=l+ni
+c
+      li=lc+3
+      do i=0,nlhs-1
+c     .  namei
+         call namstr(istk(li),istk(l),ni,1)
+         istk(il+7+2*i)=istk(il+6+2*i)+ni
+         l=l+ni
+c     .  rhsi
+         call intstr(istk(li+nsiz),istk(l),ni,1)
+         istk(il+8+2*i)=istk(il+7+2*i)+ni
+         l=l+ni
+
+         li=li+nsiz+1
+      enddo
+
+      l=sadr(l)
+      istk(ilr)=istk(ilr-1)+l-lr
+      lr=l
+      ilr=ilr+1
+
+      lc=li
+      goto 10
+ 116  continue
+c     skip logical evaluation
+      lc=lc+3
+      goto 10
+ 117  continue
+c     comment
+      n=istk(lc+1)
+      il=iadr(lr)
+      istk(il)=10
+      istk(il+1)=1
+      istk(il+2)=2
+      istk(il+3)=0
+      istk(il+4)=1
+      istk(il+5)=istk(il+4)+2
+      istk(il+6)=istk(il+5)+n
+      l=il+7
+c     type 31
+      istk(l)=3
+      istk(l+1)=1
+      l=l+2
+      call icopy(n,istk(lc+2),1,istk(l),1)
+      l=l+n
+      l=sadr(l)
+      istk(ilr)=istk(ilr-1)+l-lr
+      lr=l
+      ilr=ilr+1
+c
+      lc=lc+2+n
+      goto 10
 c
   120 format('(f',i2,'.',i2,')')
   130 format('(1pd',i2,'.',i2,')')
