@@ -27,6 +27,9 @@
  */
 
 /* charproc.c */
+#if defined  sgi && ! defined  __STDC__
+#define __STDC__ 1  
+#endif
 
 #include "../machine.h"
 #include "x_ptyxP.h"
@@ -56,10 +59,15 @@
 #endif
 
 #include <string.h> /* in case of dmalloc */ 
-#include <malloc.h>  /* in case of dmalloc */ 
+#ifdef __STDC__
+#include <stdlib.h>
+#else
+#include <malloc.h>
+#endif
+
 
 #ifdef WITH_TK
-#include "../tksci/tksci.h"
+#include "../tclsci/TCL_Global.h"
 #endif
 
 
@@ -67,8 +75,10 @@
 #include "All-extern.h"
 
 #define CTRL_B                0x0002  /* back a character */
+#define CTRL_C                0x0003  /* redo line */
 #define CTRL_D                0x0004  /* delete next char */
 #define CTRL_K                0x000b  /* delete to end of line */
+#define CTRL_Z                0x0020  /* stop */
 
 
 /** JPC **/
@@ -81,9 +91,11 @@ typedef int XtGravity;
 static Char buffer[BUF_SIZE];
 static Char *bptr = buffer;
 static int bcnt = 0;
-
+extern int getdiary();
+void C2F(diary) __PARAMS((char *str,int *n));
+void diary_nnl __PARAMS((char *str,int *n));
 typedef  int (*osc_func) ();
-
+extern int C2F(ismenu)();
 /*
  * Check for both EAGAIN and EWOULDBLOCK, because some supposedly POSIX
  * systems are broken and return EWOULDBLOCK when they should return EAGAIN.
@@ -197,26 +209,26 @@ extern int scstable[];
 
 
 
-static void dotext  _PARAMS((register TScreen *screen, unsigned int flags, char charset, char *buf, char *ptr));  
-static void WriteText  _PARAMS((register TScreen *screen, register char *str, register int len, unsigned int flags));  
-static void bitset  _PARAMS((unsigned int *p, int mask));  
-static void bitclr  _PARAMS((unsigned int *p, int mask));  
-static void SwitchBufs  _PARAMS((register TScreen *screen));  
-static void VTExpose  _PARAMS((Widget w, XEvent *event, Region region));  
-static void VTGraphicsOrNoExpose  _PARAMS((XEvent *event));  
-static void VTNonMaskableEvent  _PARAMS((Widget w, XtPointer closure, XEvent *event, Boolean *cont));  
-static void VTResize  _PARAMS((Widget w));  
-static void VTallocbuf  _PARAMS((void));  
-static void VTClassInit  _PARAMS((void));  
-static void VTInitialize  _PARAMS((Widget wrequest, Widget wnew, ArgList args, Cardinal *num_args));  
-static void VTDestroy  _PARAMS((Widget w));  
-static void VTRealize  _PARAMS((Widget w, XtValueMask *valuemask, XSetWindowAttributes *values));  
-static Boolean VTSetValues  _PARAMS((Widget cur, Widget request, Widget new, ArgList args, Cardinal *num_args));  
-static void HandleKeymapChange  _PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
-static void HandleBell  _PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
-static void HandleVisualBell  _PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
-static void HandleIgnore  _PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
-static void DoSetSelectedFont  _PARAMS((Widget w, XtPointer client_data, Atom *selection, Atom *type, XtPointer value, long unsigned int *length, int *format));  
+static void dotext  __PARAMS((register TScreen *screen, unsigned int flags, char charset, char *buf, char *ptr));  
+static void WriteText  __PARAMS((register TScreen *screen, register char *str, register int len, unsigned int flags));  
+static void bitset  __PARAMS((unsigned int *p, int mask));  
+static void bitclr  __PARAMS((unsigned int *p, int mask));  
+static void SwitchBufs  __PARAMS((register TScreen *screen));  
+static void VTExpose  __PARAMS((Widget w, XEvent *event, Region region));  
+static void VTGraphicsOrNoExpose  __PARAMS((XEvent *event));  
+static void VTNonMaskableEvent  __PARAMS((Widget w, XtPointer closure, XEvent *event, Boolean *cont));  
+static void VTResize  __PARAMS((Widget w));  
+static void VTallocbuf  __PARAMS((void));  
+static void VTClassInit  __PARAMS((void));  
+static void VTInitialize  __PARAMS((Widget wrequest, Widget wnew, ArgList args, Cardinal *num_args));  
+static void VTDestroy  __PARAMS((Widget w));  
+static void VTRealize  __PARAMS((Widget w, XtValueMask *valuemask, XSetWindowAttributes *values));  
+static Boolean VTSetValues  __PARAMS((Widget cur, Widget request, Widget new, ArgList args, Cardinal *num_args));  
+static void HandleKeymapChange  __PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
+static void HandleBell  __PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
+static void HandleVisualBell  __PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
+static void HandleIgnore  __PARAMS((Widget w, XEvent *event, String *params, Cardinal *param_count));  
+static void DoSetSelectedFont  __PARAMS((Widget w, XtPointer client_data, Atom *selection, Atom *type, XtPointer value, long unsigned int *length, int *format));  
 
 
 /*
@@ -495,11 +507,9 @@ WidgetClass xtermWidgetClass = (WidgetClass) & xtermClassRec;
 
 /* I/O Function for scilab : this function are used when Xscilab is on */
 
-int in_put();
-
-int XEvorgetchar()
+int XEvorgetchar(int interrupt)
 {
-  return (in_put());
+  return (in_put(interrupt));
 }
 
 void Xputstring(str,n)
@@ -510,9 +520,7 @@ void Xputstring(str,n)
   for ( i =0 ; i < n; i++) Xputchar(str[i]);
 }
 
-void C2F(xscisncr)(str,n,dummy)
-     char *str;
-     integer *n,dummy;
+void C2F(xscisncr)(char *str,integer *n,integer dummy)
 {
   int i;
   for ( i =0 ; i < *n; i++) {
@@ -522,9 +530,7 @@ void C2F(xscisncr)(str,n,dummy)
 
 /** print str on Scilab window \n are changed to \r\n **/
 
-void C2F(xscisrn)(str,n,dummy)
-     char *str;
-     integer *n,dummy;
+void C2F(xscisrn)(char *str,integer *n,integer dummy)
 {
   int i;
   for ( i =0 ; i < *n; i++) {
@@ -536,10 +542,7 @@ void C2F(xscisrn)(str,n,dummy)
   }
 }
 
-void C2F(xscistring)(str,n,dummy)
-     char *str;
-     int *n;
-     long int dummy;
+void C2F(xscistring)(char *str,integer *n,long int dummy)
 {
   int i ;
   for ( i =0 ; i < *n; i++) {
@@ -557,7 +560,7 @@ void C2F(xscimore)(n)
   *n=0;
   ln=strlen(MORESTR);
   Xputstring(MORESTR,ln);
-  n1=XEvorgetchar();
+  n1=XEvorgetchar(1);
   if ( n1 == 110 )  *n=1;
   Xputstring("\r\n",2);
 }
@@ -601,11 +604,10 @@ void Scistring(str)
 #include <varargs.h>
 #endif 
 
-/* 
-  Fonction de type print(format,args,....) 
-  qui imprime dans la fenetre Scilab
-*/
-
+/*---------------------------------------------------
+ * functions similar to print(format,arg1,...,argn) 
+ * but with output redirected to scilab window 
+ *-------------------------------------------------*/
 
 #ifdef __STDC__ 
 void  sciprint(char *fmt,...) 
@@ -625,16 +627,57 @@ void sciprint(va_alist) va_dcl
   va_start(ap);
   fmt = va_arg(ap, char *);
 #endif
+  (void ) vsprintf(s_buf, fmt, ap );
+  lstr=strlen(s_buf);
+
   C2F(xscion)(&i);
   if (i == 0) 
     {
-      (void)  vfprintf(stdout, fmt, ap );
+      printf("%s",s_buf); 
     }
   else 
     {
-      (void ) vsprintf(s_buf, fmt, ap );
-      lstr=strlen(s_buf);
-      C2F(xscisncr)(s_buf,&lstr,0L);
+      C2F(xscisrn)(s_buf,&lstr,0L);
+      /* C2F(xscisncr)(s_buf,&lstr,0L); */
+    }
+  if (getdiary()) diary_nnl(s_buf,&lstr);
+  va_end(ap);
+}
+
+/* almost the same but no diary record */ 
+/* usefull for automatic tests */ 
+
+#ifdef __STDC__ 
+void  sciprint_nd(char *fmt,...) 
+#else 
+/*VARARGS0*/
+void sciprint_nd(va_alist) va_dcl
+#endif 
+{
+  int i;
+  integer lstr;
+  va_list ap;
+  char s_buf[1024];
+#ifdef __STDC__
+  va_start(ap,fmt);
+#else
+  char *fmt;
+  va_start(ap);
+  fmt = va_arg(ap, char *);
+#endif
+
+  (void ) vsprintf(s_buf, fmt, ap );
+  lstr=strlen(s_buf);
+
+  C2F(xscion)(&i);
+  if (i == 0) 
+    {
+      printf("%s",s_buf); 
+    }
+  else 
+    {
+      C2F(xscisrn)(s_buf,&lstr,0L);
+      /* C2F(xscisncr)(s_buf,&lstr,0L); */
     }
   va_end(ap);
 }
@@ -664,6 +707,7 @@ int sciprint2(va_alist) va_dcl
   iv = va_arg(ap,int);
   fmt = va_arg(ap, char *);
 #endif
+
   C2F(xscion)(&i);
   if (i == 0) 
     {
@@ -675,6 +719,12 @@ int sciprint2(va_alist) va_dcl
       lstr=strlen(s_buf);
       C2F(xscisncr)(s_buf,&lstr,0L);
     }
+  if (getdiary()) {
+    retval= vsprintf(s_buf, fmt, ap );
+    lstr=strlen(s_buf);
+    diary_nnl(s_buf,&lstr);
+  }
+
   va_end(ap);
   return retval;
 }
@@ -1181,7 +1231,7 @@ void Xputchar(c)
 
   case CASE_OSC:
     /* Operating System Command: ESC ] */
-    do_osc((osc_func) in_put());
+    do_osc((osc_func) in_put(0));
     parsestate = groundtable;
     break;
 
@@ -1269,7 +1319,27 @@ static int pty_read_bytes;
   
 ************************************************************************/
    
-extern int ctrl_action();
+static int ctrl_action(int i)
+{
+  int j = SIGINT;
+  if(iscntrl(i) ) {
+    /* stroke is line editing command */
+    switch(i) {
+    case CTRL_C:
+	/* fprintf(stderr,"zzledt1 : CTRL_C\n"); */
+	C2F(sigbas)(&j);
+	return(1);
+	break;
+    case CTRL_Z:
+	/* doesn't work on sun4 */
+	fprintf(stderr,"zzledt1 : CTRL_Z\n");
+	return(1);
+	break;
+      };
+  };
+  return(0);
+}
+
 
 
 void xevents1()
@@ -1291,6 +1361,11 @@ void xevents1()
 	  cok = bcnt;
 	}
     }
+  
+#ifdef WITH_TK
+  flushTKEvents();
+#endif
+  
 }
 
 
@@ -1299,7 +1374,7 @@ void xevents1()
  * returns a char and deal with events 
  ****************************************/
 
-int in_put()
+int in_put(int interrupt)
 {
   register TScreen *screen = &term->screen;
   register int i;
@@ -1329,6 +1404,9 @@ int in_put()
       cok = 0;
       break;
     }
+    /* escape if we accept input interruption and an event had been detected */
+    if (interrupt&&(C2F(ismenu)()==1)) return(-1);
+
     pty_read_bytes = 0;
     /* update the screen */
     if (screen->scroll_amt)
@@ -1981,7 +2059,7 @@ void unparseputc(c, fd)
   int fd;
 {
   char buf[2];
-  register i = 1;
+  register int i = 1;
   extern XtermWidget term;
 
   if ((buf[0] = c) == '\r' && (term->flags & LINEFEED))
@@ -2077,9 +2155,9 @@ void SwitchBufPtrs(screen)
 
 #define PROMPT "==>"
 
-void VTRun(nostartup)
-     int nostartup;
+void VTRun(char *startup, int lstartup,int memory)
 {
+  static int ini=-1, ierr =0 ;
   register TScreen *screen = &term->screen;
   if (!screen->Vshow) set_vt_visibility(TRUE);
   if (screen->allbuf == NULL)  VTallocbuf();
@@ -2089,10 +2167,11 @@ void VTRun(nostartup)
 
   bcnt = 0;
   bptr = buffer;
-  {
-    /** An interaction Loop */
-    F2C(scilab)(&nostartup);
-  }
+
+  C2F(inisci)(&ini, &memory, &ierr);
+  if (ierr > 0) sci_exit(1) ;
+  /** An interaction Loop for scilab interpreter */
+  C2F(scirun)(startup,strlen(startup));
   HideCursor();
   screen->cursor_set = OFF;
 }
@@ -2177,7 +2256,7 @@ void VTInit1(parent)
   XtOverrideTranslations(parent, XtParseTranslationTable(xterm_trans));
   (void) XSetWMProtocols(XtDisplay(parent), XtWindow(parent),
 			 &wm_delete_window, 1);
-  set_scilab_icon();
+  /*set_scilab_icon(); Old icon... no more used V.C. 04/02/2004*/
   size_hints.min_width = 400;
   size_hints.min_height = 300;
   size_hints.flags =  PMinSize;
@@ -2347,7 +2426,7 @@ static void VTRealize(w, valuemask, values)
   {
     fprintf(stderr, "%s:  unable to locate a suitable font\n",
 	    xterm_name);
-    ClearExit(1);
+   sci_clear_and_exit(1);
   }
   /* making cursor */
   if (!screen->pointer_cursor)
@@ -3101,7 +3180,7 @@ void HandleSetFont(w, event, params, param_count)
       Bell();
       return;
     }
-    if (*param_count > maxparams)
+    if (*param_count > (Cardinal)maxparams)
     {				/* see if extra args given */
       Bell();
       return;
